@@ -43,17 +43,45 @@
 #include "UIcss_f.h"
 
 /*----------------------------------------------------------------------
-  New: Create a new XHTML document
+  NewXHTML: Create a new XHTML document
   ----------------------------------------------------------------------*/
 #ifdef __STDC__
-void                New (Document document, View view)
+void                NewXHTML (Document document, View view)
 #else  /* __STDC__ */
-void                New (document, view)
+void                NewXHTML (document, view)
 Document            document;
 View                view;
 #endif /* __STDC__ */
 {
-  OpenNew (document, view, TRUE);
+  OpenNew (document, view, docHTML);
+}
+
+/*----------------------------------------------------------------------
+  NewMathML: Create a new MathML document
+  ----------------------------------------------------------------------*/
+#ifdef __STDC__
+void                NewMathML (Document document, View view)
+#else  /* __STDC__ */
+void                NewMathML (document, view)
+Document            document;
+View                view;
+#endif /* __STDC__ */
+{
+  OpenNew (document, view, docMath);
+}
+
+/*----------------------------------------------------------------------
+  NewSVG: Create a new XHTML document
+  ----------------------------------------------------------------------*/
+#ifdef __STDC__
+void                NewSVG (Document document, View view)
+#else  /* __STDC__ */
+void                NewSVG (document, view)
+Document            document;
+View                view;
+#endif /* __STDC__ */
+{
+  OpenNew (document, view, docSVG);
 }
 
 /*----------------------------------------------------------------------
@@ -67,7 +95,7 @@ Document           document;
 View               view;
 #endif /* __STDC__ */
 {
-  OpenNew (document, view, FALSE);
+  OpenNew (document, view, docCSS);
 }
 
 
@@ -75,11 +103,11 @@ View               view;
   InitializeNewDoc builds the initial contents of a new document
   ----------------------------------------------------------------------*/
 #ifdef __STDC__
-void                InitializeNewDoc (STRING url, ThotBool isHTML)
+void                InitializeNewDoc (STRING url, int docType)
 #else  /* __STDC__ */
-void                InitializeNewDoc (url, isHTML)
-STRING              url;
-ThotBool            isHTML;
+void                InitializeNewDoc (url, docType)
+STRING       url;
+int          docType;
 #endif /* __STDC__ */
 {
   ElementType          elType;
@@ -91,15 +119,13 @@ ThotBool            isHTML;
   STRING               documentname;
   STRING               s;
   CHAR_T               tempfile[MAX_LENGTH];
+  STRING               profile;
   int                  doc;
 
   pathname = TtaAllocString (MAX_LENGTH);
   documentname = TtaAllocString (MAX_LENGTH);
   NormalizeURL (url, 0, pathname, documentname, NULL);
-  if (isHTML)
-    doc = InitDocView (0, documentname, docHTML, 0);
-  else
-    doc = InitDocView (0, documentname, docCSS, 0);
+  doc = InitDocView (0, documentname, docType, 0, FALSE);
   TtaFreeMemory (documentname);
   TtaFreeMemory (pathname);
 
@@ -109,24 +135,37 @@ ThotBool            isHTML;
   DocumentURLs[doc] = s;
   DocumentMeta[doc] = (DocumentMetaDataElement *) TtaGetMemory (sizeof (DocumentMetaDataElement));
   DocumentMeta[doc]->form_data = NULL;
+  DocumentMeta[doc]->initial_url = NULL;
   DocumentMeta[doc]->method = CE_ABSOLUTE;
   DocumentMeta[doc]->put_default_name = FALSE;
-  /* by default the document will be saved in XHTML */
-  DocumentMeta[doc]->xmlformat = TRUE;
   DocumentSource[doc] = 0;
+  /* default parsing level */
+  ParsingLevel[doc] = L_Transitional;
 
   ResetStop (doc);
   language = TtaGetDefaultLanguage ();
   root = TtaGetMainRoot (doc);
   /* disable auto save */
   TtaSetDocumentBackUpInterval (doc, 0);
+  /* Set the document charset */
+  TtaSetDocumentCharset (doc, ISO_8859_1);
 
   elType = TtaGetElementType (root);
   attrType.AttrSSchema = elType.ElSSchema;
-  if (isHTML)
-    {
-      LoadUserStyleSheet (doc);
 
+  if (docType == docHTML)
+    {
+      /*-------------  New XHTML document ------------*/
+      /* force the XML parsing */
+      DocumentMeta[doc]->xmlformat = TRUE;
+      /* check the current profile */
+      profile = TtaGetEnvString ("Profile");
+      if (!ustrncmp (profile, TEXT("XHTML-basic"), 10))
+	ParsingLevel[doc] = L_Basic;
+      else if (!ustrncmp (profile, TEXT("XHTML-strict"), 10))
+	ParsingLevel[doc] = L_Strict;
+      
+      LoadUserStyleSheet (doc);
       /* attach an attribute PrintURL to the document root */
       attrType.AttrTypeNum = HTML_ATTR_PrintURL;
       attr = TtaNewAttribute (attrType);
@@ -201,8 +240,41 @@ ThotBool            isHTML;
       SelectionDoc = doc;
       UpdateContextSensitiveMenus (doc);
     }
+  else if (docType == docMath)
+    {
+      /*-------------  New MathML document ------------*/
+      /* force the XML parsing */
+      DocumentMeta[doc]->xmlformat = TRUE;
+
+      /* Search the first element to set initial selection */
+      elType.ElTypeNum = MathML_EL_Construct;
+      el = TtaSearchTypedElement (elType, SearchInTree, root);
+      /* set the initial selection */
+      TtaSelectElement (doc, el);
+      if (SelectionDoc != 0)
+	UpdateContextSensitiveMenus (SelectionDoc);
+      SelectionDoc = doc;
+      UpdateContextSensitiveMenus (doc);
+     }
+  else if (docType == docSVG)
+    {
+      /*-------------  New SVG document ------------*/
+      /* force the XML parsing */
+      DocumentMeta[doc]->xmlformat = TRUE;
+
+      /* Search the first element to set initial selection */
+      elType.ElTypeNum = GraphML_EL_GraphicsElement;
+      el = TtaSearchTypedElement (elType, SearchInTree, root);
+      /* set the initial selection */
+      TtaSelectElement (doc, el);
+      if (SelectionDoc != 0)
+	UpdateContextSensitiveMenus (SelectionDoc);
+      SelectionDoc = doc;
+      UpdateContextSensitiveMenus (doc);
+     }
   else
     {
+      /*-------------  Other documents ------------*/
       /* attach the default attribute PrintURL to the document root */
       attrType.AttrTypeNum = TextFile_ATTR_PrintURL;
       attr = TtaNewAttribute (attrType);
@@ -963,6 +1035,7 @@ View                view;
    Element             el, new, cell, row;
    AttributeType       attrType;
    Attribute           attr;
+   CHAR_T              stylebuff[50];
    int                 firstChar, i;
 
    if (!HTMLelementAllowed (document))
@@ -1014,23 +1087,41 @@ View                view;
          el = TtaGetTypedAncestor (el, elType);
        if (el != NULL)
 	 {
-	   /* take care of the Border attribute */
 	   attrType.AttrSSchema = elType.ElSSchema;
-	   attrType.AttrTypeNum = HTML_ATTR_Border;
-	   attr = TtaGetAttribute (el, attrType);
-	   if (attr != NULL && TBorder == 0)
-	     /* the table has a Border attribute but the user don't want any
-		border. Remove the attribute */
-	     TtaRemoveAttribute (el, attr, document);
+	   if (ParsingLevel[document] == L_Basic)
+	     {
+	       /* remove the Border attribute */
+	       attrType.AttrTypeNum = HTML_ATTR_Border;
+	       attr = TtaGetAttribute (el, attrType);
+	       if (attr != NULL)
+		 TtaRemoveAttribute (el, attr, document);
+	       /* generate a border style */
+	       attrType.AttrTypeNum = HTML_ATTR_Style_;
+	       attr = TtaNewAttribute (attrType);
+	       TtaAttachAttribute (el, attr, document);
+	       ustrcpy (stylebuff, TEXT("border:  "));
+	       usprintf (stylebuff, TEXT("border: solid %dpx"), TBorder);
+	       TtaSetAttributeText (attr, stylebuff, el, document);	       
+	     }
 	   else
 	     {
-	       if (attr == NULL)
-	         /* the Table has no Border attribute, create one */
+	       /* take care of the Border attribute */
+	       attrType.AttrTypeNum = HTML_ATTR_Border;
+	       attr = TtaGetAttribute (el, attrType);
+	       if (attr != NULL && TBorder == 0)
+		 /* the table has a Border attribute but the user don't want any
+		    border. Remove the attribute */
+		 TtaRemoveAttribute (el, attr, document);
+	       else
 		 {
-		   attr = TtaNewAttribute (attrType);
-		   TtaAttachAttribute (el, attr, document);
+		   if (attr == NULL)
+		     /* the Table has no Border attribute, create one */
+		     {
+		       attr = TtaNewAttribute (attrType);
+		       TtaAttachAttribute (el, attr, document);
+		     }
+		   TtaSetAttributeValue (attr, TBorder, el, document);
 		 }
-	       TtaSetAttributeValue (attr, TBorder, el, document);
 	     }
 
 	   if (NumberCols > 1)
@@ -2519,7 +2610,24 @@ View view;
 #endif /* __STDC__ */
 {
 #ifdef ANNOTATIONS
-  ANNOT_Create (doc, view);
+  ANNOT_Create (doc, view, TRUE);
+#endif /* ANNOTATIONS */
+}
+
+/*----------------------------------------------------------------------
+  AnnotateDocument
+  Frontend to the function that creates an annotation
+  ----------------------------------------------------------------------*/
+#ifdef __STDC__
+void AnnotateSelection (Document doc, View view)
+#else 
+void AnnotateSelection (doc, view)
+Document doc;
+View view;
+#endif /* __STDC__ */
+{
+#ifdef ANNOTATIONS
+  ANNOT_Create (doc, view, FALSE);
 #endif /* ANNOTATIONS */
 }
 
@@ -2557,6 +2665,23 @@ View view;
 #endif /* ANNOTATIONS */
 }
 
+/*----------------------------------------------------------------------
+  DeleteAnnotation
+  Frontend to the function that posts an annotation to the server
+  ----------------------------------------------------------------------*/
+#ifdef __STDC__
+void DeleteAnnotation (Document doc, View view)
+#else 
+void DeleteAnnotation (doc, view)
+Document doc;
+View view;
+#endif /* __STDC__ */
+{
+#ifdef ANNOTATIONS
+  ANNOT_Delete (doc, view);
+#endif /* ANNOTATIONS */
+}
+
 /*----------------------------------------------------------------------
    FilterAnnot
    Show/Hide the annotations
@@ -2586,6 +2711,12 @@ View                view;
 #endif
 {
 #ifdef ANNOTATIONS
+  /* temporary change for testing the xpointer generation */
+  char *xptr;
+  xptr = XPointer_build (document, view);
+  TtaFreeMemory (xptr);
+  /*
   CustomQueryMenuInit (document, view);
+  */
 #endif /* ANNOTATIONS */
 }
