@@ -18,6 +18,9 @@
 #define THOT_EXPORT extern
 #include "amaya.h"
 #include "css.h"
+#ifdef GRAPHML
+#include "GraphML.h"
+#endif /* GRAPHML */
 
 static Document     BgDocument;
 static int          RepeatValue;
@@ -441,7 +444,9 @@ char*               shape;
    STRING              url;
    int                 length, w, h;
    int                 firstchar, lastchar;
+   int                 docModified;
    DisplayMode         dispMode;
+   ThotBool            newMap;
 
    /* get the first selected element */
    TtaGiveFirstSelectedElement (doc, &el, &firstchar, &lastchar);
@@ -454,15 +459,16 @@ char*               shape;
      /* not within an HTML element. Nothing to do */
      return;
 
+   docModified = TtaIsDocumentModified (doc);
    /* ask Thot to stop displaying changes made in the document */
    dispMode = TtaGetDisplayMode (doc);
-   if (dispMode == DisplayImmediately)
-     TtaSetDisplayMode (doc, DeferredDisplay);
+   /*if (dispMode == DisplayImmediately)
+     TtaSetDisplayMode (doc, DeferredDisplay);*/
 
    TtaOpenUndoSequence (doc, el, el, 0, 0);
    newElem = NULL;
    attrRefimg = NULL;
-
+   newMap = FALSE;
    if (elType.ElTypeNum == HTML_EL_PICTURE_UNIT)
      /* an image is selected. Create an area for it */
      {
@@ -486,6 +492,7 @@ char*               shape;
 	     /* create the MAP element */
 	     elType.ElTypeNum = HTML_EL_MAP;
 	     map = TtaNewElement (doc, elType);
+	     newMap = TRUE;
 	     newElem = map;
 	     parent = image;
 	     do
@@ -494,9 +501,10 @@ char*               shape;
 		  parent = TtaGetParent (el);
 		  elType = TtaGetElementType (parent);
 	       }
-	     while (elType.ElTypeNum != HTML_EL_BODY);
+	     while (elType.ElTypeNum != HTML_EL_BODY &&
+		    elType.ElTypeNum != HTML_EL_Division );
 	     TtaInsertSibling (map, el, FALSE, doc);
-	     CreateTargetAnchor (doc, map, FALSE);
+	     CreateTargetAnchor (doc, map, FALSE, FALSE);
 	     attrType.AttrTypeNum = HTML_ATTR_NAME;
 	     attr = TtaGetAttribute (map, attrType);
 
@@ -617,7 +625,22 @@ char*               shape;
 	TtaSetDisplayMode (doc, dispMode);
 	TtaSelectElement (doc, child);
 	if (shape[0] == 'p')
-	   TtcInsertGraph (doc, 1, 'p');
+	  {
+	    TtcInsertGraph (doc, 1, 'p');
+	    if (TtaGetVolume (child) < 3)
+	      {
+		/* the polyline doesn't have enough points */
+		if (newMap)
+		  TtaDeleteTree (map, doc);
+		else
+		  TtaDeleteTree (el, doc);
+		TtaCancelLastRegisteredSequence (doc);
+		if (!docModified)
+		  TtaSetDocumentUnmodified (doc);
+		TtaSelectElement (doc, image);
+		return;
+	      }
+	  }
 	/* Compute coords attribute */
 	SetAreaCoords (doc, el, 0);
 
@@ -843,7 +866,7 @@ void ChangeBackgroundImage (document, view)
       ustrcat (s, ImageName);
    }
    BgDocument = document;
-   CreateBackgroundImageDlgWindow (TtaGetViewFrame (document, view), BaseImage + FormBackground, ImageURL, ImageLabel, ImageDir, ImageSel, RepeatImage, s);
+   CreateBackgroundImageDlgWindow (TtaGetViewFrame (document, view), s);
 #  endif /* _WINDOWS */
 }
 
@@ -1081,19 +1104,67 @@ NotifyAttribute    *event;
   CreateImage
   ----------------------------------------------------------------------*/
 #ifdef __STDC__
-void                CreateImage (Document document, View view)
+void                CreateImage (Document doc, View view)
 #else  /* __STDC__ */
-void                CreateImage (document, view)
-Document            document;
+void                CreateImage (doc, view)
+Document            doc;
 View                view;
 
 #endif /* __STDC__ */
 {
-   ElementType         elType;
+  Element            sibling, child, graphRoot;
+  ElementType        elType, selType;
+  STRING             name;
+  int                c1, i;
 
-   elType.ElSSchema = TtaGetSSchema (TEXT("HTML"), document);
-   elType.ElTypeNum = HTML_EL_PICTURE_UNIT;
-   TtaCreateElement (elType, document);
+  TtaGiveFirstSelectedElement (doc, &sibling, &c1, &i); 
+  if (sibling)
+    {
+      /* Get the type of the first selected element */
+      elType = TtaGetElementType (sibling);
+      name = TtaGetSSchemaName (elType.ElSSchema);
+      graphRoot = NULL;
+#ifdef GRAPHML
+      if (!ustrcmp (name, TEXT("GraphML")))
+	{
+	  elType.ElTypeNum = GraphML_EL_GraphML;
+	  /* look for the enclosing SVG */
+	  do
+	    {
+	      child = sibling;
+	      sibling = TtaGetParent (child);
+	      if (sibling)
+		selType = TtaGetElementType (sibling);
+	    }
+	  while (sibling && selType.ElTypeNum != elType.ElTypeNum &&
+		 selType.ElSSchema == elType.ElSSchema);
+	  if (sibling)
+	    {
+	      graphRoot = sibling;
+	      sibling = child;
+	    }
+	}
+      if (graphRoot)
+	{
+	  /* selection is within a SVG element */
+	  elType.ElTypeNum = GraphML_EL_image;
+	  /*TtaAskFirstCreation ();*/
+	  child = TtaNewElement (doc, elType);
+	  TtaInsertSibling (child, sibling, FALSE, doc);
+	  sibling = child;
+	  elType.ElTypeNum = GraphML_EL_PICTURE_UNIT;
+	  /*TtaAskFirstCreation ();*/
+	  child = TtaNewElement (doc, elType);	  
+	  TtaInsertFirstChild (&child, sibling, doc);
+	}
+      else
+#endif /* GRAPHML */
+	{
+	  elType.ElSSchema = TtaGetSSchema (TEXT("HTML"), doc);
+	  elType.ElTypeNum = HTML_EL_PICTURE_UNIT;
+	  TtaCreateElement (elType, doc);
+	}
+    }
 }
 
 
