@@ -1,6 +1,6 @@
 /*
  *
- *  (c) COPYRIGHT MIT and INRIA, 1996.
+ *  (c) COPYRIGHT MIT and INRIA, 1996-2000
  *  Please first read the full copyright statement in file COPYRIGHT.
  *
  */
@@ -29,8 +29,9 @@
 #include "UIcss_f.h"
 #include "css_f.h"
 #include "fetchHTMLname_f.h"
-#include "init_f.h"
+#include "fetchXMLname_f.h"
 #include "html2thot_f.h"
+#include "init_f.h"
 #include "styleparser_f.h"
 
 #ifdef _WINDOWS
@@ -50,9 +51,9 @@ static Document	    ApplyClassDoc;
   The parameter removeSpan is True when the span has to be removed.
   ----------------------------------------------------------------------*/
 #ifdef __STDC__
-void              RemoveElementStyle (Element el, Document doc, ThotBool removeSpan)
+static void       RemoveElementStyle (Element el, Document doc, ThotBool removeSpan)
 #else
-void              RemoveElementStyle (el, doc, removeSpan)
+static void       RemoveElementStyle (el, doc, removeSpan)
 Element           el;
 Document          doc;
 ThotBool          removeSpan;
@@ -67,8 +68,7 @@ ThotBool          removeSpan;
       return;
    elType = TtaGetElementType (el);
    /* if it's a MathML element, remove the style attribute defined in the
-      MathML DTD, otherwise, remove the style attribute defined in the
-      HTML DTD */
+      MathML DTD */
    if (!ustrcmp (TtaGetSSchemaName (elType.ElSSchema), TEXT("MathML")))
       {
       attrType.AttrSSchema = elType.ElSSchema;
@@ -77,8 +77,7 @@ ThotBool          removeSpan;
    else
 #ifdef GRAPHML
    /* if it's a GraphML element, remove the style attribute defined in the
-      GraphML DTD, otherwise, remove the style attribute defined in the
-      HTML DTD */
+      GraphML DTD */
    if (!ustrcmp (TtaGetSSchemaName (elType.ElSSchema), TEXT("GraphML")))
       {
       attrType.AttrSSchema = elType.ElSSchema;
@@ -86,6 +85,7 @@ ThotBool          removeSpan;
       }
    else
 #endif
+      /* remove the style attribute defined in the HTML DTD */
       {
       attrType.AttrSSchema = TtaGetSSchema (TEXT("HTML"), doc);
       attrType.AttrTypeNum = HTML_ATTR_Style_;
@@ -226,14 +226,35 @@ NotifyAttribute    *event;
 #endif
 {
    ElementType         elType;
-   ThotBool            isHTML;
+   Element             el;
+   CHAR_T*             style = NULL;
+   int                 len;
 
-   /*  A rule applying to BODY is really meant to address HTML */
+   el = event->element;
+   /*  A rule applying to BODY is really meant to address the HTML element */
    elType = TtaGetElementType (event->element);
-   isHTML = (ustrcmp (TtaGetSSchemaName (elType.ElSSchema), TEXT("HTML")) == 0);
-   if (elType.ElTypeNum == HTML_EL_BODY && isHTML)
-     TtaCleanStylePresentation (TtaGetParent (event->element), NULL, event->document);
-  TtaCleanStylePresentation (event->element, NULL, event->document);
+   if (elType.ElTypeNum == HTML_EL_BODY &&
+       ustrcmp (TtaGetSSchemaName (elType.ElSSchema), TEXT("HTML")) == 0)
+      el = TtaGetParent (el);
+   len = TtaGetTextAttributeLength (event->attribute);
+   if ((len < 0) || (len > 10000))
+      /* error */
+      return FALSE;
+   if (len == 0)
+      /* empty Style attribute */
+      return FALSE;
+   else
+     {
+	/* parse the old content and remove the corresponding presentation
+	   rules */
+	style = TtaAllocString (len + 2);
+	if (!style)
+	   return FALSE;
+	TtaGiveTextAttributeValue (event->attribute, style, &len);
+	style[len] = WC_EOS;
+	ParseHTMLSpecificStyle (el, style, event->document, 1, TRUE);
+	TtaFreeMemory (style);
+     }
   return FALSE;  /* let Thot perform normal operation */
 }
 
@@ -308,7 +329,8 @@ NotifyAttribute    *event;
 	    braces++;
 	  if (i > 0 &&
 	      (OldBuffer[i-1] == TEXT('}') ||
-           ((OldBuffer[i-1] == TEXT(';') || OldBuffer[i-1] == TEXT('>')) && braces == 0)))
+	       ((OldBuffer[i-1] == TEXT(';') || OldBuffer[i-1] == TEXT('>')) &&
+		braces == 0)))
 	    {
 	      if (OldBuffer[i-1] == TEXT('}'))
 		braces--;
@@ -367,7 +389,8 @@ NotifyAttribute    *event;
 			url1 = GetCSSBackgroundURL (ptr1);
 			if (url1 != NUL)
 			{
-			sprintf (path, "%s%s%d%s", TempFileDirectory, DIR_STR, event->document, DIR_STR, url1);
+			sprintf (path, "%s%s%d%s", TempFileDirectory, DIR_STR,
+			         event->document, DIR_STR, url1);
 			pImage = SearchLoadedImage (path, event->document);
 			
 			}
@@ -426,9 +449,6 @@ NotifyAttribute    *event;
    el = event->element;
    doc = event->document;
 
-   /* First remove all the presentation specific rules applied to the element. */
-   UpdateStyleDelete (event);
-
    len = TtaGetTextAttributeLength (event->attribute);
    if ((len < 0) || (len > 10000))
       return;
@@ -437,8 +457,7 @@ NotifyAttribute    *event;
 	/* empty Style attribute. Delete it */
 	elType = TtaGetElementType (el);
         /* if it's a MathML element, delete the style attribute defined in the
-           MathML DTD, otherwise, delete the style attribute defined in the
-           HTML DTD */
+           MathML DTD */
 	if (!ustrcmp (TtaGetSSchemaName (elType.ElSSchema), TEXT("MathML")))
 	   {
 	   atType.AttrSSchema = elType.ElSSchema;
@@ -447,8 +466,7 @@ NotifyAttribute    *event;
 	else
 #ifdef GRAPHML
         /* if it's a GraphML element, delete the style attribute defined in the
-           GraphML DTD, otherwise, delete the style attribute defined in the
-           HTML DTD */
+           GraphML DTD */
 	if (!ustrcmp (TtaGetSSchemaName (elType.ElSSchema), TEXT("GraphML")))
 	   {
 	   atType.AttrSSchema = elType.ElSSchema;
@@ -456,9 +474,10 @@ NotifyAttribute    *event;
 	   }
 	else
 #endif
+	   /* delete the style attribute defined in the HTML DTD */
 	   {
-    	atType.AttrSSchema = TtaGetSSchema (TEXT("HTML"), doc);
-    	atType.AttrTypeNum = HTML_ATTR_Style_;
+	   atType.AttrSSchema = TtaGetSSchema (TEXT("HTML"), doc);
+	   atType.AttrTypeNum = HTML_ATTR_Style_;
 	   }
 	at = TtaGetAttribute (el, atType);
 	if (at != NULL)
@@ -492,7 +511,7 @@ NotifyAttribute    *event;
 	   /* a new SPAN element has been created. Generate the PRules
 	      for the SPAN element */
 	   el = newParent;
-	ParseHTMLSpecificStyle (el, style, doc, FALSE);
+	ParseHTMLSpecificStyle (el, style, doc, 1, FALSE);
 	TtaFreeMemory (style);
      }
 }
@@ -581,27 +600,28 @@ Document            doc;
   /* process the first selected element */
   elType = TtaGetElementType (firstSelectedEl);
   if (elType.ElTypeNum == HTML_EL_TEXT_UNIT)
-     /* it's a text element */
-     if (firstSelectedChar <= 1)
+    {
+      /* it's a text element */
+      if (firstSelectedChar <= 1)
 	/* selection starts at the beginning of the element */
 	/* this text element is then entirely selected */
 	{
-	parent = TtaGetParent (firstSelectedEl);
-	elType = TtaGetElementType (parent);
-	if (elType.ElTypeNum == HTML_EL_Span &&
-	    !ustrcmp (TtaGetSSchemaName (elType.ElSSchema), TEXT("HTML")))
-	   /* parent is a SPAN element */
-	   if (firstSelectedEl == TtaGetFirstChild (parent) &&
-	       firstSelectedEl == TtaGetLastChild (parent))
+	  parent = TtaGetParent (firstSelectedEl);
+	  elType = TtaGetElementType (parent);
+	  if (elType.ElTypeNum == HTML_EL_Span &&
+	      !ustrcmp (TtaGetSSchemaName (elType.ElSSchema), TEXT("HTML")))
+	    /* parent is a SPAN element */
+	    if (firstSelectedEl == TtaGetFirstChild (parent) &&
+		firstSelectedEl == TtaGetLastChild (parent))
 	      /* this text element is the only child of the SPAN */
 	      /* Process the SPAN instead of the text element */
 	      {
-	      firstSelectedEl = parent;
-	      if (lastSelectedEl == firstSelectedEl)
-	         lastSelectedEl = parent;
+		firstSelectedEl = parent;
+		if (lastSelectedEl == firstSelectedEl)
+		  lastSelectedEl = parent;
 	      }
 	}
-     else
+      else
 	/* that element is only partly selected. Split it */
 	{
 	el = firstSelectedEl;
@@ -614,6 +634,7 @@ Document            doc;
 	   lastSelectedEl = firstSelectedEl;
 	   }
 	}
+    }
 
   TtaOpenUndoSequence (doc, firstSelectedEl, lastSelectedEl, 0, 0);
   /* process all selected elements */
@@ -645,50 +666,52 @@ Document            doc;
 	 {
 	  elType = TtaGetElementType (curEl);
 	  if (elType.ElTypeNum == HTML_EL_TEXT_UNIT)
-	     /* that's a text element */
-	     if (ustrcmp (TtaGetSSchemaName (elType.ElSSchema), TEXT("HTML")))
+	    {
+	      /* that's a text element */
+	      if (ustrcmp (TtaGetSSchemaName (elType.ElSSchema), TEXT("HTML")))
 		/* not a HTML element, move to the parent element */
 		curEl = TtaGetParent (curEl);
-	     else
+	      else
 	        /* we are in a HTML element. Create an enclosing SPAN element*/
 	        {
-	        MakeASpan (curEl, &span, doc);
-	        if (span)
-		   /* a SPAN element was created */
-		   {
-		   if (curEl == firstSelectedEl)
-		      {
-		      firstSelectedEl = span;
-		      if (firstSelectedEl == lastSelectedEl)
-		         lastSelectedEl = span;
-		      }
-		   else if (curEl == lastSelectedEl)
-		      lastSelectedEl = span;
-		   curEl = span;
-		   }
+		  MakeASpan (curEl, &span, doc);
+		  if (span)
+		    /* a SPAN element was created */
+		    {
+		      if (curEl == firstSelectedEl)
+			{
+			  firstSelectedEl = span;
+			  if (firstSelectedEl == lastSelectedEl)
+			    lastSelectedEl = span;
+			}
+		      else if (curEl == lastSelectedEl)
+			lastSelectedEl = span;
+		      curEl = span;
+		    }
 	        }
+	    }
 	  if (!ustrcmp (TtaGetSSchemaName (elType.ElSSchema), TEXT("MathML")))
-	     {
-	     attrType.AttrSSchema = elType.ElSSchema;
-	     attrType.AttrTypeNum = MathML_ATTR_class;
-	     }
+	    {
+	      attrType.AttrSSchema = elType.ElSSchema;
+	      attrType.AttrTypeNum = MathML_ATTR_class;
+	    }
 	  else
 #ifdef GRAPHML
-	  if (!ustrcmp (TtaGetSSchemaName (elType.ElSSchema), TEXT("GraphML")))
-	     {
-	     attrType.AttrSSchema = elType.ElSSchema;
-	     attrType.AttrTypeNum = GraphML_ATTR_class;
-	     }
-	  else
+	    if (!ustrcmp (TtaGetSSchemaName (elType.ElSSchema), TEXT("GraphML")))
+	      {
+		attrType.AttrSSchema = elType.ElSSchema;
+		attrType.AttrTypeNum = GraphML_ATTR_class;
+	      }
+	    else
 #endif
-	     {
-	     attrType.AttrSSchema = TtaGetSSchema (TEXT("HTML"), doc);
-	     attrType.AttrTypeNum = HTML_ATTR_Class;
-	     }
+	      {
+		attrType.AttrSSchema = TtaGetSSchema (TEXT("HTML"), doc);
+		attrType.AttrTypeNum = HTML_ATTR_Class;
+	      }
 	  /* set the Class attribute of the element */
 	  attr = TtaGetAttribute (curEl, attrType);
 	  if (!attr)
-	     {
+	    {
 	      attr = TtaNewAttribute (attrType);
 	      TtaAttachAttribute (curEl, attr, doc);
 	      TtaSetAttributeText (attr, a_class, curEl, doc);
@@ -742,11 +765,11 @@ void                *param;
       else
 	ptr = MakeRelativeURL (settings->value.pointer, DocumentURLs[doc]);
       settings->value.pointer = ptr;
-      PToCss (settings, string, sizeof(string));
+      PToCss (settings, string, sizeof(string), el);
       TtaFreeMemory (ptr);
     }
   else
-    PToCss (settings, string, sizeof(string));
+    PToCss (settings, string, sizeof(string), el);
 
   if (string[0] != WC_EOS && *css_rules != WC_EOS)
     ustrcat (css_rules, TEXT("; "));
@@ -796,7 +819,8 @@ int        *len;
 	  if (*len > 0)
 	    ustrcat(buf, TEXT(";"));
 	  *len = ustrlen (buf);
-	  TtaApplyAllSpecificSettings (el, doc, SpecificSettingsToCSS, &buf[*len]);
+	  TtaApplyAllSpecificSettings (el, doc, SpecificSettingsToCSS,
+				       &buf[*len]);
 	  *len = ustrlen (buf);
 	}
       else if (elType.ElTypeNum == HTML_EL_BODY)
@@ -807,7 +831,8 @@ int        *len;
 	  if (*len > 0)
 	    ustrcat(buf, TEXT(";"));
 	  *len = ustrlen (buf);
-	  TtaApplyAllSpecificSettings (el, doc, SpecificSettingsToCSS, &buf[*len]);
+	  TtaApplyAllSpecificSettings (el, doc, SpecificSettingsToCSS,
+				       &buf[*len]);
 	  *len = ustrlen (buf);
 	}
     }
@@ -850,11 +875,17 @@ Document            doc;
   /* get the current style attribute*/
   schName = TtaGetSSchemaName (elType.ElSSchema);
   if (ustrcmp (schName, TEXT("MathML")) == 0)
-    attrType.AttrTypeNum = MathML_ATTR_style_;
+    {
+      attrType.AttrSSchema = elType.ElSSchema;
+      attrType.AttrTypeNum = MathML_ATTR_style_;
+    }
   else
 #ifdef GRAPHML
   if (ustrcmp (schName, TEXT("GraphML")) == 0)
-    attrType.AttrTypeNum = GraphML_ATTR_style_;
+    {
+      attrType.AttrSSchema = elType.ElSSchema;
+      attrType.AttrTypeNum = GraphML_ATTR_style_;
+    }
   else
 #endif
     {
@@ -1021,45 +1052,47 @@ Document            doc;
 	  }
       }
     if (elType.ElTypeNum != HTML_EL_TEXT_UNIT)
-      if (elType.ElTypeNum != HTML_EL_Comment_)
-	 /* the last child of the STYLE element is neither a text leaf nor
-	    a comment. Don't do anything */
-	 child = NULL;
-      else
-	 /* the last child of the STYLE element is a comment */
-	 /* insert the new style rule within the Comment_line */
-	 {
-	 line = TtaGetLastChild (child);
-	 if (line)
-	   /* there is already a Comment_line */
-	   {
-           child = TtaGetLastChild (line);
-	   len = TtaGetTextLength (child) + 1;
-	   text = TtaAllocString (len);
-	   TtaGiveTextContent (child, text, &len, &lang);
-	   empty = TRUE;
-	   insertNewLine = TRUE;
-	   for (i = len - 1; i >= 0 && empty; i--)
-	     {
-	       empty = text[i] <= SPACE;
-               if ((int) text[i] == EOL || (int) text[i] == __CR__)
-	         insertNewLine = FALSE;
-	     }
-	   TtaFreeMemory (text);
-	   }
-	 else
-	   /* create a Comment_line within the Comment */
-	   {
-	   elType.ElTypeNum = HTML_EL_Comment_line;
-           line = TtaNewTree (doc, elType, "");
-           TtaInsertFirstChild (&line, child, doc);
-	   child = TtaGetLastChild (line);
-	   insertNewLine = FALSE;
-	   /* remember the element to register in the undo queue */
-           found = FALSE;
-	   el = line;
-	   }
-         }
+      {
+	if (elType.ElTypeNum != HTML_EL_Comment_)
+	  /* the last child of the STYLE element is neither a text leaf nor
+	     a comment. Don't do anything */
+	  child = NULL;
+	else
+	  /* the last child of the STYLE element is a comment */
+	  /* insert the new style rule within the Comment_line */
+	  {
+	    line = TtaGetLastChild (child);
+	    if (line)
+	      /* there is already a Comment_line */
+	      {
+		child = TtaGetLastChild (line);
+		len = TtaGetTextLength (child) + 1;
+		text = TtaAllocString (len);
+		TtaGiveTextContent (child, text, &len, &lang);
+		empty = TRUE;
+		insertNewLine = TRUE;
+		for (i = len - 1; i >= 0 && empty; i--)
+		  {
+		    empty = text[i] <= SPACE;
+		    if ((int) text[i] == EOL || (int) text[i] == __CR__)
+		      insertNewLine = FALSE;
+		  }
+		TtaFreeMemory (text);
+	      }
+	    else
+	      /* create a Comment_line within the Comment */
+	      {
+		elType.ElTypeNum = HTML_EL_Comment_line;
+		line = TtaNewTree (doc, elType, "");
+		TtaInsertFirstChild (&line, child, doc);
+		child = TtaGetLastChild (line);
+		insertNewLine = FALSE;
+		/* remember the element to register in the undo queue */
+		found = FALSE;
+		el = line;
+	      }
+	  }
+      }
     }
 
   if (child)
@@ -1218,6 +1251,9 @@ STRING              first;
         }
      }
 #endif
+  /* look for all class names that are used in the STYLE element and in
+     all style sheets currently associated with the document */
+  /*** TODO ***/
   return (nb);
 }
 	    
@@ -1265,14 +1301,20 @@ View                view;
       elType = TtaGetElementType (ClassReference);
     }
 
-  /* check if the element has an style attribute */
+  /* check if the element has a style attribute */
   schName = TtaGetSSchemaName (elType.ElSSchema);
   if (ustrcmp (schName, TEXT("MathML")) == 0)
-    attrType.AttrTypeNum = MathML_ATTR_style_;
+    {
+      attrType.AttrSSchema = elType.ElSSchema;
+      attrType.AttrTypeNum = MathML_ATTR_style_;
+    }
   else
 #ifdef GRAPHML
   if (ustrcmp (schName, TEXT("GraphML")) == 0)
-    attrType.AttrTypeNum = GraphML_ATTR_style_;
+    {
+      attrType.AttrSSchema = elType.ElSSchema;
+      attrType.AttrTypeNum = GraphML_ATTR_style_;
+    }
   else
 #endif
     {
@@ -1285,7 +1327,7 @@ View                view;
     return;
 
   /* update the class name selector. */
-  elHtmlName = GITagName (ClassReference);
+  elHtmlName =  GetXMLElementName (TtaGetElementType (ClassReference), doc);
   if (elHtmlName[0] == TEXT('?'))
     InitConfirm (doc, 1, TtaGetMessage (AMAYA, AM_SEL_CLASS));
   else
@@ -1317,16 +1359,16 @@ View                view;
 	{
 	  len = 50;
 	  TtaGiveTextAttributeValue (attr, a_class, &len);
-#     ifndef _WINDOWS
+#ifndef _WINDOWS
 	  TtaSetSelector (BaseDialog + ClassSelect, -1, a_class);
-#     endif /* _WINDOWS */
+#endif /* _WINDOWS */
 	  ustrcpy (CurrentClass, a_class);
 	}
       else
 	{
-#     ifndef _WINDOWS
+#ifndef _WINDOWS
 	  TtaSetSelector (BaseDialog + ClassSelect, 0, NULL);
-#     endif /* _WINDOWS */
+#endif /* _WINDOWS */
 	  ustrcpy (CurrentClass, elHtmlName);
 	}
   
@@ -1371,18 +1413,18 @@ View                view;
   ApplyClassDoc = doc;
 
   /* updating the class name selector. */
-#  ifndef _WINDOWS
+#ifndef _WINDOWS
    ustrcpy (bufMenu, TtaGetMessage (LIB, TMSG_APPLY));
    TtaNewSheet (BaseDialog + AClassForm, TtaGetViewFrame (doc, 1), 
 		TtaGetMessage (AMAYA, AM_APPLY_CLASS), 1,
 		bufMenu, FALSE, 2, 'L', D_DONE);
-#  endif /* !_WINDOWS */
+#endif /* !_WINDOWS */
   NbClass = BuildClassList (doc, ListBuffer, MAX_CSS_LENGTH, TEXT("default"));
-#  ifndef _WINDOWS
+#ifndef _WINDOWS
   TtaNewSelector (BaseDialog + AClassSelect, BaseDialog + AClassForm,
 		  TtaGetMessage (AMAYA, AM_SEL_CLASS),
 		  NbClass, ListBuffer, 5, NULL, FALSE, TRUE);
-#  endif /* !_WINDOWS */
+#endif /* !_WINDOWS */
 
   /* preselect the entry corresponding to the class of the first selected
      element. */
@@ -1411,25 +1453,25 @@ View                view;
     {
       len = 50;
       TtaGiveTextAttributeValue (attr, a_class, &len);
-#     ifndef _WINDOWS
+#ifndef _WINDOWS
       TtaSetSelector (BaseDialog + AClassSelect, -1, a_class);
-#     endif /* !_WINDOWS */
+#endif /* !_WINDOWS */
       ustrcpy (CurrentClass, a_class);
     }
   else
     {
-#     ifndef _WINDOWS
+#ifndef _WINDOWS
       TtaSetSelector (BaseDialog + AClassSelect, 0, NULL);
-#     endif /* !_WINDOWS */
+#endif /* !_WINDOWS */
       ustrcpy (CurrentClass, TEXT("default"));
     }
 
    /* pop-up the dialogue box. */
-#  ifndef _WINDOWS
+#ifndef _WINDOWS
   TtaShowDialogue (BaseDialog + AClassForm, TRUE);
-#  else  /* _WINDOWS */
+#else  /* _WINDOWS */
   CreateApplyClassDlgWindow (TtaGetViewFrame (doc, 1), NbClass, ListBuffer);
-#  endif /* _WINDOWS */
+#endif /* _WINDOWS */
 }
 
 /*----------------------------------------------------------------------
