@@ -354,8 +354,8 @@ ThotBool            remove;
 }
 
 /*----------------------------------------------------------------------
-   SearchNAMEattribute
-   Search in document doc an element having an attribut of type attrType
+   GetElemWithAttr
+   Search in document doc an element having an attribute of type attrType
    whose value is nameVal.
    Return that element or NULL if not found.
    If ignore is not NULL, it is an attribute that should be ignored when
@@ -467,6 +467,21 @@ Attribute           ignore;
 	  }
      }
 #endif
+#ifdef ANNOTATIONS
+   if (!elFound)
+     {
+       /* search all elements having an attribute ID (defined in the
+	  XLink S schema) */
+       attrType.AttrSSchema = TtaGetSSchema (TEXT("XLink"), doc);
+       if (attrType.AttrSSchema)
+	  /* this document uses the XLink DTD */
+	  {
+          attrType.AttrTypeNum = XLink_ATTR_id;
+          elFound = GetElemWithAttr (doc, attrType, nameVal, ignore);
+	  }
+     }
+#endif /* ANNOTATIONS */
+
    return (elFound);
 }
 
@@ -568,7 +583,10 @@ void *context;
 		{
 		/* jump in the same document */
 		  /* record current position in the history */
-		  AddDocHistory (doc, DocumentURLs[doc], DocumentMeta[doc]->form_data, DocumentMeta[doc]->method);
+		  AddDocHistory (doc, DocumentURLs[doc], 
+				 DocumentMeta[doc]->initial_url, 
+				 DocumentMeta[doc]->form_data,
+				 DocumentMeta[doc]->method);
 		}
 	      /* show the target element in all views */
 	      for (view = 1; view < 6; view++)
@@ -577,8 +595,6 @@ void *context;
 	    }
 	}
     }
-  if (targetDocument > 0)
-    TtaRaiseView (targetDocument, 1); 
   TtaFreeMemory (url);
   if (sourceDocUrl)
     TtaFreeMemory (sourceDocUrl);
@@ -602,7 +618,7 @@ Document         doc;
    AttributeType          attrType;
    Attribute              HrefAttr, PseudoAttr, attr;
 #ifdef ANNOTATIONS
-   Attribute              isAnnotLink;
+   ThotBool               isAnnotLink;
 #endif /* ANNOTATIONS */
    ElementType            elType;
    Document               targetDocument;
@@ -616,7 +632,7 @@ Document         doc;
    info = NULL;
    HrefAttr = NULL;
 #ifdef ANNOTATIONS
-   isAnnotLink = NULL;
+   isAnnotLink = FALSE;
 #endif /* ANNOTATIONS */
    HTMLSSchema = TtaGetSSchema (TEXT("HTML"), doc);
 
@@ -641,6 +657,12 @@ Document         doc;
 	     {
 	        attrType.AttrSSchema = TtaGetSSchema (TEXT("XLink"), doc);
 		attrType.AttrTypeNum = XLink_ATTR_href_;
+#ifdef ANNOTATIONS
+		/* is it an annotation link? */
+		if (elType.ElSSchema == attrType.AttrSSchema
+		    && elType.ElTypeNum == XLink_EL_XLink)
+		  isAnnotLink = TRUE;
+#endif /* ANNOTATIONS */
 	     }
 
 	HrefAttr = TtaGetAttribute (anchor, attrType);
@@ -671,12 +693,6 @@ Document         doc;
 		     TtaAttachAttribute (anchor, PseudoAttr, doc);
 		   }
 		 TtaSetAttributeText (PseudoAttr, TEXT("active"), anchor, doc);
-
-#ifdef ANNOTATIONS
-		 /* see if it's a link to an annotation */
-		 attrType.AttrTypeNum = HTML_ATTR_IsAnnotation;
-		 isAnnotLink = TtaGetAttribute (anchor, attrType);
-#endif /* ANNOTATIONS */
 	       }
 	     /* get the URL itself */
 	     TtaGiveTextAttributeValue (HrefAttr, url, &length);
@@ -813,11 +829,16 @@ Document            document;
    Element             anchor, elFound, ancestor;
    ElementType         elType, elType1;
    SSchema             HTMLschema, XLinkSchema;
-   ThotBool	       ok, isHTML;
+   ThotBool	       ok, isHTML, isXLink;
 
    elType = TtaGetElementType (element);
    HTMLschema = TtaGetSSchema (TEXT("HTML"), document);
    isHTML = TtaSameSSchemas (elType.ElSSchema, HTMLschema);
+   if (!isHTML)
+       isXLink = TtaSameSSchemas (elType.ElSSchema, 
+				  TtaGetSSchema (TEXT("XLink"), document));
+   else
+     isXLink = 0;
 
    /* Check if the current element is interested in double clicks */
    ok = FALSE;
@@ -827,24 +848,26 @@ Document            document;
        elType.ElTypeNum == HTML_EL_SYMBOL_UNIT)
      /* it's a basic element. It is interested whatever its namespace */
      ok = TRUE;
-   else
-     if (isHTML)
-	if (elType.ElTypeNum == HTML_EL_LINK ||
-	    elType.ElTypeNum == HTML_EL_C_Empty ||
-	    elType.ElTypeNum == HTML_EL_Radio_Input ||
-	    elType.ElTypeNum == HTML_EL_Checkbox_Input ||
-	    elType.ElTypeNum == HTML_EL_Frame ||
-	    elType.ElTypeNum == HTML_EL_Option_Menu ||
-	    elType.ElTypeNum == HTML_EL_Submit_Input ||
-	    elType.ElTypeNum == HTML_EL_Reset_Input ||
-	    elType.ElTypeNum == HTML_EL_BUTTON ||
-	    elType.ElTypeNum == HTML_EL_File_Input ||
-	    elType.ElTypeNum == HTML_EL_FRAME ||
-	    elType.ElTypeNum == HTML_EL_Anchor)
-	   ok = TRUE;
+   else if (isHTML
+	    && (elType.ElTypeNum == HTML_EL_LINK ||
+		elType.ElTypeNum == HTML_EL_C_Empty ||
+		elType.ElTypeNum == HTML_EL_Radio_Input ||
+		elType.ElTypeNum == HTML_EL_Checkbox_Input ||
+		elType.ElTypeNum == HTML_EL_Frame ||
+		elType.ElTypeNum == HTML_EL_Option_Menu ||
+		elType.ElTypeNum == HTML_EL_Submit_Input ||
+		elType.ElTypeNum == HTML_EL_Reset_Input ||
+		elType.ElTypeNum == HTML_EL_BUTTON ||
+		elType.ElTypeNum == HTML_EL_File_Input ||
+		elType.ElTypeNum == HTML_EL_FRAME ||
+		elType.ElTypeNum == HTML_EL_Anchor))
+     ok = TRUE;
+   else if (isXLink)
+     ok = TRUE;
+
    if (!ok)
-      /* DoubleClick is disabled for this element type */
-      return (FALSE);
+     /* DoubleClick is disabled for this element type */
+     return (FALSE);
 
    if (isHTML && (elType.ElTypeNum == HTML_EL_Frame ||
 		  elType.ElTypeNum == HTML_EL_Submit_Input ||
@@ -940,17 +963,25 @@ Document            document;
      }
 
    /* Search the anchor or LINK element */
-   anchor = SearchAnchor (document, element, TRUE, FALSE);
-   if (anchor == NULL)
-      if (isHTML && (elType.ElTypeNum == HTML_EL_LINK ||
-		     elType.ElTypeNum == HTML_EL_FRAME))
-	   anchor = element;
-      else
-	{
-	   elType1.ElTypeNum = HTML_EL_LINK;
-	   elType1.ElSSchema = HTMLschema;
-	   anchor = TtaGetTypedAncestor (element, elType1);
-	}
+   if (!isXLink)
+     {
+       anchor = SearchAnchor (document, element, TRUE, FALSE);
+       if (anchor == NULL)
+	 {
+	   if (isHTML && (elType.ElTypeNum == HTML_EL_LINK ||
+			  elType.ElTypeNum == HTML_EL_FRAME))
+	     anchor = element;
+	   else
+	     {
+	       elType1.ElTypeNum = HTML_EL_LINK;
+	       elType1.ElSSchema = HTMLschema;
+	       anchor = TtaGetTypedAncestor (element, elType1);
+	     }
+	 }
+     }
+   else
+     anchor = NULL;
+
    /* if not found, search a cite or href attribute (from HTML or XLink
       namespaces) on an ancestor */
    if (anchor == NULL)
@@ -984,6 +1015,150 @@ Document            document;
    return (FollowTheLink (anchor, element, document));
 }
 
+/*----------------------------------------------------------------------
+  DisplayUrlAnchor    Display the url when an anchor is selectionned 
+  ----------------------------------------------------------------------*/
+#ifdef __STDC__
+static void     DisplayUrlAnchor (Element element, Document document)
+#else  /* __STDC__ */
+static void     DisplayUrlAnchor (element, document)
+Element             element;
+Document            document;
+#endif /* __STDC__ */
+{
+   AttributeType       attrType;
+   Attribute           attr, HrefAttr = NULL;
+   Element             anchor, ancestor;
+   ElementType         elType, elType1;
+   SSchema             HTMLschema, XLinkSchema;
+   ThotBool	       ok, isHTML, isXLink;
+   STRING              url, pathname, documentname;
+   int                 length;
+
+   elType = TtaGetElementType (element);
+   HTMLschema = TtaGetSSchema (TEXT("HTML"), document);
+   isHTML = TtaSameSSchemas (elType.ElSSchema, HTMLschema);
+   if (!isHTML)
+     isXLink = TtaSameSSchemas (elType.ElSSchema, 
+				TtaGetSSchema (TEXT("XLink"), document));
+   else
+     isXLink = 0;
+   
+   /* Check if the current element is interested in display url */
+   ok = FALSE;
+   if (elType.ElTypeNum == HTML_EL_PICTURE_UNIT ||
+       elType.ElTypeNum == HTML_EL_TEXT_UNIT ||
+       elType.ElTypeNum == HTML_EL_GRAPHICS_UNIT ||
+       elType.ElTypeNum == HTML_EL_SYMBOL_UNIT)
+     /* it's a basic element. It is interested whatever its namespace */
+     ok = TRUE;
+   else if (isHTML &&
+	    (elType.ElTypeNum == HTML_EL_LINK ||
+	     elType.ElTypeNum == HTML_EL_FRAME ||
+	     elType.ElTypeNum == HTML_EL_Anchor))
+     ok = TRUE;
+   else if (isXLink)
+     ok = TRUE;
+   
+   /* Search the anchor or LINK element */
+   if (!isXLink)
+     {
+       anchor = SearchAnchor (document, element, TRUE, FALSE);
+       if (anchor == NULL)
+	 {
+	   if (isHTML && (elType.ElTypeNum == HTML_EL_LINK ||
+			  elType.ElTypeNum == HTML_EL_FRAME))
+	     anchor = element;
+	   else
+	     {
+	       elType1.ElTypeNum = HTML_EL_LINK;
+	       elType1.ElSSchema = HTMLschema;
+	       anchor = TtaGetTypedAncestor (element, elType1);
+	     }
+	 }
+     }
+   else
+     anchor = NULL;
+   
+   /* if not found, search a cite or href attribute (from HTML or XLink
+      namespaces) on an ancestor */
+   if (anchor == NULL)
+     {
+       ancestor = element;
+       XLinkSchema = TtaGetSSchema (TEXT("XLink"), document);
+       do
+	 {
+	   attrType.AttrSSchema = HTMLschema;
+	   attrType.AttrTypeNum = HTML_ATTR_HREF_;
+	   attr = TtaGetAttribute (ancestor, attrType);
+	   if (!attr)
+	     {
+	       attrType.AttrTypeNum = HTML_ATTR_cite;
+	       attr = TtaGetAttribute (ancestor, attrType);
+	     }
+	   if (!attr)
+	     {
+	       attrType.AttrSSchema = XLinkSchema;
+	       attrType.AttrTypeNum = XLink_ATTR_href_;
+	       attr = TtaGetAttribute (ancestor, attrType);
+	     }
+	   if (attr)
+	     anchor = ancestor;
+	   else
+	     ancestor = TtaGetParent (ancestor);
+	 }
+       while (anchor == NULL && ancestor != NULL);
+     }
+   
+   /* Search the HREF attribute */
+   if (anchor != NULL)
+     {
+       elType = TtaGetElementType (anchor);
+       attrType.AttrSSchema = HTMLschema;
+       /* search the HREF or CITE attribute */
+       if (isHTML &&
+	   (elType.ElTypeNum == HTML_EL_Quotation ||
+	    elType.ElTypeNum == HTML_EL_Block_Quote ||
+	    elType.ElTypeNum == HTML_EL_INS ||
+	    elType.ElTypeNum == HTML_EL_DEL))
+	 attrType.AttrTypeNum = HTML_ATTR_cite;
+       else if (isHTML && elType.ElTypeNum == HTML_EL_FRAME)
+	 attrType.AttrTypeNum = HTML_ATTR_FrameSrc;
+       else
+	 if (isHTML)
+	   attrType.AttrTypeNum = HTML_ATTR_HREF_;
+	 else
+	   {
+	     attrType.AttrSSchema = TtaGetSSchema (TEXT("XLink"), document);
+	     attrType.AttrTypeNum = XLink_ATTR_href_;
+	   }
+       
+       HrefAttr = TtaGetAttribute (anchor, attrType);
+     }
+   
+   if (HrefAttr != NULL)
+     {
+       /* Get a buffer for the target URL */
+       length = TtaGetTextAttributeLength (HrefAttr);
+       length++;
+       url = TtaAllocString (length);
+       if (url != NULL)
+	 {
+	   /* Get the URL itself */
+	   TtaGiveTextAttributeValue (HrefAttr, url, &length);
+	   /* Normalize the URL */
+	   pathname = TtaAllocString (MAX_LENGTH);
+	   documentname = TtaAllocString (MAX_LENGTH);
+	   NormalizeURL (url, document, pathname, documentname, NULL);
+	   /* Display the URL in the status line */
+	   TtaSetStatus (document, 1, pathname, NULL);
+	   TtaFreeMemory (pathname);
+	   TtaFreeMemory (documentname);
+	   TtaFreeMemory (url);
+	 }
+     }
+}
+
 /*----------------------------------------------------------------------
   DoAction         
   ----------------------------------------------------------------------*/
@@ -1003,6 +1178,20 @@ View            view;
      ActivateElement (firstSel, doc);
 }
 
+/*----------------------------------------------------------------------
+  IgnoreEvent       An empty function to be able to ignore events.
+  ----------------------------------------------------------------------*/
+#ifdef __STDC__
+ThotBool            IgnoreEvent (NotifyElement *event)
+#else  /* __STDC__ */
+ThotBool            IgnoreEvent (event)
+NotifyElement      *event;
+#endif /* __STDC__ */
+{
+  /* don't let Thot perform it's normal operation */
+  return TRUE;
+}
+
 /*----------------------------------------------------------------------
   DoubleClick     The user has double-clicked an element.         
   ----------------------------------------------------------------------*/
@@ -1039,12 +1228,34 @@ NotifyElement      *event;
 
   TtaGetEnvBoolean ("ENABLE_DOUBLECLICK", &usedouble);  
   if (usedouble)
-    return TRUE;
+    {
+      DisplayUrlAnchor (event->element, event->document);
+      return TRUE;
+    }
   else
     /* don't let Thot perform normal operation */
     return (ActivateElement (event->element, event->document));
 }
 
+/*----------------------------------------------------------------------
+  AnnotSimpleClick     The user has clicked on an annotation icon
+  ----------------------------------------------------------------------*/
+#ifdef __STDC__
+ThotBool            AnnotSimpleClick (NotifyElement *event)
+#else  /* __STDC__ */
+ThotBool            AnnotSimpleClick (event)
+NotifyElement      *event;
+
+#endif /* __STDC__ */
+{
+#ifdef ANNOTATIONS
+  /* if it's an annotation link, highlight the annotated text  */
+  ANNOT_SelectSourceDoc (event->document, event->element);
+#endif /* ANNOTATIONS */
+  return SimpleClick (event);
+}
+
+
 /*----------------------------------------------------------------------
    UpdateTitle update the content of the Title field on top of the 
    main window, according to the contents of element el.   
@@ -1103,63 +1314,71 @@ Document       doc;
 
   TtaSetItemOff (doc, 1, Special, BShowLogFile);
   HTMLErrorsFound = FALSE;
+  XMLErrorsFound = FALSE;
 
   if (DocumentURLs[doc] != NULL)
     {
-      /* remove the temporary copy of the file */
-      tempdocument = GetLocalPath (doc, DocumentURLs[doc]);
-      TtaFileUnlink (tempdocument);
-      TtaFreeMemory (tempdocument);
-
-      if (IsHTTPPath (DocumentURLs[doc]))
+      if (DocumentTypes[doc] != docLog)
 	{
-	  usprintf (htmlErrFile, TEXT("%s%c%d%cHTML.ERR"), TempFileDirectory, DIR_SEP, doc, DIR_SEP);
+	  /* remove the temporary copy of the file */
+	  tempdocument = GetLocalPath (doc, DocumentURLs[doc]);
+	  TtaFileUnlink (tempdocument);
+	  TtaFreeMemory (tempdocument);
+	  /* remove the log file */
+	  usprintf (htmlErrFile, TEXT("%s%c%d%cHTML.ERR"),
+		    TempFileDirectory, DIR_SEP, doc, DIR_SEP);
 	  if (TtaFileExist (htmlErrFile))
 	    TtaFileUnlink (htmlErrFile);
+
+#ifdef ANNOTATIONS
+	  ANNOT_FreeDocumentResource (doc);
+#endif /* ANNOTATIONS */
+
+	  if (DocumentTypes[doc] == docImage)
+	    DocumentTypes[doc] = docHTML;
 	}
-
-      if (DocumentTypes[doc] == docImage)
-	DocumentTypes[doc] = docHTML;
-      else if (DocumentTypes[doc] == docImageRO)
-	DocumentTypes[doc] = docHTMLRO;
-
       TtaFreeMemory (DocumentURLs[doc]);
       DocumentURLs[doc] = NULL;
+      ReadOnlyDocument[doc] = FALSE;
       if (DocumentMeta[doc])
 	{
-        if (DocumentMeta[doc]->form_data)
-	  TtaFreeMemory (DocumentMeta[doc]->form_data);
-        TtaFreeMemory (DocumentMeta[doc]);
-        DocumentMeta[doc] = NULL;
+	  DocumentMetaClear (DocumentMeta[doc]);
+	  TtaFreeMemory (DocumentMeta[doc]);
+	  DocumentMeta[doc] = NULL;
 	}
-#ifdef ANNOTATIONS
-       LINK_DelMetaFromMemory (doc);
-#endif /* ANNOTATIONS */
       if (HighlightDocument == doc)
 	ResetHighlightedElement ();
-      if (DocumentSource[doc])
+      if (DocumentTypes[doc] != docLog)
 	{
-	  if (DocumentTypes[doc] != docLog)
+	  if (DocumentSource[doc])
 	    {
 	      sourceDoc = DocumentSource[doc];
 	      TtcCloseDocument (sourceDoc, 1);
 	      FreeDocumentResource (sourceDoc);
+	      DocumentSource[doc] = 0;
 	    }
-	  DocumentSource[doc] = 0;
+	  /* is this document the source of another document? */
+	  for (i = 1; i < DocumentTableLength; i++)
+	    if (DocumentURLs[i] != NULL)
+	      if (DocumentSource[i] == doc)
+		{
+		  DocumentSource[i] = 0;
+		  if (DocumentTypes[i] == docLog)
+		    {
+		      /* close the window of the log file attached to the
+			 current document */
+		      TtaCloseDocument (i);
+		      TtaFreeMemory (DocumentURLs[i]);
+		      DocumentURLs[i] = NULL;
+		      /* switch off the button Show Log file */
+		      TtaSetItemOff (doc, 1, Special, BShowLogFile);
+		    }
+		}
+	  RemoveDocCSSs (doc);
+	  /* avoid to free images of backup documents */
+	  if (BackupDocument != doc)
+	    RemoveDocumentImages (doc);
 	}
-      /* is this document the source of another document? */
-      for (i = 1; i < DocumentTableLength; i++)
-         if (DocumentURLs[i] != NULL)
-	    if (DocumentSource[i] == doc)
-	      {
-		DocumentSource[i] = 0;
-		if (DocumentTypes[i] == docLog)
-		  TtaCloseDocument (i);
-	      }
-      RemoveDocCSSs (doc);
-      /* avoid to free images of backup documents */
-      if (BackupDocument != doc)
-	RemoveDocumentImages (doc);
     }
 }
  
@@ -1757,7 +1976,7 @@ NotifyElement* event;
    Attribute	       attr;
    Document	       doc, otherDoc;
    int		       val, x, y, width, height;
-   ThotBool	       otherDocIsHTML, done;
+   ThotBool	       otherDocIsStruct, done;
 
    if (!event)
        return;
@@ -1766,21 +1985,23 @@ NotifyElement* event;
    /* get the other Thot document to be synchronized with the one where the
       user has just clicked */
    otherDoc = 0;
-   otherDocIsHTML = FALSE;
-   if (DocumentTypes[doc] == docHTML || DocumentTypes[doc] == docHTMLRO)
-      /* the user clicked on a HTML document, the other doc is the
+   otherDocIsStruct = FALSE;
+   if (DocumentTypes[doc] == docHTML ||
+       DocumentTypes[doc] == docMath ||
+       DocumentTypes[doc] == docSVG )
+      /* the user clicked on a structured document, the other doc is the
          corresponding source document */
       otherDoc = DocumentSource[doc];
-   else if (DocumentTypes[doc] == docSource ||
-	    DocumentTypes[doc] == docSourceRO)
+   else if (DocumentTypes[doc] == docSource)
       /* the user clicked on a source document, the other doc is the
-         corresponding HTML document */
+         corresponding structured document */
       {
-      otherDocIsHTML = TRUE;
+      otherDocIsStruct = TRUE;
       for (i = 1; i < DocumentTableLength; i++)
          if (DocumentURLs[i] != NULL)
 	    if (DocumentTypes[i] == docHTML ||
-		DocumentTypes[i] == docHTMLRO)
+		DocumentTypes[i] == docMath ||
+		DocumentTypes[i] == docSVG)
 	       if (DocumentSource[i] == doc)
 		  {
 	          otherDoc = i;
@@ -1832,11 +2053,35 @@ NotifyElement* event;
 	       attribute */
 	    ResetHighlightedElement ();
 	    /* Put a Highlight attribute on the element found */
-	    if (otherDocIsHTML)
+	    if (otherDocIsStruct)
 	       {
-	       attrType.AttrSSchema = TtaGetSSchema (TEXT("HTML"), otherDoc);
-	       attrType.AttrTypeNum = HTML_ATTR_Highlight;
-	       val = HTML_ATTR_Highlight_VAL_Yes_;
+	       if (DocumentTypes[otherDoc] == docHTML)
+		 {
+		   attrType.AttrSSchema = TtaGetSSchema (TEXT("HTML"),
+							 otherDoc);
+		   attrType.AttrTypeNum = HTML_ATTR_Highlight;
+		   val = HTML_ATTR_Highlight_VAL_Yes_;
+		 }
+	       else if (DocumentTypes[otherDoc] == docMath)
+		 {
+		   attrType.AttrSSchema = TtaGetSSchema (TEXT("MathML"),
+							 otherDoc);
+		   attrType.AttrTypeNum = MathML_ATTR_Highlight;
+		   val = MathML_ATTR_Highlight_VAL_Yes_;
+		 }
+	       else if (DocumentTypes[otherDoc] == docSVG)
+		 {
+		   attrType.AttrSSchema = TtaGetSSchema (TEXT("GraphML"),
+							 otherDoc);
+		   attrType.AttrTypeNum = GraphML_ATTR_Highlight;
+		   val = GraphML_ATTR_Highlight_VAL_Yes_;
+		 }
+	       else
+		 {
+		   attrType.AttrSSchema = NULL;
+		   attrType.AttrTypeNum = 0;
+		   val = 0;
+		 }
 	       }
 	    else
 	       {
@@ -1844,22 +2089,27 @@ NotifyElement* event;
 	       attrType.AttrTypeNum = TextFile_ATTR_Highlight;
 	       val = TextFile_ATTR_Highlight_VAL_Yes_;
 	       }
-	    attr = TtaNewAttribute (attrType);
-	    TtaAttachAttribute (otherEl, attr, otherDoc);
-	    TtaSetAttributeValue (attr, val, otherEl, otherDoc);
-	    /* record the highlighted element */
-	    HighlightDocument = otherDoc;
-	    HighlightElement = otherEl;
-	    HighLightAttribute = attr;
-	    /* Scroll all views where the element appears to show it */
-	    for (view = 1; view < 6; view++)
-	       if (TtaIsViewOpened (otherDoc, view))
-		  {
-		  TtaGiveBoxAbsPosition (otherEl, otherDoc, view, UnPixel, &x, &y);
-		  TtaGiveWindowSize (otherDoc, view, UnPixel, &width, &height);
-		  if (y < 0 || y > height - 15)
-		     TtaShowElement (otherDoc, view, otherEl, 25);
-		  }
+	    if (attrType.AttrSSchema)
+	      {
+		attr = TtaNewAttribute (attrType);
+		TtaAttachAttribute (otherEl, attr, otherDoc);
+		TtaSetAttributeValue (attr, val, otherEl, otherDoc);
+		/* record the highlighted element */
+		HighlightDocument = otherDoc;
+		HighlightElement = otherEl;
+		HighLightAttribute = attr;
+		/* Scroll all views where the element appears to show it */
+		for (view = 1; view < 6; view++)
+		  if (TtaIsViewOpened (otherDoc, view))
+		    {
+		      TtaGiveBoxAbsPosition (otherEl, otherDoc, view, UnPixel,
+					     &x, &y);
+		      TtaGiveWindowSize (otherDoc, view, UnPixel, &width,
+					 &height);
+		      if (y < 0 || y > height - 15)
+			 TtaShowElement (otherDoc, view, otherEl, 25);
+		    }
+	      }
 	    }
 	 done = TRUE;
 	 }
