@@ -32,6 +32,7 @@ static int          OldHeight;
 #include "css_f.h"
 #include "EDITimage_f.h"
 #include "EDITORactions_f.h"
+#include "fetchXMLname_f.h"
 #ifdef GRAPHML
 #include "GraphMLbuilder_f.h"
 #endif
@@ -47,6 +48,7 @@ static int          OldHeight;
 #include "styleparser_f.h"
 #include "tree.h"
 #include "XLinkedit_f.h"
+#include "fetchHTMLname_f.h"
 
 #ifdef _WINDOWS
 extern HWND currentWindow;
@@ -255,10 +257,12 @@ STRING              targetName;
        /* create a XLink link */
        attrType.AttrSSchema = TtaGetSSchema (TEXT("XLink"), doc);
        if (!attrType.AttrSSchema)
-	  attrType.AttrSSchema = TtaNewNature (TtaGetDocumentSSchema (doc),
-					       TEXT("XLink"), TEXT("XLinkP"));
+	 attrType.AttrSSchema = TtaNewNature (doc, TtaGetDocumentSSchema (doc),
+					      TEXT("XLink"), TEXT("XLinkP"));
        attrType.AttrTypeNum = XLink_ATTR_href_;
        /* create a xlink:type attribute with value "simple" */
+       if (TtaIsLeaf (elType))
+	 element = TtaGetParent (element);
        SetXLinkTypeSimple (element, doc, AttrHREFundoable);
      }
    attr = TtaGetAttribute (element, attrType);
@@ -488,7 +492,11 @@ ThotBool	    withUndo;
    AttributeType       attrType;
    STRING              buffer = NULL;
    int                 length;
+#ifndef _WINDOWS
+   int                 i;
+#endif
    ThotBool            isHTML;
+   CHAR_T              s[MAX_LENGTH];
 
    /* ask the user to select target document and target anchor */
    TtaSetStatus (doc, 1, TtaGetMessage (AMAYA, AM_SEL_TARGET), NULL);
@@ -552,6 +560,7 @@ ThotBool	    withUndo;
 	    attrType.AttrTypeNum = XLink_ATTR_href_;
 	  }
 	attr = TtaGetAttribute (el, attrType);
+	AttrHREFvalue[0] = WC_EOS;
 	if (attr != 0)
 	  {
 	     /* get a buffer for the attribute value */
@@ -564,17 +573,32 @@ ThotBool	    withUndo;
 	  }
 
 #ifndef _WINDOWS
-	TtaNewForm (BaseDialog + AttrHREFForm, TtaGetViewFrame (doc, 1),
-		    TtaGetMessage (AMAYA, AM_ATTRIBUTE), TRUE, 2, 'L',
-		    D_CANCEL);
+	/* Dialogue form for open URL or local */
+	i = 0;
+	ustrcpy (&s[i], TtaGetMessage (LIB, TMSG_LIB_CONFIRM));
+	i += ustrlen (&s[i]) + 1;
+	ustrcpy (&s[i], TtaGetMessage (AMAYA, AM_BROWSE));
+	
+	TtaNewSheet (BaseDialog + AttrHREFForm, TtaGetViewFrame (doc, 1),
+		     TtaGetMessage (AMAYA, AM_ATTRIBUTE), 2, s,
+		     TRUE, 2, 'L', D_CANCEL);
 	TtaNewTextForm (BaseDialog + AttrHREFText, BaseDialog + AttrHREFForm,
-			TtaGetMessage (AMAYA, AM_HREF_VALUE), 50, 1, FALSE);
+			TtaGetMessage (AMAYA, AM_LOCATION), 50, 1, TRUE);
+	TtaNewLabel (BaseDialog + HREFLocalName,
+		     BaseDialog + AttrHREFForm, " ");
 	/* initialise the text field in the dialogue box */
 	TtaSetTextForm (BaseDialog + AttrHREFText, AttrHREFvalue);
+	ustrcpy (s, DirectoryName);
+	ustrcat (s, DIR_STR);
+	ustrcat (s, DocumentName);
 	TtaSetDialoguePosition ();
-	TtaShowDialogue (BaseDialog + AttrHREFForm, FALSE);
+	TtaShowDialogue (BaseDialog + AttrHREFForm, TRUE);
 #else  /* _WINDOWS */
-	CreateTextDlgWindow (currentWindow, AttrHREFvalue);
+    if (AttrHREFvalue[0] != WC_EOS)
+       usprintf (s, TEXT("%s"), AttrHREFvalue);
+    else
+      usprintf (s, TEXT("%s%c%s"), DirectoryName, DIR_SEP, DocumentName);
+    CreateHRefDlgWindow (currentWindow, s, DocSelect, DirSelect, 2);
 #endif  /* _WINDOWS */
      }
 }
@@ -654,19 +678,21 @@ Element             selectedElement;
 }
 
 
-
 /*----------------------------------------------------------------------
    CreateTargetAnchor creates a NAME or ID attribute with a default    
-   value for element el.                           
+   value for element el.
+   If the withUndo parameter is true, we'll register the undo sequence.
+   If the forceID parameter, we'll always use an ID attribute, rather
+   than a NAME one in some cases.
   ----------------------------------------------------------------------*/
 #ifdef __STDC__
-void                CreateTargetAnchor (Document doc, Element el, ThotBool withUndo)
+void                CreateTargetAnchor (Document doc, Element el, ThotBool forceID, ThotBool withUndo)
 #else  /* __STDC__ */
-void                CreateTargetAnchor (doc, el, withUndo)
+void                CreateTargetAnchor (doc, el, forceID, withUndo)
 Document            doc;
 Element             el;
-Boolean		    withUndo;
-
+ThotBool            forceID;
+ThotBool	    withUndo;
 #endif /* __STDC__ */
 {
    AttributeType       attrType;
@@ -676,7 +702,7 @@ Boolean		    withUndo;
    SSchema	       HTMLSSchema;
    Language            lang;
    STRING              text;
-   STRING               url = TtaAllocString (MAX_LENGTH);
+   STRING              url = TtaAllocString (MAX_LENGTH);
    int                 length, i, space;
    ThotBool            found;
    ThotBool            withinHTML, new;
@@ -689,7 +715,12 @@ Boolean		    withUndo;
    attrType.AttrSSchema = HTMLSSchema;
    if (withinHTML && (elType.ElTypeNum == HTML_EL_Anchor ||
 		      elType.ElTypeNum == HTML_EL_MAP))
-     attrType.AttrTypeNum = HTML_ATTR_NAME;
+     {
+       if (forceID)
+	 attrType.AttrTypeNum = HTML_ATTR_ID;
+       else
+	 attrType.AttrTypeNum = HTML_ATTR_NAME;
+     }
    else
      {
      if (ustrcmp(TtaGetSSchemaName (elType.ElSSchema), TEXT("MathML")) == 0)
@@ -710,7 +741,7 @@ Boolean		    withUndo;
      }
    attr = TtaGetAttribute (el, attrType);
 
-   if (attr == 0)
+   if (attr == NULL)
      {
 	attr = TtaNewAttribute (attrType);
 	TtaAttachAttribute (el, attr, doc);
@@ -750,9 +781,9 @@ Boolean		    withUndo;
 	    url[length++] = EOS;
 	    while (!found && i < length)
 	      {
-		if (url[i] == ' ' || url[i] == EOS)
+		if (url[i] == TEXT(' ') || url[i] == WC_EOS)
 		  {
-		    found = (i - space > 3 || (i != space && url[i] == EOS));
+		    found = (i - space > 3 || (i != space && url[i] == WC_EOS));
 		    if (found)
 		      {
 			/* url = the word */
@@ -766,8 +797,19 @@ Boolean		    withUndo;
 		    i++;
 		    space = i;
 		  }
-		else
+		else if (url[i] == TEXT('_') ||
+			 url[i] == TEXT(':') ||
+			 ((unsigned int) url[i] >= 65 &&
+			  (unsigned int) url[i] <= 90) ||
+			 ((unsigned int) url[i] >= 81 &&
+			  (unsigned int) url[i] <= 127) ||
+			 (i > 0 && (unsigned int) url[i] >= 48 &&
+			  (unsigned int) url[i] <= 57))
+		  /* valid character for an ID */
 		  i++;
+		else
+		  /* invalid name for an ID */
+		  i = length;
 	      }
 
 	    if (!found)
@@ -786,6 +828,7 @@ Boolean		    withUndo;
      }
    /* copie the text into the NAME attribute */
    TtaSetAttributeText (attr, url, el, doc);
+   TtaSetDocumentModified (doc);
    /* Check the attribute value to make sure that it's unique within */
    /* the document */
    MakeUniqueName (el, doc);
@@ -920,7 +963,7 @@ ThotBool            createLink;
 		  /* create an ID for target element */
 		{
 		  TtaOpenUndoSequence (doc, first, last, c1, cN);
-		  CreateTargetAnchor (doc, first, TRUE);
+		  CreateTargetAnchor (doc, first, FALSE, TRUE);
 		  TtaCloseUndoSequence (doc);
 		}
 	      return;
@@ -1064,7 +1107,7 @@ ThotBool            createLink;
       TtaSetAttributeText (attr, TEXT("link"), anchor, doc);
     }
   else
-    CreateTargetAnchor (doc, anchor, FALSE);
+    CreateTargetAnchor (doc, anchor, FALSE, FALSE);
 
   if (parag)
     TtaRegisterElementCreate (parag, doc);
@@ -1095,7 +1138,7 @@ Document     doc;
   SSchema	    HTMLSSchema;
   Attribute         attr;
   STRING            value;
-  CHAR_T              url[MAX_LENGTH];
+  CHAR_T            url[MAX_LENGTH];
   int               length, i;
   ThotBool          change, isHTML;
 
@@ -1125,7 +1168,7 @@ Document     doc;
        attrType.AttrTypeNum = HTML_ATTR_ID;
     }
   attr = TtaGetAttribute (el, attrType);
-  
+
   if (attr != 0)
     {
       /* the element has an attribute NAME or ID */
@@ -1177,6 +1220,218 @@ Document     doc;
     }
 }
 
+/*----------------------------------------------------------------------
+  GetNextNode
+  Returns the next node in the tree, using a complete traversal algorithm.
+  ----------------------------------------------------------------------*/
+#ifdef __STDC__
+static Element    GetNextNode (Element curr)
+#else  /* __STDC__ */
+static Element    GetNExtNode (curr)
+Element curr;
+Element last;
+
+#endif /* __STDC__ */
+{
+  Element el;
+
+  if (!curr)
+    return NULL;
+
+  /* get the next child */
+  el = TtaGetFirstChild (curr);
+  if (!el)
+    {
+      /* get the next siblign, or parent
+	 if there was no other sibling */
+      el = TtaGetSuccessor (curr);
+    }
+  return el;
+}
+
+/*----------------------------------------------------------------------
+  SearchTypedElementForward
+  Searchs for a typed element and stops when it finds it or if the
+  search reaches the last element.
+  ----------------------------------------------------------------------*/
+#ifdef __STDC__
+static Element    SearchTypedElementForward (ElementType elType_search, Element curr, Element last)
+#else  /* __STDC__ */
+static Element    SearchTypedElementForward (elType_search, curr, last)
+ElementType elType_search;
+Element curr;
+Element last;
+
+#endif /* __STDC__ */
+{
+  ElementType elType;
+  Element el;
+
+  /* start by getting the next node */
+  el = GetNextNode (curr);
+  /*  continue browsing until we get an element
+     of the searched type or the end condition is
+     reached */
+  while (el && el != last)
+    {
+      elType = TtaGetElementType (el);
+      if (TtaSameTypes (elType_search, elType))
+	  break;
+      el = GetNextNode (el);
+    }
+
+  /* don't return el if it's equal to the last element parameter */
+  return ((el != last) ? el : NULL);
+}
+
+/*----------------------------------------------------------------------
+  CreateRemoveIDAttribute
+  For all elements elName of a document, this functions eithers adds or 
+  deletes an ID attribute. 
+  The createID flag tells which operation must be done.
+  The inSelection attribute says if we must apply the operation in the
+  whole document or just in the current selection.
+  If an element already has an ID attribute, a new one won't be created.
+  TO DO: Use the thotmsg functions for the dialogs.
+  ----------------------------------------------------------------------*/
+#ifdef __STDC__
+void         CreateRemoveIDAttribute (CHAR_T *elName, Document doc, ThotBool createID, ThotBool inSelection)
+#else  /* __STDC__ */
+void         CreateRemoveIDdAttribute (elName, doc, createID, inSelection)
+CHAR_T         *elNamel
+Document	doc;
+int             createID;
+ThotBool        inSelection;
+
+#endif /* __STDC__ */
+{
+  Element             el, lastEl;
+  ElementType         elType;
+  AttributeType       attrType;
+  Attribute           attr;
+  CHAR_T             *schema_name;
+  DisplayMode         dispMode;
+  ThotBool            closeUndo;
+  int                 i, j;
+
+  /* the user must select something */
+  if (inSelection && !TtaIsDocumentSelected (doc))
+    {
+      ustrcpy (IdStatus, TEXT("Nothing selected"));
+      return;
+    }
+  /* search for the elementType corresponding to the element name given
+   by the user */
+  GIType (elName, &elType, doc);
+  if (elType.ElTypeNum == 0)
+    {
+      /* element name not found */
+      ustrcpy (IdStatus, TEXT("Unknown element"));
+      return;
+    }
+  /* in function of the target elType, we choose the correct
+     ATTR_ID value and schema */
+  attrType.AttrTypeNum = 0;
+  schema_name = TtaGetSSchemaName (elType.ElSSchema);
+  if (!ustrcmp (schema_name, TEXT("HTML")))
+    {
+      /* exception handling... we can't add an ID attribute everywhere
+       in HTML documents */
+      if (!(elType.ElTypeNum == HTML_EL_HTML
+	    || elType.ElTypeNum == HTML_EL_HEAD
+	    || elType.ElTypeNum == HTML_EL_TITLE
+	    || elType.ElTypeNum == HTML_EL_BASE
+	    || elType.ElTypeNum == HTML_EL_META
+	    || elType.ElTypeNum == HTML_EL_SCRIPT
+	    || elType.ElTypeNum == HTML_EL_STYLE_))
+	attrType.AttrTypeNum = HTML_ATTR_ID;
+    }
+  else if (!ustrcmp (schema_name, TEXT("MathML")))
+    attrType.AttrTypeNum = MathML_ATTR_id;
+  else if (!ustrcmp (schema_name, TEXT("GraphML")))
+    attrType.AttrTypeNum = GraphML_ATTR_id;
+
+  /* we didn't find an attribute or we can't put an ID attribute
+     in this element */
+  if (attrType.AttrTypeNum == 0)
+    {
+      ustrcpy (IdStatus, TEXT("DTD forbids it"));
+      return;
+    }
+  attrType.AttrSSchema = elType.ElSSchema;
+
+  /* prepare the environment before doing the operation */
+  dispMode = TtaGetDisplayMode (doc);
+  if (dispMode == DisplayImmediately)
+    TtaSetDisplayMode (doc, DeferredDisplay);
+
+  /* set the first and last elements for the search */
+  if (inSelection)
+    {
+      /* get the first and last elements of the selection */
+      TtaGiveFirstSelectedElement (doc, &el, &i, &j);
+       if (TtaIsSelectionEmpty ())
+	 lastEl = el;
+       else
+	 TtaGiveLastSelectedElement (doc, &lastEl, &j, &i);
+       /* and set the last element as the element just after
+	  the selection */
+       lastEl = TtaGetSuccessor (lastEl);
+    }
+  else
+    {
+      el = TtaGetMainRoot (doc);
+      lastEl = NULL;
+    }
+
+  /*
+  ** browse the tree and add the ID if it's missing. Variable i
+  ** stores the number of changes we have done.
+  */
+
+  /* move to the first element that is of the chosen elementType */
+  /* the element where we started is not of the chosen type */
+  if (!TtaSameTypes (TtaGetElementType (el), elType))
+    el = SearchTypedElementForward (elType, el, lastEl);
+
+  if (TtaPrepareUndo (doc))
+      closeUndo = FALSE;
+  else
+    {
+      closeUndo = TRUE;
+      TtaOpenUndoSequence (doc, NULL, NULL, 0, 0);
+    }
+  i = 0;
+  while (el)
+    {
+      /* does the element have an ID attribute already? */
+      attr = TtaGetAttribute (el, attrType);
+      if (!attr && createID) /* add it */
+	{
+	  /* we reuse an existing Amaya function */
+	  CreateTargetAnchor (doc, el, TRUE, TRUE);
+	  i++;
+	}
+      else if (attr && !createID) /* delete it */
+	{
+	  TtaRegisterAttributeDelete (attr, el, doc);
+	  TtaRemoveAttribute (el, attr, doc);
+	  i++;
+	}
+      /* get the next element */
+      el = SearchTypedElementForward (elType, el, lastEl);
+    }
+
+  if (closeUndo)
+    TtaCloseUndoSequence (doc);
+
+  /* reset the state of the document */
+  if (i)
+    TtaSetDocumentModified (doc);
+  if (dispMode == DisplayImmediately)
+    TtaSetDisplayMode (doc, dispMode);
+  usprintf (IdStatus, TEXT("%d elements changed"), i);
+}
 
 /*----------------------------------------------------------------------
    CheckPseudoParagraph
@@ -1351,7 +1606,6 @@ void ElementDeleted(event)
         }
 }
 
-
 /*----------------------------------------------------------------------
    ChangeURI
    Element el has been pasted in document doc. It comes from document
@@ -1360,8 +1614,8 @@ void ElementDeleted(event)
    context.
   ----------------------------------------------------------------------*/
 #ifdef __STDC__
-void ChangeURI (Element el, Attribute attr, Document originDocument,
-		Document doc)
+void    ChangeURI (Element el, Attribute attr,
+		   Document originDocument, Document doc)
 #else  /* __STDC__ */
 void    ChangeURI (el, attr, originDocument, doc)
 Element el;
@@ -1382,15 +1636,13 @@ Document doc;
     tempURI = TtaAllocString (MAX_LENGTH);
     if (tempURI)
       {
+      iName = 0;
       /* get the URI itself */
       TtaGiveTextAttributeValue (attr, value, &length);
       if (value[0] == '#')
-	{
 	  /* the target is in the original document */
 	  /* convert the internal link into an external link */
 	  ustrcpy (tempURI, DocumentURLs[originDocument]);
-	  iName = 0;
-	}
       else
 	{
 	  /* the target element is in another document */
@@ -1403,11 +1655,8 @@ Document doc;
 	    while (value[i] != '#' && i > 0)
 	      i--;
 	    if (i == 0)
-	      {
 		/* there is no '#' in the URI */
 		value[0] = EOS;
-		iName = 0;
-	      }
 	    else
 	      {
 		/* there is a '#' character in the URI */
@@ -2138,79 +2387,88 @@ NotifyAttribute    *event;
 {
    ElementType         elType;
    SSchema	       HTMLSSchema;
+   STRING              attr;
+
+   elType = TtaGetElementType (event->element);
+   attr = GetXMLAttributeName (event->attributeType, elType, event->document);
+   if (attr[0] == WC_EOS)
+      return TRUE;	/* don't put an invalid attribute in the menu */
 
    HTMLSSchema = TtaGetSSchema (TEXT("HTML"), event->document);
-   elType = TtaGetElementType (event->element);
-   if (ustrcmp(TtaGetSSchemaName (elType.ElSSchema), TEXT("HTML")))
-     /* it's not a HTML document */
-      return TRUE;	/* don't put any HTML attribute in the menu */
+   if (TtaSameSSchemas (elType.ElSSchema, HTMLSSchema))
+     {
+      /* it's a HTML element */
 
-   if (!TtaSameSSchemas (elType.ElSSchema, HTMLSSchema))
-      /* it's not a HTML element */
-      return TRUE;	/* don't put any HTML attribute in the menu */
-
-   /* BASE and SCRIPT do not accept any global attribute */
-   if (elType.ElTypeNum == HTML_EL_BASE ||
-       elType.ElTypeNum == HTML_EL_SCRIPT)
-      return TRUE;
-
-   /* BASEFONT and PARAM accept only ID */
-   if (elType.ElTypeNum == HTML_EL_BaseFont ||
-       elType.ElTypeNum == HTML_EL_Parameter)
-      return (event->attributeType.AttrTypeNum != HTML_ATTR_ID);
-
-   /* coreattrs */
-   if (event->attributeType.AttrTypeNum == HTML_ATTR_ID ||
-       event->attributeType.AttrTypeNum == HTML_ATTR_Class ||
-       event->attributeType.AttrTypeNum == HTML_ATTR_Style_ ||
-       event->attributeType.AttrTypeNum == HTML_ATTR_Title)
-      if (elType.ElTypeNum == HTML_EL_HEAD ||
-	  elType.ElTypeNum == HTML_EL_TITLE ||
-	  elType.ElTypeNum == HTML_EL_META ||
-	  elType.ElTypeNum == HTML_EL_STYLE_ ||
-	  elType.ElTypeNum == HTML_EL_HTML)
+       /* BASE and SCRIPT do not accept any global attribute */
+       if (elType.ElTypeNum == HTML_EL_BASE ||
+	   elType.ElTypeNum == HTML_EL_SCRIPT)
 	 return TRUE;
-      else
-	 return FALSE;
-   /* i18n */
-   if (event->attributeType.AttrTypeNum == HTML_ATTR_dir ||
-       event->attributeType.AttrTypeNum == HTML_ATTR_Langue)
-     if (elType.ElTypeNum == HTML_EL_BR ||
-	 elType.ElTypeNum == HTML_EL_Applet ||
-	 elType.ElTypeNum == HTML_EL_Horizontal_Rule ||
-	 elType.ElTypeNum == HTML_EL_FRAMESET ||
-	 elType.ElTypeNum == HTML_EL_FRAME ||
-	 elType.ElTypeNum == HTML_EL_IFRAME)
-	return TRUE;
-     else
-	return FALSE;
-   /* events */
-   if (event->attributeType.AttrTypeNum == HTML_ATTR_onclick ||
-       event->attributeType.AttrTypeNum == HTML_ATTR_ondblclick ||
-       event->attributeType.AttrTypeNum == HTML_ATTR_onmousedown ||
-       event->attributeType.AttrTypeNum == HTML_ATTR_onmouseup ||
-       event->attributeType.AttrTypeNum == HTML_ATTR_onmouseover ||
-       event->attributeType.AttrTypeNum == HTML_ATTR_onmousemove ||
-       event->attributeType.AttrTypeNum == HTML_ATTR_onmouseout ||
-       event->attributeType.AttrTypeNum == HTML_ATTR_onkeypress ||
-       event->attributeType.AttrTypeNum == HTML_ATTR_onkeydown ||
-       event->attributeType.AttrTypeNum == HTML_ATTR_onkeyup)
-     if (elType.ElTypeNum == HTML_EL_BDO ||
-	 elType.ElTypeNum == HTML_EL_Font_ ||
-	 elType.ElTypeNum == HTML_EL_BR ||
-	 elType.ElTypeNum == HTML_EL_Applet ||
-	 elType.ElTypeNum == HTML_EL_FRAMESET ||
-	 elType.ElTypeNum == HTML_EL_FRAME ||
-	 elType.ElTypeNum == HTML_EL_IFRAME ||
-	 elType.ElTypeNum == HTML_EL_HEAD ||
-	 elType.ElTypeNum == HTML_EL_TITLE ||
-	 elType.ElTypeNum == HTML_EL_META ||
-	 elType.ElTypeNum == HTML_EL_STYLE_ ||
-	 elType.ElTypeNum == HTML_EL_HTML ||
-	 elType.ElTypeNum == HTML_EL_ISINDEX)
-	return TRUE;
 
-   return FALSE;
+       /* BASEFONT and PARAM accept only ID */
+       if (elType.ElTypeNum == HTML_EL_BaseFont ||
+	   elType.ElTypeNum == HTML_EL_Parameter)
+	 return (event->attributeType.AttrTypeNum != HTML_ATTR_ID);
+
+       /* coreattrs */
+       if (event->attributeType.AttrTypeNum == HTML_ATTR_ID ||
+	   event->attributeType.AttrTypeNum == HTML_ATTR_Class ||
+	   event->attributeType.AttrTypeNum == HTML_ATTR_Style_ ||
+	   event->attributeType.AttrTypeNum == HTML_ATTR_Title)
+	 if (elType.ElTypeNum == HTML_EL_HEAD ||
+	     elType.ElTypeNum == HTML_EL_TITLE ||
+	     elType.ElTypeNum == HTML_EL_META ||
+	     elType.ElTypeNum == HTML_EL_STYLE_ ||
+	     elType.ElTypeNum == HTML_EL_HTML)
+	   return TRUE;
+	 else
+	   return FALSE; /* let Thot perform normal operation */
+       /* i18n */
+       if (event->attributeType.AttrTypeNum == HTML_ATTR_dir ||
+	   event->attributeType.AttrTypeNum == HTML_ATTR_Langue)
+	 if (elType.ElTypeNum == HTML_EL_BR ||
+	     elType.ElTypeNum == HTML_EL_Applet ||
+	     elType.ElTypeNum == HTML_EL_Horizontal_Rule ||
+	     elType.ElTypeNum == HTML_EL_FRAMESET ||
+	     elType.ElTypeNum == HTML_EL_FRAME ||
+	     elType.ElTypeNum == HTML_EL_IFRAME)
+	   return TRUE;
+	 else
+	   return FALSE;
+       /* events */
+       if (event->attributeType.AttrTypeNum == HTML_ATTR_onclick ||
+	   event->attributeType.AttrTypeNum == HTML_ATTR_ondblclick ||
+	   event->attributeType.AttrTypeNum == HTML_ATTR_onmousedown ||
+	   event->attributeType.AttrTypeNum == HTML_ATTR_onmouseup ||
+	   event->attributeType.AttrTypeNum == HTML_ATTR_onmouseover ||
+	   event->attributeType.AttrTypeNum == HTML_ATTR_onmousemove ||
+	   event->attributeType.AttrTypeNum == HTML_ATTR_onmouseout ||
+	   event->attributeType.AttrTypeNum == HTML_ATTR_onkeypress ||
+	   event->attributeType.AttrTypeNum == HTML_ATTR_onkeydown ||
+	   event->attributeType.AttrTypeNum == HTML_ATTR_onkeyup)
+	 if (elType.ElTypeNum == HTML_EL_BDO ||
+	     elType.ElTypeNum == HTML_EL_Font_ ||
+	     elType.ElTypeNum == HTML_EL_BR ||
+	     elType.ElTypeNum == HTML_EL_Applet ||
+	     elType.ElTypeNum == HTML_EL_FRAMESET ||
+	     elType.ElTypeNum == HTML_EL_FRAME ||
+	     elType.ElTypeNum == HTML_EL_IFRAME ||
+	     elType.ElTypeNum == HTML_EL_HEAD ||
+	     elType.ElTypeNum == HTML_EL_TITLE ||
+	     elType.ElTypeNum == HTML_EL_META ||
+	     elType.ElTypeNum == HTML_EL_STYLE_ ||
+	     elType.ElTypeNum == HTML_EL_HTML ||
+	     elType.ElTypeNum == HTML_EL_ISINDEX)
+	   return TRUE;
+       /* xml:space attribute */
+       if (event->attributeType.AttrTypeNum == HTML_ATTR_xml_space)
+	 /* be careful here -- document may not have been received yet */
+	 if (DocumentMeta[event->document] == NULL
+	     || DocumentMeta[event->document]->xmlformat == FALSE)
+	   return TRUE;	 
+
+       return FALSE;
+     }
+   return TRUE;	/* don't put an invalid attribute in the menu */
 }
 
 /*----------------------------------------------------------------------
