@@ -8,7 +8,7 @@
 /*
  * Amaya saving functions.
  *
- * Authors: D. Veillard, I. Vatton
+ * Authors: I. Vatton, D. Veillard
  *          R. Guetari: Unicode and Windows version
  *
  */
@@ -53,6 +53,29 @@ static int          SRC_attr_tab[] = {
 };
 static STRING       QuotedText;
 
+
+static CHAR_T *HTMLDocTypes_1[] =
+{
+  TEXT("\"-//W3C//DTD XHTML Basic 1.0//EN\""),
+  TEXT("\"-//W3C//DTD XHTML 1.0 Strict//EN\""),
+  TEXT("\"-//W3C//DTD XHTML 1.0 Transitional//EN\""),
+  TEXT("\"-//W3C//DTD XHTML 1.0 Frameset//EN\""),
+  TEXT("\"-//W3C//DTD HTML 4.01//EN\""),
+  TEXT("\"-//W3C//DTD HTML 4.01 Transitional//EN\""),
+  TEXT("\"-//W3C//DTD HTML 4.01 Frameset//EN\"")
+};
+
+static CHAR_T *HTMLDocTypes_2[] =
+{
+  TEXT("\"http://www.w3.org/TR/xhtml-basic/xhtml-basic10.dtd\">\n"),
+  TEXT("\"DTD/xhtml1-1-strict.dtd\">\n"),
+  TEXT("\"DTD/xhtml1-1-transitional.dtd\">\n"),
+  TEXT("\'DTD/xhtml1-1-frameset.dtd\">\n"),
+  TEXT("\"http://www.w3.org/TR/REC-html401/strict.dtd\">\n"),
+  TEXT("\"http://www.w3.org/TR/REC-html401/loose.dtd\">\n"),
+  TEXT("\"http://www.w3.org/TR/REC-html401/frameset.dtd\">\n"),
+};
+
 #include "AHTURLTools_f.h"
 #include "EDITimage_f.h"
 #include "EDITstyle_f.h"
@@ -64,6 +87,7 @@ static STRING       QuotedText;
 #include "init_f.h"
 #include "query_f.h"
 #include "styleparser_f.h"
+#include "Xml2thot_f.h"
 
 #ifdef _WINDOWS
 #include "wininclude.h"
@@ -261,6 +285,69 @@ NotifyAttribute    *event;
   TtaFreeMemory (QuotedText);
 }
 
+/*----------------------------------------------------------------------
+  CheckValidID
+  A NAME attribute is about to be saved. If the output format is XML
+  and the current element does not have an ID attribute, check if
+  the value of the NAME attribute is a valid XML ID and if not,
+  generate an ID attribute with a valid value.
+  ----------------------------------------------------------------------*/
+#ifdef __STDC__
+ThotBool            CheckValidID (NotifyAttribute * event)
+#else  /* __STDC__ */
+ThotBool            CheckValidID (event)
+NotifyAttribute    *event;
+
+#endif /* __STDC__ */
+{
+  AttributeType     attrType;
+  Attribute         attr;
+  STRING            value;
+  int               length, i;
+
+  if (!SaveAsXML)
+     /* we are not saving the document in XML */
+     return FALSE;  /* let Thot perform normal operation */
+
+  attrType = event->attributeType;
+  attrType.AttrTypeNum = HTML_ATTR_ID;
+  attr = TtaGetAttribute (event->element, attrType);
+  if (!attr)
+     /* this element does not have an ID attribute */
+     {
+     /* get the value of the NAME attribute */
+     length = TtaGetTextAttributeLength (event->attribute);
+     length+= 10;
+     value = TtaAllocString (length);
+     length--;
+     TtaGiveTextAttributeValue (event->attribute, &value[1], &length);
+     if (value[1] >= '0' && value[1] <= '9')
+        /* the value of the NAME attribute starts with a digit */
+        {
+	/* insert an underscore at the beginning and create an ID attribute
+	   with that value */
+	value[0] = '_';
+	length++;
+	/* check that this value is not already used by another ID attribute
+           in the document and add a number at the end if it's the case */
+        i = 0;
+	while (SearchNAMEattribute (event->document, value, NULL))
+	   /* this value is already used in the document */
+	   {
+	   i++;
+	   usprintf (&value[length], TEXT("%d"), i);
+	   }
+	/* Create the ID attr. */
+	attr = TtaNewAttribute (attrType);
+	TtaAttachAttribute (event->element, attr, event->document);
+	TtaSetAttributeText (attr, value, event->element, event->document);
+	}
+     TtaFreeMemory (value);
+     }
+  return FALSE;  /* let Thot perform normal operation */
+}
+
+
 /*----------------------------------------------------------------------
    SetRelativeURLs: try to make relative URLs within an HTML document.
   ----------------------------------------------------------------------*/
@@ -393,19 +480,19 @@ STRING              pathname;
   if (TextFormat)
      {
        SaveAsHTML = FALSE;
-       SaveAsXHTML = FALSE;
+       SaveAsXML = FALSE;
        SaveAsText = TRUE;
      }
    else if (IsXMLName (pathname) || DocumentMeta[document]->xmlformat)
      {
        SaveAsHTML = FALSE;
-       SaveAsXHTML = TRUE;
+       SaveAsXML = TRUE;
        SaveAsText = FALSE;
      }
    else
      {
        SaveAsHTML = TRUE;
-       SaveAsXHTML = FALSE;
+       SaveAsXML = FALSE;
        SaveAsText = FALSE;
      }
 
@@ -422,18 +509,26 @@ STRING              pathname;
 		D_CANCEL);
 
    /* choice between html, xhtml and text */
-   sprintf (buffer, "%s%c%s%c%s%c%s%cB%s%cB%s", "BHTML", EOS, "BXHTML", EOS,
-	    "BText", EOS, "S", EOS,
-	    TtaGetMessage (AMAYA, AM_BCOPY_IMAGES), EOS,
-	    TtaGetMessage (AMAYA, AM_BTRANSFORM_URL));
-   TtaNewToggleMenu (BaseDialog + ToggleSave, BaseDialog + SaveForm,
-		     TtaGetMessage (LIB, TMSG_DOCUMENT_FORMAT), 6, buffer,
-		     NULL, TRUE);
-    TtaSetToggleMenu (BaseDialog + ToggleSave, 0, SaveAsHTML);
-   TtaSetToggleMenu (BaseDialog + ToggleSave, 1, SaveAsXHTML);
-   TtaSetToggleMenu (BaseDialog + ToggleSave, 2, SaveAsText);
-   TtaSetToggleMenu (BaseDialog + ToggleSave, 4, CopyImages);
-   TtaSetToggleMenu (BaseDialog + ToggleSave, 5, UpdateURLs);
+   if (!TextFormat &&
+       DocumentTypes[document] != docMath &&
+       DocumentTypes[document] != docSVG)
+     {
+       sprintf (buffer, "%s%c%s%c%s%c%s%cB%s%cB%s", "BHTML", EOS, "BXML", EOS,
+		"BText", EOS, "S", EOS,
+		TtaGetMessage (AMAYA, AM_BCOPY_IMAGES), EOS,
+		TtaGetMessage (AMAYA, AM_BTRANSFORM_URL));
+       TtaNewToggleMenu (BaseDialog + ToggleSave, BaseDialog + SaveForm,
+			 TtaGetMessage (LIB, TMSG_DOCUMENT_FORMAT), 6, buffer,
+			 NULL, TRUE);
+       TtaSetToggleMenu (BaseDialog + ToggleSave, 0, SaveAsHTML);
+       TtaSetToggleMenu (BaseDialog + ToggleSave, 1, SaveAsXML);
+       TtaSetToggleMenu (BaseDialog + ToggleSave, 2, SaveAsText);
+       TtaSetToggleMenu (BaseDialog + ToggleSave, 4, CopyImages);
+       TtaSetToggleMenu (BaseDialog + ToggleSave, 5, UpdateURLs);
+     }
+   else
+     TtaNewLabel (BaseDialog + ToggleSave, BaseDialog + SaveForm, TEXT(""));
+
    TtaListDirectory (SavePath, BaseDialog + SaveForm,
 		     TtaGetMessage (LIB, TMSG_DOC_DIR),
 		     BaseDialog + DirSave, ScanFilter,
@@ -445,23 +540,21 @@ STRING              pathname;
    TtaNewLabel (BaseDialog + Label1, BaseDialog + SaveForm, "");
    TtaNewLabel (BaseDialog + Label2, BaseDialog + SaveForm, "");
    /* third line */
-   TtaNewTextForm (BaseDialog + ImgDirSave, BaseDialog + SaveForm,
+   if (!TextFormat &&
+       DocumentTypes[document] != docMath &&
+       DocumentTypes[document] != docSVG)
+     {
+       TtaNewTextForm (BaseDialog + ImgDirSave, BaseDialog + SaveForm,
 		   TtaGetMessage (AMAYA, AM_IMAGES_LOCATION), 50, 1, FALSE);
-   TtaSetTextForm (BaseDialog + ImgDirSave, SaveImgsURL);
-   TtaNewLabel (BaseDialog + Label3, BaseDialog + SaveForm, "");
-   TtaNewLabel (BaseDialog + Label4, BaseDialog + SaveForm, "");
+       TtaSetTextForm (BaseDialog + ImgDirSave, SaveImgsURL);
+       TtaNewLabel (BaseDialog + Label3, BaseDialog + SaveForm, "");
+       TtaNewLabel (BaseDialog + Label4, BaseDialog + SaveForm, "");
+     }
    TtaNewTextForm (BaseDialog + FilterText, BaseDialog + SaveForm,
 		   TtaGetMessage (AMAYA, AM_PARSE), 10, 1, TRUE);
    TtaSetTextForm (BaseDialog + FilterText, ScanFilter);
 
    TtaShowDialogue (BaseDialog + SaveForm, FALSE);
-   if (TextFormat)
-     {
-       TtaRedrawMenuEntry (BaseDialog + ToggleSave, 0, NULL, -1, FALSE);
-       TtaRedrawMenuEntry (BaseDialog + ToggleSave, 1, NULL, -1, FALSE);
-       TtaRedrawMenuEntry (BaseDialog + ToggleSave, 4, NULL, -1, FALSE);
-       TtaRedrawMenuEntry (BaseDialog + ToggleSave, 5, NULL, -1, FALSE);
-     }
 #else /* _WINDOWS */
    CreateSaveAsDlgWindow (TtaGetViewFrame (document, view), pathname);
 #endif /* _WINDOWS */
@@ -604,15 +697,12 @@ View                view;
    if ((SavingDocument != 0 && SavingDocument != doc) ||
        SavingObject != 0)
       return;
-   else if (DocumentTypes[doc] == docSource ||
-	    DocumentTypes[doc] == docSourceRO)
+   else if (DocumentTypes[doc] == docSource)
       /* it's a source "view". Don't save it */
       return;
 
    TextFormat = (DocumentTypes[doc] == docText ||
-		 DocumentTypes[doc] == docTextRO ||
-		 DocumentTypes[doc] == docCSS ||
-		 DocumentTypes[doc] == docCSSRO);
+		 DocumentTypes[doc] == docCSS);
 
    /* memorize the current document */
    if (SavingDocument == 0)
@@ -698,10 +788,8 @@ Document     doc;
    ElementType		elType;
    AttributeType	attrType;
    Attribute		attr, charsetAttr;
-   Language             lang;
-   int                  length;
-   STRING               text;
-   CHAR_T               ISOlatin;
+   CHARSET              charset;
+   STRING               ptr;
 #define MAX_CHARSET_LEN 50
    CHAR_T               Charset[MAX_CHARSET_LEN];
    CHAR_T		buffer[200];
@@ -716,32 +804,6 @@ Document     doc;
 
    root = TtaGetMainRoot (doc);
    attrType.AttrSSchema = TtaGetSSchema (TEXT("HTML"), doc);
-   attrType.AttrTypeNum = HTML_ATTR_Namespaces;
-   attr = TtaGetAttribute (root, attrType);
-   if (!useMathML && !useGraphML && attr)
-      /* delete the Namespaces attribute */
-      TtaRemoveAttribute (root, attr, doc);
-   else if (useMathML || useGraphML)
-      {
-      /* prepare the value of attribute Namespaces */
-      buffer[0] = EOS;
-      if (useMathML)
-	 {
-	 ustrcat (buffer, TEXT("\n      xmlns:m=\"http://www.w3.org/1998/Math/MathML/\""));
-	 }
-      if (useGraphML)
-	 {
-	 ustrcat (buffer, TEXT("\n      xmlns:g=\"http://www.w3.org/Graphics/SVG/Amaya2D\""));
-	 }
-      /* set the value of attribute Namespaces */
-      if (attr == NULL)
-	 {
-	 attr = TtaNewAttribute (attrType);
-	 TtaAttachAttribute (root, attr, doc);
-	 }
-      TtaSetAttributeText (attr, buffer, root, doc);
-      }
-
    /* looks for a FRAMESET element and set attribute HtmlDTD */
    useFrames = FALSE;
    el = TtaGetFirstChild (root);
@@ -754,29 +816,118 @@ Document     doc;
       else
          TtaNextSibling (&el);
       }
-   attrType.AttrTypeNum = HTML_ATTR_HtmlDTD;
-   attr = TtaGetAttribute (root, attrType);
-   if (!attr)
-      {
-      attr = TtaNewAttribute (attrType);
-      TtaAttachAttribute (root, attr, doc);
-      }
-   if (useFrames)
-      TtaSetAttributeValue (attr, HTML_ATTR_HtmlDTD_VAL_Frameset, root, doc);
-   else
-      TtaSetAttributeValue (attr, HTML_ATTR_HtmlDTD_VAL_Transitional, root,
-			    doc);
 
-   /* get the charset defined by the Charset attribute of the root element*/
-   attrType.AttrTypeNum = HTML_ATTR_Charset;
-   charsetAttr = TtaGetAttribute (root, attrType);
-   Charset[0] = EOS;
-   if (charsetAttr)
+   attrType.AttrTypeNum = HTML_ATTR_Namespaces;
+   attr = TtaGetAttribute (root, attrType);
+   if (!useMathML && !useGraphML)
+     {
+       /* delete the Namespaces attribute */
+       if (attr)
+	 TtaRemoveAttribute (root, attr, doc);
+
+       /* generating the DOCTYPE */
+       buffer[0] = EOS;
+       attrType.AttrTypeNum = HTML_ATTR_HtmlDTD;
+       attr = TtaGetAttribute (root, attrType);
+       if (!attr)
+	 {
+	   attr = TtaNewAttribute (attrType);
+	   TtaAttachAttribute (root, attr, doc);
+	 }
+
+	ustrcat (buffer, TEXT("<!DOCTYPE html PUBLIC "));
+	if (DocumentMeta[doc]->xmlformat)
+	  {
+	    /* xml format */
+	    if (useFrames)
+	      {
+		ustrcat (buffer, HTMLDocTypes_1[3]);
+		ustrcat (buffer, TEXT("\n    "));
+		ustrcat (buffer, HTMLDocTypes_2[3]);
+	      }
+	    else if (ParsingLevel[doc] == L_Basic)
+	      {
+		ustrcat (buffer, HTMLDocTypes_1[0]);
+		ustrcat (buffer, TEXT("\n    "));
+		ustrcat (buffer, HTMLDocTypes_2[0]);
+	      }
+	    else if (ParsingLevel[doc] == L_Strict)
+	      {
+		ustrcat (buffer, HTMLDocTypes_1[1]);
+		ustrcat (buffer, TEXT("\n    "));
+		ustrcat (buffer, HTMLDocTypes_2[1]);
+	      }
+	    else
+	      {
+		ustrcat (buffer, HTMLDocTypes_1[2]);
+		ustrcat (buffer, TEXT("\n    "));
+		ustrcat (buffer, HTMLDocTypes_2[2]);
+	      }
+	  }
+	else
+	  {
+	    if (useFrames)
+	      {
+		ustrcat (buffer, HTMLDocTypes_1[6]);
+		ustrcat (buffer, TEXT("\n    "));
+		ustrcat (buffer, HTMLDocTypes_2[6]);
+	      }
+	    else if (ParsingLevel[doc] == L_Strict)
+	      {
+		ustrcat (buffer, HTMLDocTypes_1[4]);
+		ustrcat (buffer, TEXT("\n    "));
+		ustrcat (buffer, HTMLDocTypes_2[4]);
+	      }
+	    else
+	      {
+		ustrcat (buffer, HTMLDocTypes_1[5]);
+		ustrcat (buffer, TEXT("\n    "));
+		ustrcat (buffer, HTMLDocTypes_2[5]);
+	      }
+	  }
+	TtaSetAttributeText (attr, buffer, root, doc);
+     }
+   else if (useMathML || useGraphML)
       {
-      length = MAX_CHARSET_LEN - 2;
-      TtaGiveTextAttributeValue (charsetAttr, Charset, &length);
+	/* prepare the value of attribute Namespaces */
+	buffer[0] = EOS;
+	if (useMathML)
+	  ustrcat (buffer, TEXT("\n      xmlns:m=\"http://www.w3.org/1998/Math/MathML/\""));
+	if (useGraphML)
+	  ustrcat (buffer, TEXT("\n      xmlns:g=\"http://www.w3.org/Graphics/SVG/Amaya2D\""));
+	/* set the value of attribute Namespaces */
+	if (attr == NULL)
+	  {
+	    attr = TtaNewAttribute (attrType);
+	    TtaAttachAttribute (root, attr, doc);
+	  }
+	TtaSetAttributeText (attr, buffer, root, doc);
+
+	/* delete the DOCTYPE attribute */
+	attrType.AttrTypeNum = HTML_ATTR_HtmlDTD;
+	attr = TtaGetAttribute (root, attrType);
+	if (attr)
+	  TtaRemoveAttribute (root, attr, doc);
       }
 
+   /* get the document charset */
+   Charset[0] = EOS;
+   charset = TtaGetDocumentCharset (doc);
+   if (charset != UNDEFINED_CHARSET)
+     {
+       ptr = TtaGetCharsetName (charset);
+       ustrcat (Charset, ptr);
+       /* set the Charset attribute of the root element*/
+       attrType.AttrTypeNum = HTML_ATTR_Charset;
+       charsetAttr = TtaGetAttribute (root, attrType);
+       if (!charsetAttr)
+	 {
+	   charsetAttr = TtaNewAttribute (attrType);
+	   TtaAttachAttribute (root, charsetAttr, doc);
+	   TtaSetAttributeText (charsetAttr, Charset, root, doc);	
+	 }
+     }
+
    /* Create (or update) a META element to specify Content-type and Charset */
    /* Get the HEAD element first */
    el = TtaGetFirstChild (root);
@@ -792,60 +943,9 @@ Document     doc;
       }
    if (head)
       {
-      if (Charset[0] == EOS)
-	/* no Charset attribute on the root element */
-	{
-        /* is there a Language attribute on the root element? */
-        attrType.AttrTypeNum = HTML_ATTR_Langue;
-        attr = TtaGetAttribute (root, attrType);
-        if (attr)
-	   /* there is a language attribute on the root */
-	   /* is it the defaut attribute set by Thot or the real one */
-	   {
-           attrType.AttrTypeNum = HTML_ATTR_RealLang;
-           if (!TtaGetAttribute (root, attrType))
-	     /* not the real one. look further */
-	      attr = NULL;
-	   }
-        if (!attr)
-	   /* no Language specified on the root. Look for the body element */
-	   {
-           attrType.AttrTypeNum = HTML_ATTR_Langue;
-	   el = head;
-           TtaNextSibling (&el);
-	   while (el && !attr)
-	     {
-             elType = TtaGetElementType (el);
-	     if (elType.ElSSchema == attrType.AttrSSchema &&
-	         elType.ElTypeNum == HTML_EL_BODY)
-	       /* it's the BODY element. Is there a language attribute on it */
-	       {
-	       attr = TtaGetAttribute (el, attrType);
-	       el = NULL;
-	       }
-	     else
-	       TtaNextSibling (&el);
-	     }
-	   }
-        if (attr)
-	   /* there is a Language attribute on the root or body element */
-	   {
-	   length = TtaGetTextAttributeLength (attr);
-	   text = TtaAllocString (length + 1);
-	   TtaGiveTextAttributeValue (attr, text, &length);
-	   lang = TtaGetLanguageIdFromName (text);
-	   TtaFreeMemory (text);
-	   ISOlatin = TtaGetAlphabet (lang);
-	   if (ISOlatin == TEXT('L'))
-	     ustrcpy (Charset, TEXT("iso-8859-1"));
-	   else if (ISOlatin == TEXT('2'))
-	     ustrcpy (Charset, TEXT("iso-8859-2"));
-	   else if (ISOlatin == TEXT('9'))
-	     ustrcpy (Charset, TEXT("iso-8859-9"));
-	   else
-	     Charset[0] = EOS;
-	   }
-	}
+      /* indicate the MIME type and the charset in a meta element with
+	 an http-equiv attr */
+      /* look for a meta/http-equiv element */
       el = TtaGetFirstChild (head);
       meta = NULL;
       lastmeta = NULL;
@@ -870,6 +970,8 @@ Document     doc;
 	    }
 	 }
       if (!meta)
+	 /* there is no meta element with a http-equiv attribute */
+	 /* create one */
 	 {
 	 elType.ElSSchema = attrType.AttrSSchema;
 	 elType.ElTypeNum = HTML_EL_META;
@@ -903,17 +1005,71 @@ Document     doc;
 	 TtaSetAttributeText (attr, buffer, meta, doc);
 	 }
       } 
-   if (!charsetAttr)
-     if (Charset[0] != EOS)
-	/* create a Charset attribute on the root element */
-	{
-	attrType.AttrTypeNum = HTML_ATTR_Charset;
-	charsetAttr = TtaNewAttribute (attrType);
-	TtaAttachAttribute (root, charsetAttr, doc);
-	TtaSetAttributeText (charsetAttr, Charset, root, doc);	
-	}
 }
 
+/*----------------------------------------------------------------------
+   RestartParser
+  ----------------------------------------------------------------------*/
+#ifdef __STDC__
+static void       RestartHTMLParser (Document doc, CHAR_T *localFile, CHAR_T *tempdir, CHAR_T *documentname)
+#else
+static void       RestartHTMLParser (doc, localFile, tempdir, documentname)
+Document          doc;
+CHAR_T           *localFile;
+CHAR_T           *tempdir;
+CHAR_T           *documentname;
+#endif
+{
+  CHARSET             charset;
+  CHAR_T              htmlErrFile [80];
+  int                 parsingLevel;
+  int                 i;
+  ThotBool            xmlDec, withDoctype, isXML;
+  DocumentType        thotType;
+
+  CheckDocHeader (localFile, &xmlDec, &withDoctype, &isXML, &parsingLevel,
+		  &charset, &thotType);
+  /* clean up previous log file */
+  HTMLErrorsFound = FALSE;
+  XMLErrorsFound = FALSE;
+  /* remove the log file */
+  usprintf (htmlErrFile, TEXT("%s%c%d%cHTML.ERR"),
+	    TempFileDirectory, DIR_SEP, doc, DIR_SEP);
+  if (TtaFileExist (htmlErrFile))
+    TtaFileUnlink (htmlErrFile);
+  for (i = 1; i < DocumentTableLength; i++)
+    if (DocumentURLs[i] != NULL)
+      if (DocumentSource[i] == doc && DocumentTypes[i] == docLog)
+	{
+	  /* remove the log file attached to the current document */
+	  TtaCloseDocument (i);
+	  /* remove the log file */
+	  TtaFileUnlink (DocumentURLs[i]);
+	  TtaFreeMemory (DocumentURLs[i]);
+	  DocumentURLs[i] = NULL;
+	  /* switch off the button Show Log file */
+	  TtaSetItemOff (doc, 1, Special, BShowLogFile);
+	}
+  
+  /* Calls the corresponding parser */
+  if (DocumentMeta[doc]->xmlformat)       
+    StartXmlParser (doc, localFile, documentname, tempdir,
+		    localFile, xmlDec, withDoctype);
+  else
+    StartParser (doc, localFile, documentname, tempdir, localFile, FALSE);
+
+  /* check parsing errors */
+  if (HTMLErrorsFound || XMLErrorsFound)
+    TtaSetItemOn (doc, 1, Special, BShowLogFile);
+  else
+    TtaSetItemOff (doc, 1, Special, BShowLogFile);
+
+  /* fetch and display all images referred by the document */
+  DocNetworkStatus[doc] = AMAYA_NET_ACTIVE;
+  FetchAndDisplayImages (doc, AMAYA_LOAD_IMAGE);
+  DocNetworkStatus[doc] = AMAYA_NET_INACTIVE;
+}
+ 
 /*----------------------------------------------------------------------
    RedisplaySourceFile
    If doc is a HTML document and the source view is open, redisplay the
@@ -927,14 +1083,15 @@ Document          doc;
 
 #endif
 {
-  CHAR_T*   localFile;
-  CHAR_T	documentname[MAX_LENGTH];
-  CHAR_T	tempdir[MAX_LENGTH];
-  NotifyElement event;
+  CHAR_T             *localFile;
+  CHAR_T	      documentname[MAX_LENGTH];
+  CHAR_T	      tempdir[MAX_LENGTH];
+  NotifyElement       event;
 
-  if (DocumentTypes[doc] == docHTML || DocumentTypes[doc] == docHTMLRO)
-    /* It's a HTML document */
-    {
+  if (DocumentTypes[doc] == docHTML ||
+      DocumentTypes[doc] == docSVG ||
+      DocumentTypes[doc] == docMath)
+    /* it's a structured document */
     if (DocumentSource[doc])
        /* The source code of this document is currently displayed */
        {
@@ -951,7 +1108,6 @@ Document          doc;
 	  SetWindowTitle (doc, DocumentSource[doc], 0);
 	  TtaFreeMemory (localFile);
        }
-    }
 }
 
 /*----------------------------------------------------------------------
@@ -983,21 +1139,33 @@ STRING            documentName;
   if (SaveAsText) 
     {
       SetInternalLinks (doc);
-      ok = TtaExportDocument (doc, tempname, TEXT("HTMLTT"));
+      if (DocumentTypes[doc] == docHTML)
+         ok = TtaExportDocument (doc, tempname, TEXT("HTMLTT"));
+      else if (DocumentTypes[doc] == docSVG)
+         ok = TtaExportDocument (doc, tempname, TEXT("GraphMLT"));
+      if (DocumentTypes[doc] == docMath)
+         ok = TtaExportDocument (doc, tempname, TEXT("MathMLT"));
     }
   else
     {
-      SetNamespacesAndDTD (doc);
-      if (SaveAsXHTML)
+      if (DocumentTypes[doc] == docHTML)
 	{
-	ok = TtaExportDocumentWithNewLineNumbers (doc, tempname, TEXT("HTMLTX"));
-	DocumentMeta[doc]->xmlformat = TRUE;
-	}
-      else
-	{
-	ok = TtaExportDocumentWithNewLineNumbers (doc, tempname, TEXT("HTMLT"));
-	DocumentMeta[doc]->xmlformat = FALSE;
+	SetNamespacesAndDTD (doc);
+        if (SaveAsXML)
+	  {
+	  ok = TtaExportDocumentWithNewLineNumbers (doc, tempname, TEXT("HTMLTX"));
+	  DocumentMeta[doc]->xmlformat = TRUE;
+	  }
+        else
+	  {
+	  ok = TtaExportDocumentWithNewLineNumbers (doc, tempname, TEXT("HTMLT"));
+	  DocumentMeta[doc]->xmlformat = FALSE;
+	  }
 	}
+      else if (DocumentTypes[doc] == docSVG)
+	ok = TtaExportDocumentWithNewLineNumbers (doc, tempname, TEXT("GraphMLT"));
+      else if (DocumentTypes[doc] == docMath)
+	ok = TtaExportDocumentWithNewLineNumbers (doc, tempname, TEXT("MathMLT"));
       if (ok)
 	{
 	  TtaSetDocumentDirectory (doc, directoryName);
@@ -1189,8 +1357,7 @@ ThotBool            use_preconditions;
   tempname = GetLocalPath (document, url);
 
   /* build the output */
-  if (DocumentTypes[document] == docSource ||
-      DocumentTypes[document] == docSourceRO)
+  if (DocumentTypes[document] == docSource)
       /* it's a source file, renumber lines */
       TtaExportDocumentWithNewLineNumbers (document, tempname,
 					   TEXT("TextFileT"));
@@ -1211,7 +1378,7 @@ ThotBool            use_preconditions;
       if (confirm)
 	{
 	  InitConfirm3L (document, view, msg, AmayaLastHTTPErrorMsg, 
-			 TtaGetMessage (AMAYA, AM_SAVE_DISK));
+			 TtaGetMessage (AMAYA, AM_SAVE_DISK), TRUE);
 	  if (UserAnswer)
 	    res = -1;
 	  else
@@ -1219,7 +1386,8 @@ ThotBool            use_preconditions;
 	}
       else
 	{
-	  InitInfo (TEXT("Save"), msg);
+	  InitConfirm3L (document, view, msg, AmayaLastHTTPErrorMsg, 
+			 AmayaLastHTTPErrorMsgR, FALSE);
 	  res = -1;
 	}
       /* erase the last status message */
@@ -1271,17 +1439,24 @@ ThotBool         use_preconditions;
   tempname = GetLocalPath (doc, url);
 
   /* First step : generate the output file and ask for confirmation */
-  SetNamespacesAndDTD (doc);
-  if (SaveAsXHTML)
+  if (DocumentTypes[doc] == docHTML)
     {
-    TtaExportDocumentWithNewLineNumbers (doc, tempname, TEXT("HTMLTX"));
-    DocumentMeta[doc]->xmlformat = TRUE;
-    }
-  else
-    {
-    TtaExportDocumentWithNewLineNumbers (doc, tempname, TEXT("HTMLT"));
-    DocumentMeta[doc]->xmlformat = FALSE;
+    SetNamespacesAndDTD (doc);
+    if (SaveAsXML)
+      {
+      TtaExportDocumentWithNewLineNumbers (doc, tempname, TEXT("HTMLTX"));
+      DocumentMeta[doc]->xmlformat = TRUE;
+      }
+    else
+      {
+      TtaExportDocumentWithNewLineNumbers (doc, tempname, TEXT("HTMLT"));
+      DocumentMeta[doc]->xmlformat = FALSE;
+      }
     }
+  else if (DocumentTypes[doc] == docSVG)
+      TtaExportDocumentWithNewLineNumbers (doc, tempname, TEXT("GraphMLT"));
+  else if (DocumentTypes[doc] == docMath)
+      TtaExportDocumentWithNewLineNumbers (doc, tempname, TEXT("MathMLT"));
 
   res = 0;
   if (confirm && with_images)
@@ -1361,7 +1536,8 @@ ThotBool         use_preconditions;
 		    url);
 	  if (confirm)
 	    {
-	      InitConfirm3L (doc, view, msg, AmayaLastHTTPErrorMsg, NULL);
+	      InitConfirm3L (doc, view, msg, AmayaLastHTTPErrorMsg,
+			     AmayaLastHTTPErrorMsgR, FALSE);
 	      if (UserAnswer)
 		res = -1;
 	      else
@@ -1369,7 +1545,8 @@ ThotBool         use_preconditions;
 	    }
 	  else
 	    {
-	      InitInfo (TEXT("Save"), msg);
+	      InitConfirm3L (doc, view, msg, AmayaLastHTTPErrorMsg, 
+			     AmayaLastHTTPErrorMsgR, FALSE);
 	      res = -1;
 	    }
 	  /* JK: to erase the last status message */
@@ -1393,7 +1570,8 @@ ThotBool         use_preconditions;
 		  usprintf (msg, TEXT("%s %s"),
 			    TtaGetMessage (AMAYA, AM_URL_SAVE_FAILED),
 			    pImage->originalName);
-		  InitConfirm3L (doc, view, msg, AmayaLastHTTPErrorMsg, NULL);
+		  InitConfirm3L (doc, view, msg, AmayaLastHTTPErrorMsg, 
+				 AmayaLastHTTPErrorMsgR, FALSE);
 		  /* erase the last status message */
 		  TtaSetStatus (doc, view, TEXT(""), NULL);
 		  if (UserAnswer)
@@ -1423,13 +1601,14 @@ ThotBool         use_preconditions;
 }
 
 /*----------------------------------------------------------------------
-   GetHTMLdocFromSource
-   If sourceDoc is a source file, return the corresponding HTML document.
+   GetDocFromSource
+   If sourceDoc is a source document, return the corresponding
+   structured document.
   ----------------------------------------------------------------------*/
 #ifdef __STDC__
-Document       GetHTMLdocFromSource (Document sourceDoc)
+Document       GetDocFromSource (Document sourceDoc)
 #else
-Document       GetHTMLdocFromSource (sourceDoc)
+Document       GetDocFromSource (sourceDoc)
 Document       sourceDoc;
 
 #endif
@@ -1438,11 +1617,13 @@ Document       sourceDoc;
   int		i;
 
   htmlDoc = 0;
-  if (DocumentTypes[sourceDoc] == docSource ||
-      DocumentTypes[sourceDoc] == docSourceRO)
-     /* It's a source file */
+  if (DocumentTypes[sourceDoc] == docSource)
+     /* It's a source document */
      for (i = 1; i < DocumentTableLength && htmlDoc == 0; i++)
-        if (DocumentTypes[i] == docHTML)
+        if (DocumentTypes[i] == docHTML ||
+	    DocumentTypes[i] == docAnnot ||
+	    DocumentTypes[i] == docSVG ||
+	    DocumentTypes[i] == docMath)
            if (DocumentSource[i] == sourceDoc)
 	      htmlDoc = i;
   return htmlDoc;
@@ -1450,8 +1631,8 @@ Document       sourceDoc;
 
 /*----------------------------------------------------------------------
    Synchronize
-   save the current view (source/html) in a temporary file and update
-   the other view (html/source).      
+   save the current view (source/structure) in a temporary file and update
+   the other view (structure/source).      
   ----------------------------------------------------------------------*/
 #ifdef __STDC__
 void                Synchronize (Document document, View view)
@@ -1471,17 +1652,18 @@ View                view;
    if (!DocumentURLs[document])
      /* the document is not loaded yet */
      return;
+   otherDoc = 0;
    if (!TtaIsDocumentUpdated (document))
      /* nothing new to be saved in this view of the document. Let see if
         the other view has been modified */
      {
        if (DocumentTypes[document] == docHTML ||
-           DocumentTypes[document] == docHTMLRO)
-           /* it's an HTML document */
+	   DocumentTypes[document] == docSVG ||
+	   DocumentTypes[document] == docMath)
+           /* it's a structured document */
 	  otherDoc = DocumentSource[document];
-       else if (DocumentTypes[document] == docSource ||
-                DocumentTypes[document] == docSourceRO)
-          otherDoc = GetHTMLdocFromSource (document);
+       else if (DocumentTypes[document] == docSource)
+          otherDoc = GetDocFromSource (document);
        else
 	  return;
        if (!TtaIsDocumentUpdated (otherDoc))
@@ -1499,46 +1681,56 @@ View                view;
      TtaSetDisplayMode (document, DeferredDisplay);
 
    if (DocumentTypes[document] == docHTML ||
-       DocumentTypes[document] == docHTMLRO)
-     /* it's an HTML document */
+       DocumentTypes[document] == docSVG ||
+       DocumentTypes[document] == docMath)
+     /* it's the structured form of the document */
      {
        /* save the current state of the document into the temporary file */
        tempdocument = GetLocalPath (document, DocumentURLs[document]);
-       SetNamespacesAndDTD (document);
-       if (DocumentMeta[document]->xmlformat)
+       if (DocumentTypes[document] == docHTML)
+	 {
+         SetNamespacesAndDTD (document);
+	 if (DocumentMeta[document]->xmlformat)
+	   TtaExportDocumentWithNewLineNumbers (document, tempdocument,
+						TEXT("HTMLTX"));
+	 else
+	   TtaExportDocumentWithNewLineNumbers (document, tempdocument,
+						TEXT("HTMLT"));
+	 }
+       else if (DocumentTypes[document] == docSVG)
 	 TtaExportDocumentWithNewLineNumbers (document, tempdocument,
-					      TEXT("HTMLTX"));
-       else
+					      TEXT("GraphMLT"));
+       else if (DocumentTypes[document] == docMath)
 	 TtaExportDocumentWithNewLineNumbers (document, tempdocument,
-					      TEXT("HTMLT"));
+					      TEXT("MathMLT"));
        RedisplaySourceFile (document);
        otherDoc = DocumentSource[document];
+       /* the other document is now different from the original file. It can
+	  be saved */
+       TtaSetItemOn (otherDoc, 1, File, BSave);
      }
-   else if (DocumentTypes[document] == docSource ||
-       DocumentTypes[document] == docSourceRO)
+   else if (DocumentTypes[document] == docSource)
      /* it's a source document */
      {
-       htmlDoc = GetHTMLdocFromSource (document);
+       htmlDoc = GetDocFromSource (document);
        otherDoc = htmlDoc;
        /* save the current state of the document into the temporary file */
        tempdocument = GetLocalPath (htmlDoc, DocumentURLs[htmlDoc]);
        TtaExportDocumentWithNewLineNumbers (document, tempdocument,
 					    TEXT("TextFileT"));
        TtaExtractName (tempdocument, tempdir, documentname);
-       StartParser (htmlDoc, tempdocument, documentname, tempdir, tempdocument,
-		    FALSE);
-       /* fetch and display all images referred by the document */
-       DocNetworkStatus[htmlDoc] = AMAYA_NET_ACTIVE;
-       FetchAndDisplayImages (htmlDoc, AMAYA_LOAD_IMAGE);
-       DocNetworkStatus[htmlDoc] = AMAYA_NET_INACTIVE;
+       RestartHTMLParser (htmlDoc, tempdocument, tempdir, documentname);
+       /* the other document is now different from the original file. It can
+	  be saved */
+       TtaSetDocumentModified (otherDoc);
+       /* the source can be closed without save */
+       TtaSetDocumentUnmodified (document);
+       /* but it could be saved too */
+       TtaSetItemOn (document, 1, File, BSave);
      }
    /* restore original display mode */
    TtaSetDisplayMode (document, dispMode);
 
-   /* the other document is now different from the original file. It can
-      be saved */
-   TtaSetDocumentModified (otherDoc);
-   TtaSetItemOn (otherDoc, 1, File, BSave);
    /* disable the Synchronize command for both documents */
    TtaSetItemOff (otherDoc, 1, File, BSynchro);
    TtaSetItemOff (document, 1, File, BSynchro);
@@ -1579,7 +1771,7 @@ View                view;
 #ifdef ANNOTATIONS
   if (DocumentTypes[doc] == docAnnot) 
     {
-      ANNOT_SaveDocument (doc);
+      ANNOT_SaveDocument (doc, view);
       return;
     }
   else
@@ -1597,16 +1789,12 @@ View                view;
     }
 
   TextFormat = (DocumentTypes[doc] == docText ||
-		DocumentTypes[doc] == docTextRO ||
 		DocumentTypes[doc] == docCSS ||
-		DocumentTypes[doc] == docCSSRO ||
-		DocumentTypes[doc] == docSource ||
-		DocumentTypes[doc] == docSourceRO);
+		DocumentTypes[doc] == docSource);
 
   /* if it's a source document, get the corresponding HTML document */
-  if (DocumentTypes[doc] == docSource ||
-      DocumentTypes[doc] == docSourceRO)
-     htmlDoc = GetHTMLdocFromSource (doc);
+  if (DocumentTypes[doc] == docSource)
+     htmlDoc = GetDocFromSource (doc);
   else
      htmlDoc = 0;
 
@@ -1644,7 +1832,7 @@ View                view;
     TtaSetDisplayMode (doc, DeferredDisplay);
 
   /* the suffix determines the output format */
-  SaveAsXHTML = IsXMLName (tempname) || DocumentMeta[doc]->xmlformat;
+  SaveAsXML = IsXMLName (tempname) || DocumentMeta[doc]->xmlformat;
   if (IsW3Path (tempname))
     /* it's a remote document */
     {
@@ -1659,8 +1847,7 @@ View                view;
 	  TtaFreeMemory (DocumentURLs[doc]);
 	  DocumentURLs[doc] = TtaWCSdup (tempname);
 	  DocumentMeta[doc]->put_default_name = TRUE; 
-	  if (DocumentTypes[doc] == docHTML ||
-	      DocumentTypes[doc] == docHTMLRO)	
+	  if (DocumentTypes[doc] == docHTML)
 	      /* it's an HTML document. It could have a source doc */
 	      /* change the URL of the source document if it exists */
 	     {
@@ -1671,8 +1858,7 @@ View                view;
 		DocumentURLs[DocumentSource[doc]] = TtaWCSdup (tempname);
 		}
 	      }
-	  else if (DocumentTypes[doc] == docSource ||
-		   DocumentTypes[doc] == docSourceRO)
+	  else if (DocumentTypes[doc] == docSource)
 	      {
 	      /* it's a source document. Change the URL of the corresponding
 		 HTML document */
@@ -1696,8 +1882,7 @@ View                view;
 	  {
 	  ok = SaveObjectThroughNet (doc, view, DocumentURLs[doc],
 				     FALSE, TRUE);
-	  if (DocumentTypes[doc] == docSource ||
-	      DocumentTypes[doc] == docSourceRO)
+	  if (DocumentTypes[doc] == docSource)
 	     /* it's a source file. lines have been renumbered */
 	     newLineNumbers = TRUE;
 	  }
@@ -1713,8 +1898,7 @@ View                view;
     {
       ustrcpy (localFile, tempname);
       if (TextFormat)
-	if (DocumentTypes[doc] == docSource ||
-	    DocumentTypes[doc] == docSourceRO)
+	if (DocumentTypes[doc] == docSource)
 	   /* it's a source file. renumber lines */
 	   {
 	   ok = TtaExportDocumentWithNewLineNumbers (doc, tempname,
@@ -1722,13 +1906,13 @@ View                view;
 	   newLineNumbers = TRUE;
 	   }
 	else
-	  {
-	    ok = TtaExportDocument (doc, tempname, TEXT("TextFileT"));
-	  }
+	  ok = TtaExportDocument (doc, tempname, TEXT("TextFileT"));
       else
 	{
+	if (DocumentTypes[doc] == docHTML)
+	  {
 	  SetNamespacesAndDTD (doc);
-	  if (SaveAsXHTML)
+	  if (SaveAsXML)
 	    {
 	    ok = TtaExportDocumentWithNewLineNumbers (doc, tempname,
 						      TEXT("HTMLTX"));
@@ -1740,7 +1924,14 @@ View                view;
 						      TEXT("HTMLT"));
 	    DocumentMeta[doc]->xmlformat = FALSE;
 	    }
-	  newLineNumbers = TRUE;
+	  }
+	else if (DocumentTypes[doc] == docSVG)
+	  ok = TtaExportDocumentWithNewLineNumbers (doc, tempname,
+						    TEXT("GraphMLT"));
+	else if (DocumentTypes[doc] == docMath)
+	  ok = TtaExportDocumentWithNewLineNumbers (doc, tempname,
+						    TEXT("MathMLT"));
+	newLineNumbers = TRUE;
 	}
       /* save a local copy of the current document */
       if (htmlDoc)
@@ -1757,21 +1948,16 @@ View                view;
   SavingDocument = 0;
   if (newLineNumbers)
      /* line numbers have been changed in the saved document */
-     if (DocumentTypes[doc] == docHTML || DocumentTypes[doc] == docHTMLRO)
+     if (DocumentTypes[doc] == docHTML)
         /* It's a HTML document. If the source view is open, redisplay the
 	   source. */
        RedisplaySourceFile (doc);
-     else if (DocumentTypes[doc] == docSource ||
-	      DocumentTypes[doc] == docSourceRO)
+     else if (DocumentTypes[doc] == docSource)
 	/* It's a source document. Reparse the corresponding HTML document */
 	if (htmlDoc)
 	   {
 	   TtaExtractName (localFile, tempdir, documentname);
-	   StartParser (htmlDoc, localFile, documentname, tempdir, localFile, FALSE);
-	   /* fetch and display all images referred by the document */
-	   DocNetworkStatus[htmlDoc] = AMAYA_NET_ACTIVE;
-	   FetchAndDisplayImages (htmlDoc, AMAYA_LOAD_IMAGE);
-	   DocNetworkStatus[htmlDoc] = AMAYA_NET_INACTIVE;
+	   RestartHTMLParser (htmlDoc, localFile, tempdir, documentname);
 	   TtaSetDocumentUnmodified (htmlDoc);
 	   /* Synchronize selections */
 	   event.document = doc;
@@ -1782,8 +1968,7 @@ View                view;
       if (DocumentMeta[doc]->method == CE_TEMPLATE)
 	{
 	  DocumentMeta[doc]->method = CE_ABSOLUTE;
-	  TtaFreeMemory (DocumentMeta[doc]->form_data);
-	  DocumentMeta[doc]->form_data = NULL;
+	  DocumentMetaClear (DocumentMeta[doc]);
 	}
       TtaSetDocumentUnmodified (doc);
       /* switch Amaya buttons and menus */
@@ -2516,6 +2701,7 @@ void                DoSaveAs ()
 	    TtaCancelLastRegisteredSequence (doc);
 	  /* add to the history the data of the previous document */
 	  AddDocHistory (doc, DocumentURLs[doc], 
+			 DocumentMeta[doc]->initial_url,
 			 DocumentMeta[doc]->form_data,
 			 DocumentMeta[doc]->method);
 	  /* change the document url */
@@ -2532,8 +2718,7 @@ void                DoSaveAs ()
 	      if (DocumentMeta[doc]->method == CE_TEMPLATE)
 		{
 		  DocumentMeta[doc]->method = CE_ABSOLUTE;
-		  TtaFreeMemory (DocumentMeta[doc]->form_data);
-		  DocumentMeta[doc]->form_data = NULL;
+		  DocumentMetaClear (DocumentMeta[doc]);
 		}
 	      TtaSetDocumentUnmodified (doc);
 	      /* switch Amaya buttons and menus */
