@@ -17,29 +17,21 @@
 #include "amaya.h"
 #include "css.h"
 #include "html2thot_f.h"
+#include "Xml2thot_f.h"
 #include "MathML.h"
 #include "parser.h"
 #include "styleparser_f.h"
 #include "style.h"
 #include "undo.h"
 
-typedef CHAR_T  MathEntityName[30];
-typedef struct _MathEntity
-  {			 /* a Math entity representing an operator char */
-     MathEntityName      MentityName;	/* entity name */
-     int                 charCode;	/* decimal code of char */
-     CHAR_T		 alphabet;	/* 'L' = ISO-Latin-1, 'G' = Symbol */
-  }
-MathEntity;
-
-static MathEntity        MathEntityTable[] =
+XmlEntity  MathEntityTable[] =
 {
    /* This table MUST be in alphabetical order */
    /* This table contains characters from the Symbol font plus some
       specific MathML entities */
    {TEXT("Agr"), 65, 'G'},
    {TEXT("And"), 217, 'G'},
-   {TEXT("ApplyFunction"), 32, 'L'},  /* render as white space */
+   {TEXT("ApplyFunction"), 129, 'L'},   /* rendered as a thin space */
    {TEXT("Backslash"), 92, 'L'},
    {TEXT("Bgr"), 66, 'G'},
    {TEXT("Cap"), 199, 'G'},
@@ -75,8 +67,8 @@ static MathEntity        MathEntityTable[] =
    {TEXT("ImaginaryI"), 105, 'L'},
    {TEXT("Integral"), 242, 'G'},
    {TEXT("Intersection"), 199, 'G'},
-   {TEXT("InvisibleComma"), 129, 'L'},
-   {TEXT("InvisibleTimes"), 0, SPACE},
+   {TEXT("InvisibleComma"), 129, 'L'},   /* thin space */
+   {TEXT("InvisibleTimes"), 129, 'L'},   /* thin space */
    {TEXT("Kgr"), 75, 'G'},
    {TEXT("KHgr"), 67, 'G'},
    {TEXT("Lambda"), 76, 'G'},
@@ -95,6 +87,7 @@ static MathEntity        MathEntityTable[] =
    {TEXT("Omega"), 87, 'G'},
    {TEXT("Or"), 218, 'G'},
    {TEXT("OverBar"), 45, 'G'},
+   {TEXT("OverBrace"), 132, 'G'},    /* 132?? */
    {TEXT("PI"), 213, 'G'},
    {TEXT("PartialD"), 182, 'G'},
    {TEXT("Phi"), 70, 'G'},
@@ -126,6 +119,7 @@ static MathEntity        MathEntityTable[] =
    {TEXT("Tilde"), 126, 'L'},
    {TEXT("TripleDot"), 188, 'G'},
    {TEXT("UnderBar"), 45, 'G'},
+   {TEXT("UnderBrace"), 133, 'G'},    /* 133?? */
    {TEXT("Union"), 200, 'G'},
    {TEXT("UpArrow"), 173, 'G'},
    {TEXT("Upsi"), 85, 'G'},
@@ -135,7 +129,7 @@ static MathEntity        MathEntityTable[] =
    {TEXT("VerticalBar"), 124, 'L'},
    {TEXT("Xi"), 88, 'G'},
    {TEXT("Zgr"), 90, 'G'},
-   {TEXT("af"), 32, 'L'},             /* render as white space */
+   {TEXT("af"), 129, 'L'},             /* rendered as thin space */
    {TEXT("aleph"), 192, 'G'},
    {TEXT("alpha"), 97, 'G'},
    {TEXT("and"), 217, 'G'},
@@ -185,7 +179,7 @@ static MathEntity        MathEntityTable[] =
    {TEXT("int"), 242, 'G'},
    {TEXT("iota"), 105, 'G'},
    {TEXT("isin"), 206, 'G'},
-   {TEXT("it"), 0, SPACE},
+   {TEXT("it"), 129, 'L'},          /* rendered as a thin space */
    {TEXT("kappa"), 107, 'G'},
    {TEXT("lArr"), 220, 'G'},
    {TEXT("lambda"), 108, 'G'},
@@ -261,7 +255,7 @@ static MathEntity        MathEntityTable[] =
    {TEXT("therefore"), 92, 'G'},
    {TEXT("theta"), 113, 'G'},
    {TEXT("thetav"), 74, 'G'},
-   {TEXT("thickspace"), 32, 'L'},
+   {TEXT("thickspace"), 130, 'L'},
    {TEXT("times"), 180, 'G'},
    {TEXT("trade"), 212, 'G'},
    {TEXT("tradesf"), 212, 'G'},
@@ -277,83 +271,6 @@ static MathEntity        MathEntityTable[] =
 
 };
 
-static AttributeMapping MathMLAttributeMappingTable[] =
-{
-   /* The first entry MUST be unknown_attr */
-   /* The rest of this table MUST be in alphabetical order */
-   {TEXT("unknown_attr"), TEXT(""), 'A', MathML_ATTR_Invalid_attribute},
-   {TEXT("accent"), TEXT(""), 'A', MathML_ATTR_accent},
-   {TEXT("accentunder"), TEXT(""), 'A', MathML_ATTR_accentunder},
-   {TEXT("actiontype"), TEXT(""), 'A', MathML_ATTR_actiontype},
-   {TEXT("align"), TEXT(""), 'A', MathML_ATTR_align},
-   {TEXT("alignmentscope"), TEXT(""), 'A', MathML_ATTR_alignmentscope},
-   {TEXT("alt"), TEXT(""), 'A', MathML_ATTR_alt},
-   {TEXT("background"), TEXT(""), 'A', MathML_ATTR_background_},
-   {TEXT("bevelled"), TEXT(""), 'A', MathML_ATTR_bevelled},
-   {TEXT("class"), TEXT(""), 'A', MathML_ATTR_class},
-   {TEXT("close"), TEXT(""), 'A', MathML_ATTR_close},
-   {TEXT("columnalign"), TEXT(""), 'A', MathML_ATTR_columnalign},
-   {TEXT("columnlines"), TEXT(""), 'A', MathML_ATTR_columnlines},
-   {TEXT("columnspacing"), TEXT(""), 'A', MathML_ATTR_columnspacing},
-   {TEXT("columnspan"), TEXT(""), 'A', MathML_ATTR_columnspan},
-   {TEXT("columnwidth"), TEXT(""), 'A', MathML_ATTR_columnwidth},
-   {TEXT("color"), TEXT(""), 'A', MathML_ATTR_color},
-   {TEXT("denomalign"), TEXT(""), 'A', MathML_ATTR_denomalign},
-   {TEXT("depth"), TEXT(""), 'A', MathML_ATTR_depth_},
-   {TEXT("displaystyle"), TEXT(""), 'A', MathML_ATTR_displaystyle},
-   {TEXT("edge"), TEXT(""), 'A', MathML_ATTR_edge},
-   {TEXT("equalcolumns"), TEXT(""), 'A', MathML_ATTR_equalcolumns},
-   {TEXT("equalrows"), TEXT(""), 'A', MathML_ATTR_equalrows},
-   {TEXT("fence"), TEXT(""), 'A', MathML_ATTR_fence},
-   {TEXT("fontfamily"), TEXT(""), 'A', MathML_ATTR_fontfamily},
-   {TEXT("fontstyle"), TEXT(""), 'A', MathML_ATTR_fontstyle},
-   {TEXT("fontsize"), TEXT(""), 'A', MathML_ATTR_fontsize},
-   {TEXT("fontweight"), TEXT(""), 'A', MathML_ATTR_fontweight},
-   {TEXT("form"), TEXT(""), 'A', MathML_ATTR_form},
-   {TEXT("frame"), TEXT(""), 'A', MathML_ATTR_frame},
-   {TEXT("framespacing"), TEXT(""), 'A', MathML_ATTR_framespacing},
-   {TEXT("groupalign"), TEXT(""), 'A', MathML_ATTR_groupalign},
-   {TEXT("height"), TEXT(""), 'A', MathML_ATTR_height_},
-   {TEXT("id"), TEXT(""), 'A', MathML_ATTR_id},
-   {TEXT("index"), TEXT(""), 'A', MathML_ATTR_index},
-   {TEXT("largeop"), TEXT(""), 'A', MathML_ATTR_largeop},
-   {TEXT("linethickness"), TEXT(""), 'A', MathML_ATTR_linethickness},
-   {TEXT("lquote"), TEXT(""), 'A', MathML_ATTR_lquote},
-   {TEXT("lspace"), TEXT(""), 'A', MathML_ATTR_lspace},
-   {TEXT("maxsize"), TEXT(""), 'A', MathML_ATTR_maxsize},
-   {TEXT("minlabelspacing"), TEXT(""), 'A', MathML_ATTR_minlabelspacing},
-   {TEXT("minsize"), TEXT(""), 'A', MathML_ATTR_minsize},
-   {TEXT("movablelimits"), TEXT(""), 'A', MathML_ATTR_movablelimits},
-   {TEXT("name"), TEXT(""), 'A', MathML_ATTR_name},
-   {TEXT("notation"), TEXT(""), 'A', MathML_ATTR_notation},
-   {TEXT("numalign"), TEXT(""), 'A', MathML_ATTR_numalign},
-   {TEXT("open"), TEXT(""), 'A', MathML_ATTR_open},
-   {TEXT("other"), TEXT(""), 'A', MathML_ATTR_other},
-   {TEXT("rowalign"), TEXT(""), 'A', MathML_ATTR_rowalign},
-   {TEXT("rowlines"), TEXT(""), 'A', MathML_ATTR_rowlines},
-   {TEXT("rowspacing"), TEXT(""), 'A', MathML_ATTR_rowspacing},
-   {TEXT("rowspan"), TEXT(""), 'A', MathML_ATTR_rowspan_},
-   {TEXT("rquote"), TEXT(""), 'A', MathML_ATTR_rquote},
-   {TEXT("rspace"), TEXT(""), 'A', MathML_ATTR_rspace},
-   {TEXT("scriptlevel"), TEXT(""), 'A', MathML_ATTR_scriptlevel},
-   {TEXT("scriptminsize"), TEXT(""), 'A', MathML_ATTR_scriptminsize},
-   {TEXT("scriptsizemultiplier"), TEXT(""), 'A', MathML_ATTR_scriptsizemultiplier},
-   {TEXT("selection"), TEXT(""), 'A', MathML_ATTR_selection},
-   {TEXT("separator"), TEXT(""), 'A', MathML_ATTR_separator},
-   {TEXT("separators"), TEXT(""), 'A', MathML_ATTR_separators},
-   {TEXT("side"), TEXT(""), 'A', MathML_ATTR_side},
-   {TEXT("stretchy"), TEXT(""), 'A', MathML_ATTR_stretchy},
-   {TEXT("style"), TEXT(""), 'A', MathML_ATTR_style_},
-   {TEXT("subscriptshift"), TEXT(""), 'A', MathML_ATTR_subscriptshift},
-   {TEXT("superscriptshift"), TEXT(""), 'A', MathML_ATTR_superscriptshift},
-   {TEXT("symmetric"), TEXT(""), 'A', MathML_ATTR_symmetric},
-   {TEXT("width"), TEXT(""), 'A', MathML_ATTR_width_},
-   {TEXT("xref"), TEXT(""), 'A', MathML_ATTR_xref},
-   {TEXT("zzghost"), TEXT(""), 'A', MathML_ATTR_Ghost_restruct},
-
-   {TEXT(""), TEXT(""), EOS, 0}		/* Last entry. Mandatory */
-};
-
 /* mapping table of attribute values */
 
 static AttrValueMapping MathMLAttrValueMappingTable[] =
@@ -371,6 +288,10 @@ static AttrValueMapping MathMLAttrValueMappingTable[] =
  {MathML_ATTR_denomalign, TEXT("left"), MathML_ATTR_denomalign_VAL_left_},
  {MathML_ATTR_denomalign, TEXT("right"), MathML_ATTR_denomalign_VAL_right_},
 
+ {MathML_ATTR_display, TEXT("block"), MathML_ATTR_display_VAL_block},
+ {MathML_ATTR_display, TEXT("display"), MathML_ATTR_display_VAL_block},
+ {MathML_ATTR_display, TEXT("inline"), MathML_ATTR_display_VAL_inline_},
+
  {MathML_ATTR_displaystyle, TEXT("true"), MathML_ATTR_displaystyle_VAL_true},
  {MathML_ATTR_displaystyle, TEXT("false"), MathML_ATTR_displaystyle_VAL_false},
 
@@ -408,6 +329,11 @@ static AttrValueMapping MathMLAttrValueMappingTable[] =
  {MathML_ATTR_numalign, TEXT("left"), MathML_ATTR_numalign_VAL_left_},
  {MathML_ATTR_numalign, TEXT("right"), MathML_ATTR_numalign_VAL_right_},
 
+ {MathML_ATTR_overflow, TEXT("scroll"), MathML_ATTR_overflow_VAL_scroll},
+ {MathML_ATTR_overflow, TEXT("elide"), MathML_ATTR_overflow_VAL_elide},
+ {MathML_ATTR_overflow, TEXT("truncate"), MathML_ATTR_overflow_VAL_truncate},
+ {MathML_ATTR_overflow, TEXT("scale"), MathML_ATTR_overflow_VAL_scale_},
+
  {MathML_ATTR_separator, TEXT("true"), MathML_ATTR_separator_VAL_true},
  {MathML_ATTR_separator, TEXT("false"), MathML_ATTR_separator_VAL_false},
 
@@ -422,6 +348,9 @@ static AttrValueMapping MathMLAttrValueMappingTable[] =
  {MathML_ATTR_symmetric, TEXT("true"), MathML_ATTR_symmetric_VAL_true},
  {MathML_ATTR_symmetric, TEXT("false"), MathML_ATTR_symmetric_VAL_false},
 
+ {MathML_ATTR_xml_space, TEXT("default"), MathML_ATTR_xml_space_VAL_xml_space_default},
+ {MathML_ATTR_xml_space, TEXT("preserve"), MathML_ATTR_xml_space_VAL_xml_space_preserve},
+
  {0, TEXT(""), 0}			/* Last entry. Mandatory */
 };
 
@@ -439,39 +368,17 @@ static AttrValueMapping MathMLAttrValueMappingTable[] =
    attribute of name Attr and returns the corresponding Thot attribute type.
   ----------------------------------------------------------------------*/
 #ifdef __STDC__
-void          MapMathMLAttribute (CHAR_T* Attr, AttributeType *attrType, STRING elementName, Document doc)
+void          MapMathMLAttribute (CHAR_T *attrName, AttributeType *attrType, STRING elementName, Document doc)
 #else
-void          MapMathMLAttribute (Attr, attrType, elementName, doc)
-STRING              Attr;
+void          MapMathMLAttribute (attrName, attrType, elementName, doc)
+STRING              attrName;
 AttributeType      *attrType;
 STRING		    elementName;
 Document            doc;
 #endif
 {
-   int                 i;
-
-   attrType->AttrTypeNum = 0;
-   attrType->AttrSSchema = NULL;
-   i = 0;
-   do
-     if (ustrcasecmp (MathMLAttributeMappingTable[i].XMLattribute, Attr))
-       i++;
-     else
-       if (MathMLAttributeMappingTable[i].XMLelement[0] == EOS)
-	  {
-	  attrType->AttrTypeNum = MathMLAttributeMappingTable[i].ThotAttribute;
-	  attrType->AttrSSchema = GetMathMLSSchema (doc);
-	  }
-       else if (!ustrcasecmp (MathMLAttributeMappingTable[i].XMLelement,
-			       elementName))
-	  {
-	  attrType->AttrTypeNum = MathMLAttributeMappingTable[i].ThotAttribute;
-	  attrType->AttrSSchema = GetMathMLSSchema (doc);
-	  }
-       else
-	  i++;
-   while (attrType->AttrTypeNum <= 0 &&
-	  MathMLAttributeMappingTable[i].AttrOrContent != EOS);
+  attrType->AttrSSchema = GetMathMLSSchema (doc);
+  MapXMLAttribute (MATH_TYPE, attrName, elementName, doc, &(attrType->AttrTypeNum));
 }
 
 /*----------------------------------------------------------------------
@@ -480,9 +387,11 @@ Document            doc;
    ThotAtt and its value AttrVal. Returns the corresponding Thot value.
   ----------------------------------------------------------------------*/
 #ifdef __STDC__
-void                MapMathMLAttributeValue (STRING AttrVal, AttributeType attrType, int *value)
+void        MapMathMLAttributeValue (STRING AttrVal,
+				     AttributeType attrType,
+				     int *value)
 #else
-void                MapMathMLAttributeValue (AttrVal, attrType, value)
+void        MapMathMLAttributeValue (AttrVal, attrType, value)
 STRING              AttrVal;
 AttributeType       attrType;
 int		   *value;
@@ -509,34 +418,36 @@ int		   *value;
    Search that entity in the entity table and return the corresponding value.
   ----------------------------------------------------------------------*/
 #ifdef __STDC__
-void	MapMathMLEntity (STRING entityName, STRING entityValue, int valueLength, STRING alphabet)
+void	MapMathMLEntity (STRING entityName,
+			 STRING entityValue,
+			 STRING alphabet)
 #else
-void	MapMathMLEntity (entityName, entityValue, valueLength, alphabet)
-STRING entityName;
-STRING entityValue;
-int valueLength;
-STRING alphabet;
-
+void	MapMathMLEntity (entityName, entityValue, alphabet)
+STRING  entityName;
+STRING  entityValue;
+STRING  alphabet;
 #endif
-
 {
    int	i;
+  ThotBool       found;
 
-   for (i = 0; MathEntityTable[i].charCode >= 0 &&
-	       ustrcmp (MathEntityTable[i].MentityName, entityName);
-	       i++);
-   if (!ustrcmp (MathEntityTable[i].MentityName, entityName))
-      /* entity found */
-      {
+  found = FALSE;
+  for (i = 0; MathEntityTable[i].charCode >= 0 && !found; i++)
+    found = !ustrcmp (MathEntityTable[i].charName, entityName);
+
+  if (found)
+    /* entity found */
+    {
+      i--;
       entityValue[0] = (UCHAR_T) MathEntityTable[i].charCode;
       entityValue[1] = EOS;
-      *alphabet = MathEntityTable[i].alphabet;
-      }
-   else
-      {
+      *alphabet = MathEntityTable[i].charAlphabet;
+    }
+  else
+    {
       entityValue[0] = EOS;
       *alphabet = EOS;
-      }
+    }
 }
 
 /*----------------------------------------------------------------------
@@ -545,14 +456,14 @@ STRING alphabet;
    Create an attribute EntityName containing the entity name.
   ----------------------------------------------------------------------*/
 #ifdef __STDC__
-void        MathMLEntityCreated (USTRING entityValue, Language lang, STRING entityName, Document doc)
+void    MathMLEntityCreated (USTRING entityValue, Language lang,
+			     STRING entityName, Document doc)
 #else
-void        MathMLEntityCreated (entityValue, lang, entityName, doc)
-USTRING entityValue;
-Language lang;
-STRING  entityName;
-Document doc;
-
+void    MathMLEntityCreated (entityValue, lang, entityName, doc)
+USTRING     entityValue;
+Language    lang;
+STRING      entityName;
+Document    doc;
 #endif
 {
    ElementType	 elType;
@@ -605,71 +516,117 @@ Document doc;
    TtaSetAttributeText (attr, buffer, elText, doc);
 }
 
+/*---------------------------------------------------------------------------
+   MapMathMLEntityWithExpat
+   Search that entity in the entity table and return the corresponding value.
+  ---------------------------------------------------------------------------*/
+#ifdef __STDC__
+void	MapMathMLEntityWithExpat (STRING entityName,
+				  int* entityValue,
+				  STRING alphabet)
+#else
+void	MapMathMLEntityWithExpat (entityName, entityValue, alphabet)
+STRING  entityName;
+int    *entityValue;
+STRING  alphabet;
+#endif
+{
+  int       i;
+  ThotBool  found;
+
+  found = FALSE;
+  for (i = 0; MathEntityTable[i].charCode >= 0 && !found; i++)
+    found = !ustrcmp (MathEntityTable[i].charName, entityName);
+
+  if (found)
+    /* entity found */
+    {
+      i--;
+      *entityValue = MathEntityTable[i].charCode;
+      *alphabet = MathEntityTable[i].charAlphabet;
+    }
+  else
+    *alphabet = EOS;
+}
+
 /*----------------------------------------------------------------------
-  CheckTextElement  Put the content of input buffer into the document.
+   MathMLEntityCreatedWithExpat
+   A MathML entity has been created by the XML parser.
+   Create an attribute EntityName containing the entity name.
   ----------------------------------------------------------------------*/
 #ifdef __STDC__
-static void	CheckTextElement (Element *el, Document doc)
+void  MathMLEntityCreatedWithExpat (int entityValue, Language lang,
+				    STRING  entityName, ParserData *XmlContext)
 #else
-static void     CheckTextElement (el, doc)
-Element *el;
-Document doc;
+void  MathMLEntityCreatedWithExpat (entityValue, lang, entityName, XmlContext)
+int         entityValue;
+Language    lang;
+STRING      entityName;
+ParserData *XmlContext;
 
 #endif
 {
-   ElementType parentType, elType;
-   Element     parent, new;
-   int         len;
-   Language    lang;
-   CHAR_T      alphabet;
-   CHAR_T      text[4];
-
-   len = TtaGetTextLength (*el);
-   if (len == 1)
-      /* the element contains a single character */
-      {
-      /* get that character */
-      len = 2;
-      TtaGiveTextContent (*el, text, &len, &lang);
-      alphabet = TtaGetAlphabet (lang);
-      if (text[0] != WC_EOS)
-	 /* it's not a NULL character */
-	 {
-	 /* what's the parent of that character? */
-	 parent = TtaGetParent (*el);
-	 parentType = TtaGetElementType (parent);
-	 elType = parentType;
-	 if (parentType.ElTypeNum == MathML_EL_MF &&
-	     (text[0] == TEXT('(') ||
-	      text[0] == TEXT(')') ||
-	      text[0] == TEXT('[') ||
-	      text[0] == TEXT(']') ||
-	      text[0] == TEXT('{') ||
-	      text[0] == TEXT('}')))
-	   /* It's a stretchable character in a mf element */
-	   /* Transform the text element into a Thot SYMBOL */
-	   elType.ElTypeNum = MathML_EL_SYMBOL_UNIT;
-	 else if (parentType.ElTypeNum == MathML_EL_MF &&
-		  text[0] == TEXT('|'))
-	   /* It's a vertical bar in a mf element */
-	   /* Transform the text element into a Thot GRAPHIC */
-	   {
-	     elType.ElTypeNum = MathML_EL_GRAPHICS_UNIT;
-	     text[0] = TEXT('v');
-	   }
-	 else
-	   /* a TEXT element is OK */
-	   elType.ElTypeNum = MathML_EL_TEXT_UNIT;
-	 if (elType.ElTypeNum != MathML_EL_TEXT_UNIT)
-	   {
-	     new = TtaNewElement (doc, elType);
-	     TtaInsertSibling (new, *el, FALSE, doc);
-	     TtaDeleteTree (*el, doc);
-	     *el = new;
-	     TtaSetGraphicsShape (new, (char)text[0], doc);
-	   }
-	 }
-      }
+  ElementType	 elType;
+  Element	 elText;
+  AttributeType  attrType;
+  Attribute	 attr;
+  int		 len, code;
+#define MAX_ENTITY_LENGTH 80
+  CHAR_T	 buffer[MAX_ENTITY_LENGTH];
+  
+  if (lang < 0)
+    /* Unknown entity */
+    {
+      /* By default display a question mark */
+      buffer[0] = '?';
+      buffer[1] = WC_EOS;
+      lang = TtaGetLanguageIdFromAlphabet('L');
+      /* Let's see if we can do more */
+      if (entityName[0] == '#')
+	/* It's a number */
+	{
+	  if (entityName[1] == 'x')
+	    /* It's a hexadecimal number */
+	    usscanf (&entityName[2], TEXT("%x"), &code);
+	  else
+	    /* It's a decimal number */
+	    usscanf (&entityName[1], TEXT("%d"), &code);
+	  
+	  GetFallbackCharacter (code, buffer, &lang);
+	}
+    }
+  else
+    {
+      buffer[0] = ((UCHAR_T) entityValue);
+      buffer[1] = WC_EOS;
+    }
+  
+  elType.ElTypeNum = MathML_EL_TEXT_UNIT; 
+  elType.ElSSchema = GetMathMLSSchema (XmlContext->doc);
+  elText = TtaNewElement (XmlContext->doc, elType);
+  XmlSetElemLineNumber (elText);
+  InsertXmlElement (&elText);
+  TtaSetTextContent (elText, buffer, lang, XmlContext->doc);
+  XmlContext->lastElementClosed = TRUE;
+  XmlContext->mergeText = FALSE; 
+  
+  /* Make that text leaf read-only */
+  TtaSetAccessRight (elText, ReadOnly, XmlContext->doc);
+  
+  /* Associate an attribute EntityName with the new text leaf */
+  attrType.AttrSSchema = GetMathMLSSchema (XmlContext->doc);
+  attrType.AttrTypeNum = MathML_ATTR_EntityName;
+  attr = TtaNewAttribute (attrType);
+  TtaAttachAttribute (elText, attr, XmlContext->doc);
+  
+  len = ustrlen (entityName);
+  if (len > MAX_ENTITY_LENGTH -3)
+    len = MAX_ENTITY_LENGTH -3;
+  buffer[0] = '&';
+  ustrncpy (&buffer[1], entityName, len);
+  buffer[len+1] = ';';
+  buffer[len+2] = WC_EOS;
+  TtaSetAttributeText (attr, buffer, elText, XmlContext->doc);
 }
 
 /*----------------------------------------------------------------------
@@ -1047,8 +1004,9 @@ void SetSingleIntHorizStretchAttr (el, doc, selEl)
 		if (alphabet == 'G')
 		   /* a single Symbol character */
 		   if ((int)text[0] == 172 || (int)text[0] == 174 ||
-		       (int)text[0] == 45)
-		      /* horizontal arrow or horizontal bar */
+		       (int)text[0] == 45  ||
+		       (int)text[0] == 132 || (int)text[0] == 133)
+		      /* horizontal arrow, horizontal bar, horizontal brace */
 		      {
 		      c = EOS;
 		      /* attach a IntHorizStretch attribute */
@@ -1066,12 +1024,16 @@ void SetSingleIntHorizStretchAttr (el, doc, selEl)
 			  *selEl = symbolEl;
 		      TtaDeleteTree (textEl, doc);
 		      if ((int)text[0] == 172)
-			c = '<';  /* arrow left */
-		      if ((int)text[0] == 174)
-			c = '>';  /* arrow right */
-		      if ((int)text[0] == 45)    /* - (minus) */
+			c = 'L';  /* arrow left */
+		      else if ((int)text[0] == 174)
+			c = 'R';  /* arrow right */
+		      else if ((int)text[0] == 45)    /* - (minus) */
 			/* a horizontal line in the middle of the box */
 			c = 'h'; 
+		      else if ((int)text[0] == 132)
+			c = 'o';  /* Over brace */
+		      else if ((int)text[0] == 133)
+			c = 'u';  /* Under brace */
 		      if (c != EOS)
 			TtaSetGraphicsShape (symbolEl, c, doc);
 		      }
@@ -1187,43 +1149,45 @@ void SetIntVertStretchAttr (el, doc, base, selEl)
      }
   if (operator != NULL)
      {
-	   textEl = TtaGetFirstChild (operator);
-	   if (textEl != NULL)
+     textEl = TtaGetFirstChild (operator);
+     if (textEl != NULL)
+        {
+	elType = TtaGetElementType (textEl);
+	if (elType.ElTypeNum == MathML_EL_TEXT_UNIT)
+	   {
+	   len = TtaGetTextLength (textEl);
+	   if (len == 1)
 	      {
-	      elType = TtaGetElementType (textEl);
-	      if (elType.ElTypeNum == MathML_EL_TEXT_UNIT)
-	         {
-	         len = TtaGetTextLength (textEl);
-	         if (len == 1)
+	      len = 2;
+	      TtaGiveTextContent (textEl, text, &len, &lang); 
+	      alphabet = TtaGetAlphabet (lang);
+	      if (alphabet == 'G')
+		 /* a single Symbol character */
+		 if ((int)text[0] == 242)
+		    /* Integral */
 		    {
-		    len = 2;
-		    TtaGiveTextContent (textEl, text, &len, &lang); 
-		    alphabet = TtaGetAlphabet (lang);
-		    if (alphabet == 'G')
-		       /* a single Symbol character */
-		       if ((int)text[0] == 242)
-			 /* Integral */
-			 {
-			 /* attach a IntVertStretch attribute */
-			 attrType.AttrSSchema = elType.ElSSchema;
-			 attrType.AttrTypeNum = MathML_ATTR_IntVertStretch;
-			 attr = TtaNewAttribute (attrType);
-			 TtaAttachAttribute (el, attr, doc);
-			 TtaSetAttributeValue (attr, MathML_ATTR_IntVertStretch_VAL_yes_, el, doc);
-			 /* replace the TEXT element by a Thot SYMBOL element*/
-			 elType.ElTypeNum = MathML_EL_SYMBOL_UNIT;
-			 symbolEl = TtaNewElement (doc, elType);
-			 TtaInsertSibling (symbolEl, textEl, FALSE, doc);
-			 if (selEl != NULL)
-			   if (*selEl == textEl)
-			      *selEl = symbolEl;
-			 TtaDeleteTree (textEl, doc);
-			 c = 'i';
-			 TtaSetGraphicsShape (symbolEl, c, doc);
-			 }
+		    /* attach a IntVertStretch attribute */
+		    attrType.AttrSSchema = elType.ElSSchema;
+		    attrType.AttrTypeNum = MathML_ATTR_IntVertStretch;
+		    attr = TtaNewAttribute (attrType);
+		    TtaAttachAttribute (el, attr, doc);
+		    TtaSetAttributeValue (attr,
+					  MathML_ATTR_IntVertStretch_VAL_yes_,
+					  el, doc);
+		    /* replace the TEXT element by a Thot SYMBOL element*/
+		    elType.ElTypeNum = MathML_EL_SYMBOL_UNIT;
+		    symbolEl = TtaNewElement (doc, elType);
+		    TtaInsertSibling (symbolEl, textEl, FALSE, doc);
+		    if (selEl != NULL)
+		       if (*selEl == textEl)
+			  *selEl = symbolEl;
+		    TtaDeleteTree (textEl, doc);
+		    c = 'i';
+		    TtaSetGraphicsShape (symbolEl, c, doc);
 		    }
-	         }
 	      }
+	   }
+	}
      }
 }
 
@@ -1571,68 +1535,6 @@ void CheckMTable (elMTABLE, doc, placeholder)
   CheckAllRows (elMTABLE, doc, placeholder);
 }
 
-/*----------------------------------------------------------------------
-   SetMcharContent
-   Set the content of the mchar element according to the value of
-   its name attribute
- -----------------------------------------------------------------------*/
-#ifdef __STDC__
-void SetMcharContent (Element el, Document doc)
-#else /* __STDC__*/
-void SetMcharContent (el, doc)
-  Element	el;
-  Document	doc;
-#endif /* __STDC__*/
-{
-  ElementType	elType;
-  AttributeType	attrType;
-  Attribute	attr;
-  Element       leaf;
-  int           length;
-  CHAR_T        name[MAX_ENTITY_LENGTH];
-  UCHAR_T	value[MAX_ENTITY_LENGTH];	
-  CHAR_T	alphabet;
-  Language	lang;
-
-  if (el)
-     {
-     /* get the name attribute */
-     elType = TtaGetElementType (el);
-     attrType.AttrSSchema = elType.ElSSchema;
-     attrType.AttrTypeNum = MathML_ATTR_name;
-     attr = TtaGetAttribute (el, attrType);
-     if (attr)
-        {
-	leaf = TtaGetFirstChild (el);
-	if (!leaf)
-	  /* there is no text leaf. Create one */
-	  {
-	  elType.ElTypeNum = MathML_EL_TEXT_UNIT;
-	  leaf = TtaNewElement (doc, elType);
-	  TtaInsertFirstChild (&leaf, el, doc);
-	  }
-        length = MAX_ENTITY_LENGTH - 1;
-        TtaGiveTextAttributeValue (attr, name, &length);
-	MapMathMLEntity (name, value, MAX_ENTITY_LENGTH - 1, &alphabet);
-	if (alphabet == EOS)
-	   /* unknown name */
-	   {
-	   /* by default display a question mark */
-	   value[0] = '?';
-	   value[1] = EOS;
-	   lang = TtaGetLanguageIdFromAlphabet('L');
-	   }
-	else
-	   {
-	   lang = TtaGetLanguageIdFromAlphabet(alphabet);
-	   }
-        SetElemLineNumber (leaf);
-	TtaSetTextContent (leaf, value, lang, doc);
-	TtaSetAccessRight (leaf, ReadOnly, doc);
-        }
-     }
-}
-
 /*----------------------------------------------------------------------
    SetFontstyleAttr
    The content of a MI element has been created or modified.
@@ -1941,14 +1843,17 @@ Element                 el;
 
 /*----------------------------------------------------------------------
    CheckFence
-   If el is a MO element, if it's a child of a MROW (or equivalent)
-   element and if it contains a single fence character, transform the MO
-   into a MF and the fence character into a Thot symbol.
+   If el is a MO element,
+    - if it's a large operator (&Sum; for instance), put a presentation
+      rule to enlarge the character.
+    - if it's a child of a MROW (or equivalent) element and if it contains
+      a single fence character, transform the MO into a MF and the fence
+      character into a Thot stretchable symbol.
   ----------------------------------------------------------------------*/
 #ifdef __STDC__
-static void      CheckFence (Element el, Document doc)
+void      CheckFence (Element el, Document doc)
 #else
-static void      CheckFence (el, doc)
+void      CheckFence (el, doc)
 Element                 el;
 Document		doc;
 
@@ -1963,72 +1868,82 @@ Document		doc;
    CHAR_T	 alphabet;
    UCHAR_T       text[2];
    unsigned char c;
+   PresentationValue   pval;
+   PresentationContext ctxt;
 
    elType = TtaGetElementType (el);
    if (elType.ElTypeNum == MathML_EL_MO)
      /* the element is a MO */
      {
-     if (ChildOfMRowOrInferred (el))
-        {
-        content = TtaGetFirstChild (el);
-        if (content != NULL)
-	  {
-	  elType = TtaGetElementType (content);
-	  if (elType.ElTypeNum == MathML_EL_TEXT_UNIT)
-	    {
-	    len = TtaGetTextLength (content);
-	    if (len == 1)
+     content = TtaGetFirstChild (el);
+     if (content != NULL)
+       {
+       elType = TtaGetElementType (content);
+       if (elType.ElTypeNum == MathML_EL_TEXT_UNIT)
+	 {
+	 len = TtaGetTextLength (content);
+	 if (len == 1)
+	   /* the MO element contains a single character */
+	   {
+	   len = 2;
+	   TtaGiveTextContent (content, text, &len, &lang);
+	   alphabet = TtaGetAlphabet (lang);
+	   if ((alphabet == 'G') &&
+	       ((int)text[0] == 229 || (int)text[0] == 213))  /* Sigma,  Pi */
+	     /* it's a large operator */
+	     {
+	     ctxt = TtaGetSpecificStyleContext (doc);
+	     ctxt->destroy = FALSE;
+	     /* the specific presentation to be created is not a CSS rule */
+	     ctxt->cssLevel = 0;
+	     pval.typed_data.unit = STYLE_UNIT_PERCENT;
+	     pval.typed_data.real = FALSE;
+	     pval.typed_data.value = 180;
+	     TtaSetStylePresentation (PRSize, content, NULL, ctxt, pval);
+	     }
+	   else if (ChildOfMRowOrInferred (el))
+	     /* the MO element is a child of a MROW element */
 	      {
-	      len = 2;
-	      TtaGiveTextContent (content, text, &len, &lang);
-	      alphabet = TtaGetAlphabet (lang);
-	      if (alphabet == 'L')
-		 /* it contains a single character */
-		 if (text[0] == TEXT('(') || text[0] == TEXT(')') ||
-		     text[0] == TEXT('[') || text[0] == TEXT(']') ||
-		     text[0] == TEXT('{') || text[0] == TEXT('}') ||
-		     text[0] == TEXT('|'))
-		    {
-		    /* remove the content of the MO element */
-		    TtaDeleteTree (content, doc);
-		    /* change the MO element into a MF element */
-		    ChangeTypeOfElement (el, doc, MathML_EL_MF);
+	      if ((alphabet == 'L') &&
+		  (text[0] == TEXT('(') || text[0] == TEXT(')') ||
+		   text[0] == TEXT('[') || text[0] == TEXT(']') ||
+		   text[0] == TEXT('{') || text[0] == TEXT('}') ||
+		   text[0] == TEXT('|')))
+		/* it's a stretchable parenthesis or equivalent */
+		{
+		/* remove the content of the MO element */
+		TtaDeleteTree (content, doc);
+		/* change the MO element into a MF element */
+		ChangeTypeOfElement (el, doc, MathML_EL_MF);
 		    
-		    /* is there an attribute stretchy on this mo element? */
-		    attrType.AttrSSchema = elType.ElSSchema;
-		    attrType.AttrTypeNum = MathML_ATTR_stretchy;
-		    attrStretchy = TtaGetAttribute (el, attrType);
-		    if (attrStretchy)
-		       val = TtaGetAttributeValue (attrStretchy);
-		    else
-		       val = MathML_ATTR_stretchy_VAL_true;
-		    if (val == MathML_ATTR_stretchy_VAL_true)
-		       {
-		       /* attach a IntVertStretch attribute to the MF element*/
-		       attrType.AttrTypeNum = MathML_ATTR_IntVertStretch;
-		       attr = TtaNewAttribute (attrType);
-		       TtaAttachAttribute (el, attr, doc);
-		       TtaSetAttributeValue (attr,
-				MathML_ATTR_IntVertStretch_VAL_yes_, el, doc);
-		       }
-		    /* create a new content for the MF element */
-		    if (text[0] == TEXT('|'))
-		       {
-			 elType.ElTypeNum = MathML_EL_GRAPHICS_UNIT;
-			 c = 'v';
-			 }
-		      else
-			 {
-		         elType.ElTypeNum = MathML_EL_SYMBOL_UNIT;
-		         c = (char) text[0];
-			 }
-		      content = TtaNewElement (doc, elType);
-		      TtaInsertFirstChild (&content, el, doc);
-		      TtaSetGraphicsShape (content, c, doc);
-		      }
+		/* is there an attribute stretchy on this mo element? */
+		attrType.AttrSSchema = elType.ElSSchema;
+		attrType.AttrTypeNum = MathML_ATTR_stretchy;
+		attrStretchy = TtaGetAttribute (el, attrType);
+		if (attrStretchy)
+		  val = TtaGetAttributeValue (attrStretchy);
+		else
+		  val = MathML_ATTR_stretchy_VAL_true;
+		if (val == MathML_ATTR_stretchy_VAL_true)
+		  {
+		  /* attach a IntVertStretch attribute to the MF element*/
+		  attrType.AttrTypeNum = MathML_ATTR_IntVertStretch;
+		  attr = TtaNewAttribute (attrType);
+		  TtaAttachAttribute (el, attr, doc);
+		  TtaSetAttributeValue (attr,
+					MathML_ATTR_IntVertStretch_VAL_yes_,
+					el, doc);
+		  }
+		/* create a new content for the MF element */
+		elType.ElTypeNum = MathML_EL_SYMBOL_UNIT;
+		c = (char) text[0];
+		content = TtaNewElement (doc, elType);
+		TtaInsertFirstChild (&content, el, doc);
+		TtaSetGraphicsShape (content, c, doc);
+		}
 	      }
-	    }
-	  }
+	   }
+	 }
        }
      }
 }
@@ -2307,6 +2222,7 @@ void MathMLScriptShift (doc, el, value, attr)
        /* remove the presentation rule */
        {
        ctxt->destroy = TRUE;
+       pval.typed_data.value = 0;
        TtaSetStylePresentation (PRVertPos, script, NULL, ctxt, pval);
        }
     else
@@ -2317,6 +2233,8 @@ void MathMLScriptShift (doc, el, value, attr)
        value = ParseCSSUnit (value, &pval);
        if (pval.typed_data.unit != STYLE_UNIT_INVALID)
 	  {
+	  /* the specific presentation to be created is not a CSS rule */
+	  ctxt->cssLevel = 0;
           if (attr == MathML_ATTR_superscriptshift)
 	    pval.typed_data.value = - pval.typed_data.value;
 	  TtaSetStylePresentation (PRVertPos, script, NULL, ctxt, pval);
@@ -2380,10 +2298,8 @@ int             *error;
 
 #endif
 {
-   ElementType		elType, parentType, childType;
+   ElementType		elType, parentType;
    Element		child, parent, new, prev, next;
-   AttributeType	attrType;
-   Attribute		attr;
    SSchema              MathMLSSchema;
    ThotBool             ok;
 
@@ -2392,36 +2308,13 @@ int             *error;
    elType = TtaGetElementType (el);
    MathMLSSchema = GetMathMLSSchema (doc);
 
-   if (elType.ElSSchema != MathMLSSchema)
-     /* this is not a MathML element. It's the HTML element <math>, or
-	any other element containing a MathML expression */
-     {
-     if (TtaGetFirstChild (el) == NULL && !TtaIsLeaf (elType))
-	/* this element is empty. Create a MathML element as it's child */
-	{
-	elType.ElSSchema = MathMLSSchema;
-	elType.ElTypeNum = MathML_EL_MathML;
-	new = TtaNewElement (doc, elType);
-	TtaInsertFirstChild (&new, el, doc);
-	/* Create a placeholder within the MathML element */
-        elType.ElTypeNum = MathML_EL_Construct;
-	child = TtaNewElement (doc, elType);
-	TtaInsertFirstChild (&child, new, doc);
-	attrType.AttrSSchema = elType.ElSSchema;
-	attrType.AttrTypeNum = MathML_ATTR_IntPlaceholder;
-	attr = TtaNewAttribute (attrType);
-	TtaAttachAttribute (child, attr, doc);
-	TtaSetAttributeValue (attr, MathML_ATTR_IntPlaceholder_VAL_yes_,
-			      child, doc);
-	}
-     }
-   else
+   if (elType.ElSSchema == MathMLSSchema)
      {
      switch (elType.ElTypeNum)
        {
-       case MathML_EL_TEXT_UNIT:
-	  CheckTextElement (&el, doc);
-	  break;
+       case MathML_EL_MathML:
+	  /* Create placeholders within the MathML element */
+	  CreatePlaceholders (TtaGetFirstChild (el), doc);
        case MathML_EL_MI:
 	  SetFontstyleAttr (el, doc);
 	  break;
@@ -2434,11 +2327,6 @@ int             *error;
 	  CheckFence (el, doc);
 	  break;
        case MathML_EL_MSPACE:
-	  /* create a C_Space element within the MSPACE */
-	  childType.ElSSchema = elType.ElSSchema;
-	  childType.ElTypeNum = MathML_EL_C_Space;
-	  child = TtaNewElement (doc, childType);
-	  TtaInsertFirstChild (&child, el, doc);
 	  break;
        case MathML_EL_MROW:
 	  /* Create placeholders within the MROW */
@@ -2536,11 +2424,6 @@ int             *error;
 	  /* Create placeholders within the table cell */
           CreatePlaceholders (TtaGetFirstChild (el), doc);
 	  break;
-       case MathML_EL_MCHAR:
-	 /* set the content of the mchar element according to the value of
-	    its name attribute */
-	  SetMcharContent (el, doc);
-	  break;
        case MathML_EL_MACTION:
 	  /* Create placeholders within the MACTION element */
           CreatePlaceholders (TtaGetFirstChild (el), doc);
@@ -2596,7 +2479,34 @@ void SetFontfamily (doc, el, value)
   CHAR_T           css_command[buflen+20];
  
   usprintf (css_command, TEXT("font-family: %s"), value);
-  ParseHTMLSpecificStyle (el, css_command, doc, FALSE);
+  ParseHTMLSpecificStyle (el, css_command, doc, 0, FALSE);
+}
+
+/*----------------------------------------------------------------------
+ MathMLlinethickness
+ The MathML attribute linthickness is associated with element el. Generate
+ the corresponding style property for this element. 
+ -----------------------------------------------------------------------*/
+#ifdef __STDC__
+void MathMLlinethickness (Document doc, Element el, STRING value)
+#else /* __STDC__*/
+void MathMLlinethickness (doc, el, value)
+  Document doc;
+  Element el;
+  STRING value;
+#endif /* __STDC__*/
+{
+#define buflen 50
+  CHAR_T           css_command[buflen+20];
+
+  if (ustrcmp (value, TEXT("thin")) == 0)
+     ustrcpy (value, TEXT("1pt"));
+  else if (ustrcmp (value, TEXT("medium")) == 0)
+     ustrcpy (value, TEXT("1pt"));
+  else if (ustrcmp (value, TEXT("thick")) == 0)
+     ustrcpy (value, TEXT("2pt"));
+  usprintf (css_command, TEXT("stroke-width: %s"), value);
+  ParseHTMLSpecificStyle (el, css_command, doc, 0, FALSE);
 }
 
 /*----------------------------------------------------------------------
@@ -2614,7 +2524,6 @@ void MathMLAttrToStyleProperty (doc, el, value, attr)
   int attr;
 #endif /* __STDC__*/
 {
-#define buflen 50
   CHAR_T           css_command[buflen+20];
 
   switch (attr)
@@ -2629,7 +2538,7 @@ void MathMLAttrToStyleProperty (doc, el, value, attr)
        usprintf (css_command, TEXT("padding-right: %s"), value);
        break;
     }
-  ParseHTMLSpecificStyle (el, css_command, doc, FALSE);
+  ParseHTMLSpecificStyle (el, css_command, doc, 0, FALSE);
 }
 
 /*----------------------------------------------------------------------
@@ -2656,6 +2565,7 @@ void MathMLSetScriptLevel (doc, el, value)
      /* remove the presentation rule */
      {
      ctxt->destroy = TRUE;
+     pval.typed_data.value = 0;
      TtaSetStylePresentation (PRSize, el, NULL, ctxt, pval);
      }
   else
@@ -2697,6 +2607,8 @@ void MathMLSetScriptLevel (doc, el, value)
 	   percentage = 282;
 	 pval.typed_data.value = percentage;
 	 pval.typed_data.unit = STYLE_UNIT_PERCENT;
+	 /* the specific presentation to be created is not a CSS rule */
+	 ctxt->cssLevel = 0;
 	 TtaSetStylePresentation (PRSize, el, NULL, ctxt, pval);       
 	 }
        else
@@ -2755,6 +2667,7 @@ void MathMLSpacingAttr (doc, el, value, attr)
     /* remove the presentation rule */
     {
       ctxt->destroy = TRUE;
+      pval.typed_data.value = 0;
       TtaSetStylePresentation (ruleType, el, NULL, ctxt, pval);
     }
   else
@@ -2765,12 +2678,15 @@ void MathMLSpacingAttr (doc, el, value, attr)
       value = ParseCSSUnit (value, &pval);
       /***** we should accept namedspace for width *****/
       if (pval.typed_data.unit != STYLE_UNIT_INVALID)
-	TtaSetStylePresentation (ruleType, el, NULL, ctxt, pval);
+	{
+	  /* the specific presentation to be created is not a CSS rule */
+	  ctxt->cssLevel = 0;
+	  TtaSetStylePresentation (ruleType, el, NULL, ctxt, pval);
+	}
     }
   TtaFreeMemory (ctxt);
 }
 
-
 /*----------------------------------------------------------------------
    MathMLAttributeComplete
    The XML parser has completed parsing attribute attr (as well as its value)
@@ -2811,6 +2727,7 @@ Document	doc;
        attrType.AttrTypeNum == MathML_ATTR_background_ ||
        attrType.AttrTypeNum == MathML_ATTR_fontsize ||
        attrType.AttrTypeNum == MathML_ATTR_fontfamily ||
+       attrType.AttrTypeNum == MathML_ATTR_linethickness ||
        attrType.AttrTypeNum == MathML_ATTR_lspace ||
        attrType.AttrTypeNum == MathML_ATTR_rspace ||
        attrType.AttrTypeNum == MathML_ATTR_scriptlevel ||
@@ -2837,6 +2754,9 @@ Document	doc;
 	     case MathML_ATTR_fontfamily:
 	       SetFontfamily (doc, el, value);
 	       break;
+	     case MathML_ATTR_linethickness:
+	       MathMLlinethickness (doc, el, value);
+	       break;
 	     case MathML_ATTR_fontsize:
 	     case MathML_ATTR_lspace:
 	     case MathML_ATTR_rspace:
