@@ -243,6 +243,11 @@
 #define ATTR7ITEMSDIALOG                539
 #define ATTR8ITEMSDIALOG                540
 #define ATTR9ITEMSDIALOG                541
+#define ANNOTMENU                       542
+#define INITCONFIRM3LDIALOG1            543
+#define ANNOTFILTERMENU                 544
+#define MAKEIDMENU                      545
+#define SAVEASDIALOG1                   546
 #define ID_CONFIRM                      1000
 #define ID_DONE                         1001
 #define IDC_URLEDIT                     1002
@@ -595,6 +600,7 @@
 #define IDC_PROFILENAME                 1463
 #define IDC_PROXYDOMAIN                 1464
 #define IDC_ONLYPROXY                   1465
+#define IDC_FILTERBYSERVER              1466
 #define IDC_ABOUT2                      1469
 #define IDC_EDIT_NAME                   1470
 #define IDC_VERSION                     1471
@@ -638,6 +644,35 @@
 #define IDC_CURWORD                     1506
 #define IDC_LANG                        1507
 #define IDC_URL_TEXT                    1510
+#define IDC_ANNOTUSER                   1511
+#define IDC_TANNOTUSER                  1512
+#define IDC_ANNOTSERVERS                1513
+#define IDC_TANNOTSERVERS               1514
+#define IDC_TANNOTPOSTSERVER            1515
+#define IDC_ANNOTPOSTSERVER             1516
+#define IDC_TANNOTAUTOLOAD              1518
+#define IDC_ANNOTAUTOLOAD               1519
+#define ID_ANNOTSHOW                    1521
+#define ID_ANNOTHIDE                    1522
+#define ID_ANNOTSHOWALL                 1523
+#define ID_ANNOTHIDEALL                 1524
+#define IDC_FILTERBYAUTHOR              1525
+#define IDC_FILTERBYTYPE                1526
+#define IDC_TFILTEROPTIONS              1528
+#define IDC_TFILTERHIDDEN               1529
+#define IDC_FILTERSEL                   1531
+#define ID_CREATEID                     1533
+#define ID_REMOVEID                     1534
+#define IDC_IDC_TIDELEMNAME             1535
+#define IDC_IDELEMNAME                  1536
+#define IDC_IDAPPLYTODOC                1538
+#define IDC_IDAPPLYTOSEL                1539
+#define IDC_IDSTATUS                    1540
+#define IDC_TAUTHREALM                  1542
+#define IDC_TAUTHSERVER                 1543
+#define IDC_TAUTHINFO                   1544
+#define IDC_TPASSWDEDIT                 1545
+#define IDC_TNAMEEDIT                   1546
 #define ID_BUTTON40017                  40017
 #define ID_BUTTON40018                  40018
 #define ID_BUTTON40019                  40019
@@ -682,9 +717,9 @@
 // 
 #ifdef APSTUDIO_INVOKED
 #ifndef APSTUDIO_READONLY_SYMBOLS
-#define _APS_NEXT_RESOURCE_VALUE        538
+#define _APS_NEXT_RESOURCE_VALUE        546
 #define _APS_NEXT_COMMAND_VALUE         40052
-#define _APS_NEXT_CONTROL_VALUE         1511
+#define _APS_NEXT_CONTROL_VALUE         1547
 #define _APS_NEXT_SYMED_VALUE           101
 #endif
 #endif
