@@ -1,19 +1,10 @@
 /*
  *
- *  (c) COPYRIGHT INRIA, 1996.
+ *  (c) COPYRIGHT INRIA, 1996-2000
  *  Please first read the full copyright statement in file COPYRIGHT.
  *
  */
 
-/*
- * Warning:
- * This module is part of the Thot library, which was originally
- * developed in French. That's why some comments are still in
- * French, but their translation is in progress and the full module
- * will be available in English in the next release.
- * 
- */
- 
 /*
  * This module handles document contents: text and graphics
  *
@@ -52,13 +43,10 @@
   ----------------------------------------------------------------------*/
 #ifdef __STDC__
 PtrTextBuffer       NewTextBuffer (PtrTextBuffer pBuf)
-
 #else  /* __STDC__ */
 PtrTextBuffer       NewTextBuffer (pBuf)
 PtrTextBuffer       pBuf;
-
 #endif /* __STDC__ */
-
 {
    PtrTextBuffer       pNewBuf;
 
@@ -79,13 +67,10 @@ PtrTextBuffer       pBuf;
   ----------------------------------------------------------------------*/
 #ifdef __STDC__
 void                CreateTextBuffer (PtrElement pEl)
-
 #else  /* __STDC__ */
 void                CreateTextBuffer (pEl)
 PtrElement          pEl;
-
 #endif /* __STDC__ */
-
 {
    PtrTextBuffer       pBuf;
 
@@ -99,16 +84,12 @@ PtrElement          pEl;
    	DeleteTextBuffer  supprime un buffer de texte dont le pointeur est	
    	passe' en parametre.						
   ----------------------------------------------------------------------*/
-
 #ifdef __STDC__
 void                DeleteTextBuffer (PtrTextBuffer * pBuf)
-
 #else  /* __STDC__ */
 void                DeleteTextBuffer (pBuf)
 PtrTextBuffer      *pBuf;
-
 #endif /* __STDC__ */
-
 {
    if (*pBuf != NULL)
      {
@@ -127,25 +108,23 @@ PtrTextBuffer      *pBuf;
    	pEl: l'element de texte a couper.				
    	rank: indice du caractere devant lequel on coupe		
    	pDoc: document auquel appartient l'element a couper
+        elBreak: appel par la procedure BreakElement
 
   au retour:
 	pSecondPart: l'element de texte correspondant a la partie
 		     apres la coupure.
   ----------------------------------------------------------------------*/
-
 #ifdef __STDC__
-void                SplitTextElement (PtrElement pEl, int rank, PtrDocument pDoc, ThotBool withAppEvent, PtrElement *pSecondPart)
-
+void                SplitTextElement (PtrElement pEl, int rank, PtrDocument pDoc, ThotBool withAppEvent, PtrElement *pSecondPart, ThotBool elBreak)
 #else  /* __STDC__ */
-void                SplitTextElement (pEl, rank, pDoc, withAppEvent, pSecondPart)
+void                SplitTextElement (pEl, rank, pDoc, withAppEvent, pSecondPart, elBreak)
 PtrElement          pEl;
 int                 rank;
 PtrDocument         pDoc;
 ThotBool            withAppEvent;
 PtrElement	    *pSecondPart;
-
+ThotBool            elBreak;
 #endif /* __STDC__ */
-
 {
    PtrTextBuffer       pBuf;
    int                 i, len;
@@ -278,7 +257,10 @@ PtrElement	    *pSecondPart;
 		       notifyTxt.document = (Document) IdentDocument (pDoc);
 		       notifyTxt.element = (Element) pAsc;
 		       notifyTxt.target = (Element) pEl;
-		       notifyTxt.targetdocument = (Document) IdentDocument (pDoc);
+		       if (elBreak)
+		         notifyTxt.targetdocument = 0;
+		       else
+		         notifyTxt.targetdocument = (Document) IdentDocument (pDoc);
 		       CallEventType ((NotifyEvent *) & notifyTxt, FALSE);
 		       pAsc = pAsc->ElParent;
 		    }
@@ -311,11 +293,9 @@ PtrElement	    *pSecondPart;
         Cette fonction retourne TRUE si la fusion a lieu,
                                 FALSE sinon.
   ----------------------------------------------------------------------*/
-
 #ifdef __STDC__
 ThotBool            MergeTextElements (PtrElement pEl, PtrElement * pFreeEl, PtrDocument pDoc,
 				       ThotBool withAppEvent, ThotBool removeAbsBox)
-
 #else  /* __STDC__ */
 ThotBool            MergeTextElements (pEl, pFreeEl, pDoc, withAppEvent, removeAbsBox)
 PtrElement          pEl;
@@ -323,9 +303,7 @@ PtrElement         *pFreeEl;
 PtrDocument         pDoc;
 ThotBool            withAppEvent;
 ThotBool            removeAbsBox;
-
 #endif /* __STDC__ */
-
 {
    PtrElement          pEl1, pEl2;
    PtrElement          pSibling;
@@ -543,17 +521,13 @@ ThotBool            removeAbsBox;
    	de buffers de texte pointee par pBuf et retourne un pointeur	
    	sur le premier buffer de la copie realisee.			
   ----------------------------------------------------------------------*/
-
 #ifdef __STDC__
 PtrTextBuffer       CopyText (PtrTextBuffer pBuf, PtrElement pEl)
-
 #else  /* __STDC__ */
 PtrTextBuffer       CopyText (pBuf, pEl)
 PtrTextBuffer       pBuf;
 PtrElement          pEl;
-
 #endif /* __STDC__ */
-
 {
    PtrTextBuffer       pBufSource1, pBufSource2, pBufCopy1, pBufCopy2,
                        ret;
@@ -586,17 +560,13 @@ PtrElement          pEl;
    un texte contenu dans une suite de buffers. Retourne TRUE si    
    les contenus sont egaux.                                        
   ----------------------------------------------------------------------*/
-
 #ifdef __STDC__
 ThotBool            StringAndTextEqual (STRING String, PtrTextBuffer pBuf)
-
 #else  /* __STDC__ */
 ThotBool            StringAndTextEqual (String, pBuf)
 STRING              String;
 PtrTextBuffer       pBuf;
-
 #endif /* __STDC__ */
-
 {
    ThotBool            equal;
    int                 l, lenRest;
@@ -645,17 +615,13 @@ PtrTextBuffer       pBuf;
    TextsEqual compare deux textes contenus dans des suites de     
    buffers. Retourne TRUE si les deux textes sont egaux.		
   ----------------------------------------------------------------------*/
-
 #ifdef __STDC__
 ThotBool            TextsEqual (PtrTextBuffer pBuf1, PtrTextBuffer pBuf2)
-
 #else  /* __STDC__ */
 ThotBool            TextsEqual (pBuf1, pBuf2)
 PtrTextBuffer       pBuf1;
 PtrTextBuffer       pBuf2;
-
 #endif /* __STDC__ */
-
 {
    ThotBool            equal;
    PtrTextBuffer       pTB1, pTB2;
@@ -776,18 +742,14 @@ int                *len;
    de buffers.                                                     
    Retourne dans LgCopiee le nombre de caracteres copie's.         
   ----------------------------------------------------------------------*/
-
 #ifdef __STDC__
 void                CopyStringToText (CHAR_T* srceStrn, PtrTextBuffer pCopyBuf, int *LgCopiee)
-
 #else  /* __STDC__ */
 void                CopyStringToText (srceStrn, pCopyBuf, LgCopiee)
 CHAR_T*             srceStrn;
 PtrTextBuffer       pCopyBuf;
 int                *LgCopiee;
-
 #endif /* __STDC__ */
-
 {
    PtrTextBuffer       pTBDest;
    CHAR_T*             pSrce;
@@ -840,18 +802,14 @@ int                *LgCopiee;
    A l'appel, len contient la longueur maximum a copier.           
    Au retour, len contient le nombre de caracteres copie's.        
   ----------------------------------------------------------------------*/
-
 #ifdef __STDC__
 void                CopyTextToString (PtrTextBuffer pSrceBuf, CHAR_T* pStrCpy, int *len)
-
 #else  /* __STDC__ */
 void                CopyTextToString (pSrceBuf, pStrCpy, len)
 PtrTextBuffer       pSrceBuf;
 CHAR_T*             pStrCpy;
 int                *len;
-
 #endif /* __STDC__ */
-
 {
    int                 LgMax;
    int                 l;
@@ -925,19 +883,15 @@ PtrTextBuffer       pBuf;
    des points de contro^le de la polyline.                 
    On saute le point 0 (coordonnees du point limite).      
   ----------------------------------------------------------------------*/
-
 #ifdef __STDC__
 void                AddPointInPolyline (PtrTextBuffer firstBuffer, int rank, int x, int y)
-
 #else  /* __STDC__ */
 void                AddPointInPolyline (firstBuffer, rank, x, y)
 PtrTextBuffer       firstBuffer;
 int                 rank;
 int                 x;
 int                 y;
-
 #endif /* __STDC__ */
-
 {
    PtrTextBuffer       pBuf, pNextBuf;
    int                 i, j;
@@ -1033,17 +987,13 @@ int                 y;
    Le pointeur firstBuffer peut e^tre modifie' par la      
    proce'dure.                                             
   ----------------------------------------------------------------------*/
-
 #ifdef __STDC__
 void                DeletePointInPolyline (PtrTextBuffer * firstBuffer, int rank)
-
 #else  /* __STDC__ */
 void                DeletePointInPolyline (firstBuffer, rank)
 PtrTextBuffer      *firstBuffer;
 int                 rank;
-
 #endif /* __STDC__ */
-
 {
    PtrTextBuffer       pBuf;
    int                 j;
@@ -1084,19 +1034,15 @@ int                 rank;
    de la polyline.
    On saute le point 0 (coordonnees du point limite).      
   ----------------------------------------------------------------------*/
-
 #ifdef __STDC__
 void                ModifyPointInPolyline (PtrTextBuffer firstBuffer, int rank, int x, int y)
-
 #else  /* __STDC__ */
 void                ModifyPointInPolyline (firstBuffer, rank, x, y)
 PtrTextBuffer       firstBuffer;
 int                 rank;
 int                 x;
 int                 y;
-
 #endif /* __STDC__ */
-
 {
    PtrTextBuffer       pBuf;
 
@@ -1193,7 +1139,7 @@ Language           *language;
    int                 len, l;
 
    UserErrorCode = 0;
-   if (element == NULL)
+   if (element == NULL || buffer == NULL)
 	TtaError (ERR_invalid_parameter);
    else if (!((PtrElement) element)->ElTerminal)
 	TtaError (ERR_invalid_element_type);
@@ -1205,17 +1151,18 @@ Language           *language;
 	len = 0;
 	pBuf = ((PtrElement) element)->ElText;
 	ptr = buffer;
-	while (pBuf != NULL && len < (*length) - 1)
-	  {
-	     if ((*length) < len + pBuf->BuLength + 1)
+	if (length > 0)
+	  while (pBuf != NULL && len < (*length) - 1)
+	    {
+	      if ((*length) < len + pBuf->BuLength + 1)
 		l = (*length) - len;
-	     else
+	      else
 		l = pBuf->BuLength + 1;
-	     ustrncpy (ptr, pBuf->BuContent, l);
-	     ptr = ptr + (l - 1);
-	     len = len + (l - 1);
-	     pBuf = pBuf->BuNext;
-	  }
+	      ustrncpy (ptr, pBuf->BuContent, l);
+	      ptr = ptr + (l - 1);
+	      len = len + (l - 1);
+	      pBuf = pBuf->BuNext;
+	    }
 	*length = len;
 	*ptr = EOS;
 	*language = ((PtrElement) element)->ElLanguage;
