@@ -253,7 +253,7 @@ int                *info;
 {
    /* Enleve la procedure de Callback */
    /* Detruit la fenetre si elle existe encore */
-   if (FrRef[frame] != 0 && frame > 0)
+   if (frame > 0 && FrRef[frame] != 0)
       ViewClosed (frame);
 }
 
@@ -516,6 +516,8 @@ int                 value;
    int        delta, Xpos, Ypos, width, height;
    int        sPos, nbPages, remaining;
 
+   if (frame < 0)
+		return;
    /* do not redraw it if in NoComputedDisplay mode */
    if (documentDisplayMode[FrameTable[frame].FrDoc - 1] == NoComputedDisplay)
       return;
@@ -1349,32 +1351,39 @@ LPARAM      lParam;
   HWND                hwndToolTip;
   RECT                rect;
   RECT                rWindow;
+  LPNMHDR             pnmh;
   int                 frame = GetMainFrameNumber (hwnd);
   int                 view;
   int                 frameNDX;
+  int                 idCtrl;
+  int                 cx;
+  int                 cy;
+  int                 cyStatus;
+  int                 cxVSB;
+  int                 cyHSB;
+  int                 cyTB;
+  int                 x, y;
+  int                 index = 0;
+  int                 cyTxtZone;
+  DWORD               dwStyle;
   ThotBool            assoc;
 
   if (frame != -1)
     currentFrame = frame;
-
   GetWindowRect (hwnd, &rect);
   switch (mMsg)
   {
   case WM_CREATE:
     /* Create toolbar  */
     ThotTBBitmap.hInst = hInstance;
-    ThotTBBitmap.nID   = IDR_TOOLBAR;
-    
+    ThotTBBitmap.nID   = IDR_TOOLBAR;    
     ToolBar = CreateWindow (TOOLBARCLASSNAME, NULL, dwToolBarStyles,
 			    0, 0, 0, 0, hwnd, (HMENU) 1, hInstance, 0);
-    
     SendMessage (ToolBar, TB_BUTTONSTRUCTSIZE, (WPARAM) sizeof (TBBUTTON), 0L);
-    
     if ((SendMessage (ToolBar, TB_ADDBITMAP, (WPARAM) MAX_BUTTON, (LPARAM) (LPTBADDBITMAP) &ThotTBBitmap)) == -1)
       WinErrorBox (NULL, TEXT("WndProc: WM_CREATE"));
     
     hwndToolTip = ToolBar_GetToolTips (ToolBar);
-    
     if (dwToolBarStyles & TBSTYLE_TOOLTIPS)
       InitToolTip (ToolBar);	
     
@@ -1415,131 +1424,128 @@ LPARAM      lParam;
   case WM_ENTER:
     hwndTextEdit = GetFocus ();
     WIN_APP_TextCallback (hwndTextEdit, frame);
-    SetFocus (FrRef [frame]);
+    if (frame != -1)
+      SetFocus (FrRef [frame]);
     return 0L;
 
   case WM_SYSKEYDOWN:
   case WM_KEYDOWN:
-       if (frame != -1)
-          SetFocus (FrRef [frame]);
+    if (frame != -1)
+      SetFocus (FrRef [frame]);
     SendMessage (FrRef [frame], mMsg, wParam, lParam);
     return 0L;
 
   case WM_DROPFILES:
-       if (frame != -1)
-          SetFocus (FrRef [frame]);
+    if (frame != -1)
+      SetFocus (FrRef [frame]);
     SendMessage (FrRef [frame], mMsg, wParam, lParam);
     return 0L;
 	
   case WM_IME_CHAR:
   case WM_SYSCHAR:
   case WM_CHAR:
-       if (frame != -1)
-          SetFocus (FrRef [frame]);
+    if (frame != -1)
+      SetFocus (FrRef [frame]);
     SendMessage (FrRef [frame], mMsg, wParam, lParam);
     return 0L;
 
-  case WM_NOTIFY: {
-    LPNMHDR pnmh = (LPNMHDR) lParam;
-    int idCtrl = (int) wParam;
+  case WM_NOTIFY:
+    pnmh = (LPNMHDR) lParam;
+    idCtrl = (int) wParam;
     
     /*Toolbar notifications */
     if ((pnmh->code >= TBN_LAST) && (pnmh->code <= TBN_FIRST))
       return ToolBarNotify (frame, hwnd, wParam, lParam);
     
     /* Fetch tooltip text */
-    if (pnmh->code == TTN_NEEDTEXT) {
-      LPTOOLTIPTEXT lpttt = (LPTOOLTIPTEXT) lParam;
-      CopyToolTipText (frame, lpttt);
-    }
+    if (pnmh->code == TTN_NEEDTEXT)
+      {
+        LPTOOLTIPTEXT lpttt = (LPTOOLTIPTEXT) lParam;
+        CopyToolTipText (frame, lpttt);
+      }
     return 0L;
-  }
 
   case WM_COMMAND:
-    if (LOWORD (wParam) >= TBBUTTONS_BASE) {
-      buttonCommand = TRUE;
-      APP_ButtonCallback (FrameTable[frame].Button[LOWORD (wParam) - TBBUTTONS_BASE], frame, "\n");
-    } else {
-      buttonCommand = FALSE;
-      WIN_ThotCallBack (hwnd, wParam, lParam);
-    }
+    if (LOWORD (wParam) >= TBBUTTONS_BASE)
+      {
+        buttonCommand = TRUE;
+        APP_ButtonCallback (FrameTable[frame].Button[LOWORD (wParam) - TBBUTTONS_BASE], frame, "\n");
+      }
+    else
+      {
+	buttonCommand = FALSE;
+	WIN_ThotCallBack (hwnd, wParam, lParam);
+      }
     return 0L;
 
   case WM_CLOSE:
-       if (frame <= MAX_FRAME) {
-          GetDocAndView (frame, &pDoc, &view, &assoc);
-		  if (pDoc && view)
-           CloseView (pDoc, view, assoc);
-           FrMainRef[frame] = 0;
-	   }
-    return 0L;
-
   case WM_DESTROY:
-       FrMainRef[frame] = 0;
-       if (frame <= MAX_FRAME) {
-          for (frameNDX = 0; frameNDX <= MAX_FRAME; frameNDX++)
-              if (FrMainRef[frameNDX] != 0) 
-                 /* there is still an active frame */
-                 return 0L;
-          TtaQuit();
-	   }
-    PostQuitMessage (0);
+    if (frame >= 0 && frame <= MAX_FRAME)
+	{
+      GetDocAndView (frame, &pDoc, &view, &assoc);
+      if (pDoc && view)
+        CloseView (pDoc, view, assoc);
+      if (FrameTable[frame].FrDoc == 0)
+        FrMainRef[frame] = 0;
+ 
+      if (mMsg == WM_DESTROY)
+	  {
+        for (frameNDX = 0; frameNDX <= MAX_FRAME; frameNDX++)
+          if (FrMainRef[frameNDX] != 0)
+		  {
+          /* there is still an active frame */
+          FrMainRef[frame] = 0;
+          return 0L;
+		  }
+        TtaQuit();
+	  }
+	}
+    if (mMsg == WM_DESTROY)
+      PostQuitMessage (0);
     return 0L;
         
   case WM_SIZE:
-	{
-    int    cx = LOWORD (lParam);
-    int    cy = HIWORD (lParam);
-    int    cyStatus;
-    int    cxVSB;
-    int    cyHSB;
-    int    cyTB;
-    int    x, y;
-    int    index = 0;
-    int    cyTxtZone;
-    DWORD  dwStyle;
-
+    cx = LOWORD (lParam);
+    cy = HIWORD (lParam);
     /* Adjust toolbar size. */
     if (IsWindowVisible (WinToolBar[frame]))
-	{
-      dwStyle = GetWindowLong (WinToolBar[frame], GWL_STYLE);
-      
-      if (dwStyle & CCS_NORESIZE)
-	    MoveWindow (WinToolBar[frame], 0, 0, cx, cyToolBar, FALSE);
-      else
-	    ToolBar_AutoSize (WinToolBar[frame]);
+      {
+	dwStyle = GetWindowLong (WinToolBar[frame], GWL_STYLE);
+	if (dwStyle & CCS_NORESIZE)
+	  MoveWindow (WinToolBar[frame], 0, 0, cx, cyToolBar, FALSE);
+	else
+	  ToolBar_AutoSize (WinToolBar[frame]);
 
-      InvalidateRect (WinToolBar[frame], NULL, TRUE);
-      GetWindowRect (WinToolBar[frame], &rWindow);
-      ScreenToClient (hwnd, (LPPOINT) &rWindow.left);
-      ScreenToClient (hwnd, (LPPOINT) &rWindow.right);
-      cyTB = rWindow.bottom - rWindow.top;
-    }
-	else 
+	InvalidateRect (WinToolBar[frame], NULL, TRUE);
+	GetWindowRect (WinToolBar[frame], &rWindow);
+	ScreenToClient (hwnd, (LPPOINT) &rWindow.left);
+	ScreenToClient (hwnd, (LPPOINT) &rWindow.right);
+	cyTB = rWindow.bottom - rWindow.top;
+      }
+    else 
       cyTB = 0;
     
     cyTxtZone = cyTB;
-
     /* Adjust text zones */
     for (index = 0; index < MAX_TEXTZONE; index++)
-	{
-      if (FrameTable[frame].Text_Zone[index] &&
-		  IsWindowVisible (FrameTable[frame].Text_Zone[index]))
+      {
+	if (FrameTable[frame].Text_Zone[index] &&
+	    IsWindowVisible (FrameTable[frame].Text_Zone[index]))
 	  {
 	    MoveWindow (FrameTable[frame].Label[index], 15, cyTxtZone + 5, 70, 20, TRUE);
 	    MoveWindow (FrameTable[frame].Text_Zone[index], 85, cyTxtZone + 5, cx - 100, 20, TRUE);
 	    cyTxtZone += 25;
+	  }
       }
-    }
 
     /* Adjust status bar size. */
     if (IsWindowVisible (FrameTable[frame].WdStatus))
-	{
-      GetWindowRect (FrameTable[frame].WdStatus, &rWindow);
-      cyStatus = rWindow.bottom - rWindow.top;
-      MoveWindow (FrameTable[frame].WdStatus, 0, cy - cyStatus, cx, cyStatus, TRUE);
-    }
-	else
+      {
+	GetWindowRect (FrameTable[frame].WdStatus, &rWindow);
+	cyStatus = rWindow.bottom - rWindow.top;
+	MoveWindow (FrameTable[frame].WdStatus, 0, cy - cyStatus, cx, cyStatus, TRUE);
+      }
+    else
       cyStatus = 0;
 
     /* Adjust Vertical scroll bar */
@@ -1566,12 +1572,10 @@ LPARAM      lParam;
     SetScrollRange (FrameTable[frame].WdScrollV, SB_CTL, 0, cy, TRUE);
     SetScrollRange (FrameTable[frame].WdScrollH, SB_CTL, 0, cx, TRUE);
     return 0L;
-	}
 
   default: 
     return (DefWindowProc (hwnd, mMsg, wParam, lParam));
   }
-  /*return (DefWindowProc (hwnd, mMsg, wParam, lParam));*/
 }
 
 /* -------------------------------------------------------------------
@@ -1740,7 +1744,7 @@ LPARAM lParam;
             case WM_SYSCHAR:
             case WM_CHAR:
                  /* TtaAbortShowDialogue (); */
-				 key = (int) wParam;
+		 key = (int) wParam;
                  if (WIN_TtaHandleMultiKeyEvent (mMsg, wParam, lParam, (CHAR_T*)&key))
                     /* WIN_CharTranslation (FrRef[frame], frame, mMsg, wParam, lParam); */
                     WIN_CharTranslation (FrRef[frame], frame, mMsg, (WPARAM) key, lParam);
@@ -1964,14 +1968,12 @@ void               *event;
 	   else if (t1 + (Time) DoubleClickDelay > ev->xbutton.time)
 	     {
 	       TtaAbortShowDialogue ();
-	       TtaLockMainLoop();
 	       TtaFetchOneEvent (&event);
 	       while (event.type != ButtonRelease)
 		 {
 		   TtaHandleOneEvent (&event);
 		   TtaFetchOneEvent (&event);
 		 }
-	       TtaUnlockMainLoop();
 
 	       /* memorise la position de la souris */
 	       if (ClickFrame == frame
@@ -1997,7 +1999,6 @@ void               *event;
 	       
 	       /* Regarde s'il s'agit d'un drag ou d'une simple marque d'insertion */
 	       comm = 0;	/* il n'y a pas de drag */
-	       TtaLockMainLoop();
 	       TtaFetchOneEvent (&event);
 	       FrameToView (frame, &document, &view);
 	       h = FrameTable[frame].FrHeight;
@@ -2028,7 +2029,6 @@ void               *event;
 		   TtaFetchOrWaitEvent (&event);
 		 }
 	       TtaHandleOneEvent (&event);
-	       TtaUnlockMainLoop();
 	       
 	       /* S'il y a un drag on termine la selection */
 	       FrameToView (frame, &document, &view);
@@ -2348,20 +2348,18 @@ PtrAbstractBox     *pave;
    ClickFrame = 0;
    ClickX = 0;
    ClickY = 0;
-   TtaLockMainLoop();
    while (ClickIsDone == 1)
      {
 #ifndef _WINDOWS 
-         TtaFetchOneEvent (&event);
-         TtaHandleOneEvent (&event);
+       TtaFetchOneEvent (&event);
+       TtaHandleOneEvent (&event);
 #else /* _WINDOWS */
-         GetMessage (&event, NULL, 0, 0);
-         curFrame = GetFrameNumber (event.hwnd);
-         TtaHandleOneWindowEvent (&event);
-         SetCursor (cursor);
+       GetMessage (&event, NULL, 0, 0);
+       curFrame = GetFrameNumber (event.hwnd);
+       TtaHandleOneEvent (&event);
+       SetCursor (cursor);
 #endif /* !_WINDOWS */
      }
-   TtaUnlockMainLoop();
 
    /* Restauration du curseur */
    for (i = 1; i <= MAX_FRAME; i++)
