@@ -1,6 +1,6 @@
 /*
  *
- *  (c) COPYRIGHT INRIA, 1996 - 2000
+ *  (c) COPYRIGHT INRIA, 1996-2000
  *  Please first read the full copyright statement in file COPYRIGHT.
  *
  */
@@ -18,8 +18,8 @@
 #include "fileaccess.h"
 
 #ifdef NODISPLAY
-/*** For the ThotKernel, variables FirstSelectedElement and
-     LastSelectedElement, are defined here, to be used at the end of
+/*** For ThotKernel, variables FirstSelectedElement and
+     LastSelectedElement are defined here, to be used at the end of
      function MergeTextElements  ***/
 #undef THOT_EXPORT
 #define THOT_EXPORT
@@ -82,31 +82,61 @@ PtrDocument         pDoc;
 }
 
 /*----------------------------------------------------------------------
- * TtaGetDocumentCharset
- *----------------------------------------------------------------------*/
+  TtaGetDocumentCharset gets the document charset
+  Returns UNDEFINED_CHARSET when the document uses the default charset.
+ ----------------------------------------------------------------------*/
 #ifdef __STDC__
-CHARSET TtaGetDocumentCharset (PtrDocument pDoc)
+CHARSET     TtaGetDocumentCharset (Document document)
 #else  /* __STDC__ */
-CHARSET TtaGetDocumentCharset (pDoc)
-PtrDocument pDoc;
+CHARSET     TtaGetDocumentCharset (document)
+Document     document;
 #endif /* __STDC__ */
 {
-    return pDoc->Charset;
+  PtrDocument pDoc;
+
+  UserErrorCode = 0;
+  /* verifies the parameter document */
+  if (document < 1 || document > MAX_DOCUMENTS || LoadedDocument[document - 1] == NULL)
+    {
+      TtaError (ERR_invalid_document_parameter);
+      return (UNDEFINED_CHARSET);
+    }
+  else
+    {
+      pDoc = LoadedDocument[document - 1];
+      if (pDoc->DocDefaultCharset)
+	return (UNDEFINED_CHARSET);
+      else
+      return (pDoc->DocCharset);
+    }
 }
 
 
 /*----------------------------------------------------------------------
- * TtaSetDocumentCharset
- *----------------------------------------------------------------------*/
+  TtaSetDocumentCharset sets the document charset
+ ----------------------------------------------------------------------*/
 #ifdef __STDC__
-void TtaSetDocumentCharset (PtrDocument pDoc, CHARSET charSet)
+void         TtaSetDocumentCharset (Document document, CHARSET charSet)
 #else  /* __STDC__ */
-void TtaSetDocumentCharset (pDoc, charSet)
-PtrDocument pDoc;
-CHARSET     charSet;
+void         TtaSetDocumentCharset (document, charSet)
+Document     document;
+CHARSET      charSet;
 #endif /* __STDC__ */
 {
-    pDoc->Charset = charSet;
+  PtrDocument pDoc;
+
+  UserErrorCode = 0;
+  /* verifies the parameter document */
+  if (document < 1 || document > MAX_DOCUMENTS)
+    TtaError (ERR_invalid_document_parameter);
+  else if (LoadedDocument[document - 1] == NULL)
+    TtaError (ERR_invalid_document_parameter);
+  else
+    {
+      pDoc = LoadedDocument[document - 1];
+      pDoc->DocCharset = charSet;
+      pDoc->DocDefaultCharset = FALSE;
+    }
 }
 
 /*----------------------------------------------------------------------
@@ -132,52 +162,51 @@ STRING              documentName;
 int                 accessMode;
 #endif /* __STDC__ */
 {
-   PtrDocument         pDoc;
-   Document            document;
-   int                 lg;
-   ThotBool            ok;
+  PtrDocument         pDoc;
+  Document            document;
+  int                 lg;
+  ThotBool            ok;
 
-   UserErrorCode = 0;
-   document = 0;
-   /* initializes the document context */
-   CreateDocument (&pDoc);
-   if (pDoc == NULL)
-      /* too many opened documents */
+  UserErrorCode = 0;
+  document = 0;
+  /* initializes the document context */
+  CreateDocument (&pDoc, &document);
+  if (pDoc == NULL)
+    /* too many opened documents */
      TtaError (ERR_too_many_documents);
-   else
-     {
-	lg = ustrlen (documentName);
-	if (lg >= MAX_NAME_LENGTH)
-	   TtaError (ERR_string_too_long);
-	else
-	  {
-	     ustrncpy (pDoc->DocDName, documentName, MAX_NAME_LENGTH);
-	     pDoc->DocDName[MAX_NAME_LENGTH - 1] = EOS;
+  else
+    {
+      lg = ustrlen (documentName);
+      if (lg >= MAX_NAME_LENGTH)
+	TtaError (ERR_string_too_long);
+      else
+	{
+	  ustrncpy (pDoc->DocDName, documentName, MAX_NAME_LENGTH);
+	  pDoc->DocDName[MAX_NAME_LENGTH - 1] = EOS;
 	     /* suppresses the .PIV suffix if found */
-	     if (lg > 4)
-		if (ustrcmp (&(pDoc->DocDName[lg - 4]), TEXT(".PIV")) == 0)
-		   pDoc->DocDName[lg - 4] = EOS;
-	     GetDocIdent (&pDoc->DocIdent, pDoc->DocDName);
-	     ustrncpy (pDoc->DocDirectory, DocumentPath, MAX_PATH);
-	     ok = OpenDocument (pDoc->DocDName, pDoc, TRUE, FALSE, NULL,
-				FALSE, TRUE);
-	     if (!ok)
-		/* acces failure to an objectpivot */
-	       {
-		  UnloadDocument (&pDoc);
-		  TtaError (ERR_cannot_open_pivot_file);
+	  if (lg > 4)
+	    if (ustrcmp (&(pDoc->DocDName[lg - 4]), TEXT(".PIV")) == 0)
+	      pDoc->DocDName[lg - 4] = EOS;
+	  GetDocIdent (&pDoc->DocIdent, pDoc->DocDName);
+	  ustrncpy (pDoc->DocDirectory, DocumentPath, MAX_PATH);
+	  ok = OpenDocument (pDoc->DocDName, pDoc, TRUE, FALSE, NULL,
+			     FALSE, TRUE);
+	  if (!ok)
+	    /* acces failure to an objectpivot */
+	    {
+	      UnloadDocument (&pDoc);
+	      TtaError (ERR_cannot_open_pivot_file);
+	    }
+	  else
+	    {
+	      /* keep the actual schema path into the document context */
+		 ustrncpy (pDoc->DocSchemasPath, SchemaPath, MAX_PATH);
+		 if (!pDoc->DocReadOnly)
+		   pDoc->DocReadOnly = (accessMode == 0);
 	       }
-	     else
-	       {
-		  /* keep the actual schema path into the document context */
-		  ustrncpy (pDoc->DocSchemasPath, SchemaPath, MAX_PATH);
-		  document = IdentDocument (pDoc);
-		  if (!pDoc->DocReadOnly)
-		     pDoc->DocReadOnly = (accessMode == 0);
-	       }
-	  }
-     }
-   return document;
+	}
+    }
+  return document;
 }
 
 /*----------------------------------------------------------------------
@@ -259,6 +288,8 @@ STRING              TSchemaName;
     TtaError (ERR_invalid_document_parameter);
   else if (LoadedDocument[document - 1] == NULL)
     TtaError (ERR_invalid_document_parameter);
+  else if (fileName == NULL)
+    TtaError (ERR_invalid_parameter);
   else
     /* parameter document is correct */
     ok = ExportDocument (LoadedDocument[document - 1], fileName, TSchemaName,
@@ -589,6 +620,7 @@ STRING              path;
    the function simply returns the structure schema of that nature.
 
    Parameters:
+   document: the document of interest
    schema: the structure schema to which the nature is added.
    natureName: name of the nature to be added in the structure schema.
    presentationName: name of the presentation schema to be associated with
@@ -603,10 +635,11 @@ STRING              path;
   ----------------------------------------------------------------------*/
 
 #ifdef __STDC__
-SSchema             TtaNewNature (SSchema schema, CHAR_T* natureName, CHAR_T* presentationName)
+SSchema             TtaNewNature (Document document, SSchema schema, CHAR_T* natureName, CHAR_T* presentationName)
 
 #else  /* __STDC__ */
-SSchema             TtaNewNature (schema, natureName, presentationName)
+SSchema             TtaNewNature (document, schema, natureName, presentationName)
+Document            document; 
 SSchema             schema;
 CHAR_T*             natureName;
 CHAR_T*             presentationName;
@@ -619,19 +652,34 @@ CHAR_T*             presentationName;
 
    UserErrorCode = 0;
    natureSchema = NULL;
-   if (schema == NULL || natureName[0] == EOS)
+   if (document < 0 || document > MAX_DOCUMENTS)
+     {
+	TtaError (ERR_invalid_document_parameter);
+     }
+   else if (document > 0 && LoadedDocument[document - 1] == NULL)
+     {
+	TtaError (ERR_invalid_document_parameter);
+     }
+   else if (schema == NULL || natureName[0] == EOS)
      {
 	TtaError (ERR_invalid_parameter);
      }
    else
      {
-	natureRule = CreateNature (natureName, presentationName, (PtrSSchema) schema);
+	natureRule = CreateNature (natureName, presentationName,
+				   (PtrSSchema) schema);
 	if (natureRule == 0)
-	  {
+	   {
 	     TtaError (ERR_invalid_parameter);
-	  }
+	   }
 	else
+	   {
 	   natureSchema = ((PtrSSchema) schema)->SsRule[natureRule - 1].SrSSchemaNat;
+#ifndef NODISPLAY
+	   if (document > 0)
+	      AddSchemaGuestViews (LoadedDocument[document - 1], natureSchema);
+#endif
+	   }
      }
    return ((SSchema) natureSchema);
 }
@@ -727,47 +775,51 @@ CHAR_T*             presentationName;
    if (document < 1 || document > MAX_DOCUMENTS)
       TtaError (ERR_invalid_document_parameter);
    else if (LoadedDocument[document - 1] == NULL)
-        TtaError (ERR_invalid_document_parameter);
+      TtaError (ERR_invalid_document_parameter);
    else
-       /* parameter document is correct */
-       {
-           pDoc = LoadedDocument[document - 1];
+      /* parameter document is correct */
+      {
+      pDoc = LoadedDocument[document - 1];
 #ifdef NODISPLAY
-           if (pDoc->DocSSchema != NULL)
-               ustrncpy (pDoc->DocSSchema->SsDefaultPSchema, presentationName, MAX_NAME_LENGTH - 1);
+      if (pDoc->DocSSchema != NULL)
+	 ustrncpy (pDoc->DocSSchema->SsDefaultPSchema, presentationName,
+		   MAX_NAME_LENGTH - 1);
 #else
-           /* verifies that there is no opened views */
-           ok = TRUE;
-           for (view = 1; view <= MAX_VIEW_DOC && ok; view++)
-               if (pDoc->DocView[view - 1].DvPSchemaView != 0)
-                  ok = FALSE;
-           if (ok)
-              for (Assoc = 1; Assoc <= MAX_ASSOC_DOC && ok; Assoc++)
-                  if (pDoc->DocAssocFrame[Assoc - 1] != 0)
-                     ok = FALSE;
-           if (!ok)
-              TtaError (ERR_there_are_open_views);
-           else
-               /* There is no opened views */
-               {
-                   Name pschemaName;
+      /* verifies that there is no open views */
+      ok = TRUE;
+      for (view = 1; view <= MAX_VIEW_DOC && ok; view++)
+	 if (pDoc->DocView[view - 1].DvPSchemaView != 0)
+	    ok = FALSE;
+      if (ok)
+	 for (Assoc = 1; Assoc <= MAX_ASSOC_DOC && ok; Assoc++)
+	    if (pDoc->DocAssocFrame[Assoc - 1] != 0)
+	       ok = FALSE;
+      if (!ok)
+	 TtaError (ERR_there_are_open_views);
+      else
+	 /* There is no opened views */
+	 {
+	 Name pschemaName;
 
-                   ustrncpy (pschemaName, presentationName, MAX_NAME_LENGTH);
-                   if (pDoc->DocSSchema->SsPSchema != NULL)
-                      /* a presentation schema already exist. One release it */
-                      {
-                         FreePresentationSchema (pDoc->DocSSchema->SsPSchema, pDoc->DocSSchema);
-                         pDoc->DocSSchema->SsPSchema = NULL;
-                      }
-                   /* Load the presentation schema */
-                   if (pDoc->DocSSchema->SsExtension)
-                      /* to avoid that ReadPresentationSchema reloades the structure schema */
-                      pDoc->DocSSchema->SsRootElem = 1;
-                   pDoc->DocSSchema->SsPSchema = LoadPresentationSchema (pschemaName, pDoc->DocSSchema);
-                   if (pDoc->DocSSchema->SsPSchema == NULL)
-                      /* Failure while loading schema */
-                      TtaError (ERR_cannot_load_pschema);
-               }
+	 ustrncpy (pschemaName, presentationName, MAX_NAME_LENGTH);
+	 if (pDoc->DocSSchema->SsPSchema != NULL)
+	    /* a presentation schema already exist. One release it */
+	    {
+	    FreePresentationSchema (pDoc->DocSSchema->SsPSchema,
+				    pDoc->DocSSchema);
+	    pDoc->DocSSchema->SsPSchema = NULL;
+	    }
+	 /* Load the presentation schema */
+	 if (pDoc->DocSSchema->SsExtension)
+	    /* to avoid that ReadPresentationSchema reloades the structure
+	       schema */
+	    pDoc->DocSSchema->SsRootElem = 1;
+	 pDoc->DocSSchema->SsPSchema = LoadPresentationSchema (pschemaName,
+							     pDoc->DocSSchema);
+	 if (pDoc->DocSSchema->SsPSchema == NULL)
+	    /* Failure while loading schema */
+	    TtaError (ERR_cannot_load_pschema);
+	 }
 #endif
       }
 }
@@ -1301,18 +1353,13 @@ SSchema             schema;
 
    Return value:
    name of the associated presentation schema.
-
   ----------------------------------------------------------------------*/
-
 #ifdef __STDC__
 CHAR_T*              TtaGetPSchemaName (SSchema schema)
-
 #else  /* __STDC__ */
 CHAR_T*              TtaGetPSchemaName (schema)
 SSchema             schema;
-
 #endif /* __STDC__ */
-
 {
    UserErrorCode = 0;
    if (schema == NULL)
@@ -1327,19 +1374,18 @@ SSchema             schema;
    return ISObuffer;
 }
 
-/* ChSchStruct recursively searches the schema which name is "name" within
-   nature schema and extension schema used by pSS. It returns a pointer
-   which references this schema or NULL if not found. */
+/*----------------------------------------------------------------------
+  ChSchStruct recursively searches the schema which name is "name" within
+  nature schema and extension schema used by pSS. It returns a pointer
+  which references this schema or NULL if not found.
+  ----------------------------------------------------------------------*/
 #ifdef __STDC__
 static SSchema      ChSchStruct (PtrSSchema pSS, CHAR_T* name)
-
 #else  /* __STDC__ */
 static SSchema      ChSchStruct (pSS, name)
 PtrSSchema          pSS;
 CHAR_T*             name;
-
 #endif /* __STDC__ */
-
 {
    int                 nRegle;
    SSchema             ret;
