@@ -82,8 +82,8 @@ BinFile             pivFile;
 
    TtaWriteByte (pivFile, (char) C_PIV_VERSION);
    TtaWriteByte (pivFile, (char) C_PIV_VERSION);
-   /* Version courante de PIVOT: 5 */
-   version = 5;
+   /* Version courante de PIVOT: 6 */
+   version = 6;
    TtaWriteByte (pivFile, (char) version);
 }
 
@@ -152,6 +152,8 @@ TypeUnit            unit;
 {
    if (unit == UnPoint)
       TtaWriteByte (pivFile, C_PIV_PT);
+   else if (unit == UnPixel)
+      TtaWriteByte (pivFile, C_PIV_PX);
    else
       TtaWriteByte (pivFile, C_PIV_EM);
 }
@@ -567,46 +569,60 @@ BinFile       pivFile;
 PtrPRule      pPRule;
 #endif /* __STDC__ */
 {
+  PRuleType          rType;
   unsigned short     red, green, blue;
 
-  /* s'il s'agit d'une regle de dimension elastique, on ne l'ecrit pas */
-  if (pPRule->PrType == PtHeight || pPRule->PrType == PtWidth)
-    if (pPRule->PrDimRule.DrPosition)
-      return;
-
+  rType = pPRule->PrType;
   /* on ne traite que les regles de presentation directes (ni heritage */
   /* ni fonction de presentation) dont le codage pivot est defini */
-  if ((pPRule->PrType == PtHeight || pPRule->PrType == PtWidth ||
-       pPRule->PrType == PtVertPos || pPRule->PrType == PtHorizPos ||
-       pPRule->PrType == PtMarginTop || pPRule->PrType == PtMarginRight ||
-       pPRule->PrType == PtMarginBottom || pPRule->PrType == PtMarginLeft ||
-       pPRule->PrType == PtPaddingTop || pPRule->PrType == PtPaddingRight ||
-       pPRule->PrType == PtPaddingBottom || pPRule->PrType == PtPaddingLeft ||
-       pPRule->PrType == PtBorderTopWidth ||
-       pPRule->PrType == PtBorderRightWidth ||
-       pPRule->PrType == PtBorderBottomWidth ||
-       pPRule->PrType == PtBorderLeftWidth ||
-       pPRule->PrType == PtBorderTopColor ||
-       pPRule->PrType == PtBorderRightColor ||
-       pPRule->PrType == PtBorderBottomColor ||
-       pPRule->PrType == PtBorderLeftColor ||
-       pPRule->PrType == PtBorderTopStyle ||
-       pPRule->PrType == PtBorderRightStyle ||
-       pPRule->PrType == PtBorderBottomStyle ||
-       pPRule->PrType == PtBorderLeftStyle ||
-       pPRule->PrType == PtSize || pPRule->PrType == PtStyle ||
-       pPRule->PrType == PtWeight || pPRule->PrType == PtFont ||
-       pPRule->PrType == PtUnderline || pPRule->PrType == PtThickness ||
-       pPRule->PrType == PtIndent || pPRule->PrType == PtLineSpacing ||
-       pPRule->PrType == PtDepth ||
-       pPRule->PrType == PtAdjust || pPRule->PrType == PtJustify ||
-       pPRule->PrType == PtLineStyle || pPRule->PrType == PtLineWeight ||
-       pPRule->PrType == PtFillPattern ||
-       pPRule->PrType == PtBackground || pPRule->PrType == PtForeground ||
-       pPRule->PrType == PtHyphenate ||
-       pPRule->PrType == PtBreak1 || pPRule->PrType == PtBreak2 ||
-       pPRule->PrType == PtPictInfo)
-      && pPRule->PrPresMode == PresImmediate)
+  /* On traite toutefois les regles qui expriment une distance en terme
+     de pourcentage du parent */
+  if ((pPRule->PrPresMode == PresImmediate &&
+       (rType == PtHeight || rType == PtWidth ||
+	rType == PtVertPos || rType == PtHorizPos ||
+	rType == PtMarginTop || rType == PtMarginRight ||
+	rType == PtMarginBottom || rType == PtMarginLeft ||
+	rType == PtPaddingTop || rType == PtPaddingRight ||
+	rType == PtPaddingBottom || rType == PtPaddingLeft ||
+	rType == PtBorderTopWidth ||
+	rType == PtBorderRightWidth ||
+	rType == PtBorderBottomWidth ||
+	rType == PtBorderLeftWidth ||
+	rType == PtXRadius || rType == PtYRadius ||
+	rType == PtBorderTopColor ||
+	rType == PtBorderRightColor ||
+	rType == PtBorderBottomColor ||
+	rType == PtBorderLeftColor ||
+	rType == PtBorderTopStyle ||
+	rType == PtBorderRightStyle ||
+	rType == PtBorderBottomStyle ||
+	rType == PtBorderLeftStyle ||
+	rType == PtSize || rType == PtStyle ||
+	rType == PtWeight || rType == PtFont ||
+	rType == PtUnderline || rType == PtThickness ||
+	rType == PtIndent || rType == PtLineSpacing ||
+	rType == PtDepth ||
+	rType == PtAdjust || rType == PtJustify ||
+	rType == PtLineStyle || rType == PtLineWeight ||
+	rType == PtFillPattern ||
+	rType == PtBackground || rType == PtForeground ||
+	rType == PtHyphenate ||
+	rType == PtBreak1 || rType == PtBreak2 ||
+	rType == PtPictInfo)) ||
+      (pPRule->PrPresMode == PresInherit &&
+       pPRule->PrInheritMode == InheritParent &&
+       pPRule->PrInhPercent && !pPRule->PrInhAttr &&
+       pPRule->PrInhMinOrMax == 0 &&
+       (rType == PtBreak1 || rType == PtBreak2 ||
+	rType == PtIndent || rType == PtSize ||
+	rType == PtLineSpacing || rType == PtLineWeight ||
+	rType == PtMarginTop || rType == PtMarginRight ||
+	rType == PtMarginBottom || rType == PtMarginLeft ||
+	rType == PtPaddingTop || rType == PtPaddingRight ||
+	rType == PtPaddingBottom || rType == PtPaddingLeft ||
+	rType == PtBorderTopWidth || rType == PtBorderRightWidth ||
+	rType == PtBorderBottomWidth || rType == PtBorderLeftWidth ||
+	rType == PtXRadius || rType == PtYRadius)))
     {
       /* ecrit la marque de regle */
       TtaWriteByte (pivFile, (CHAR_T) C_PIV_PRESENT);
@@ -614,230 +630,265 @@ PtrPRule      pPRule;
       PutShort (pivFile, pPRule->PrViewNum);
       /* ecrit le numero de la boite de presentation concernee */
       PutShort (pivFile, 0);
-      /* ecrit le type de la regle et ses parametres */
-      switch (pPRule->PrType)
+      /* ecrit le type de la regle */
+      switch (rType)
 	{
 	case PtAdjust:
 	  /* mode de mise en ligne */
 	  TtaWriteByte (pivFile, C_PR_ADJUST);
-	  PutAlignment (pivFile, pPRule->PrAdjust);
 	  break;
 	case PtHeight:
-	  if (!pPRule->PrDimRule.DrPosition)
-	    {
-	      TtaWriteByte (pivFile, C_PR_HEIGHT);
-	      PutDimension (pivFile, pPRule);
-	    }
-	  break;
 	case PtWidth:
-	  if (!pPRule->PrDimRule.DrPosition)
+	  if (pPRule->PrDimRule.DrPosition)
 	    {
-	      TtaWriteByte (pivFile, C_PR_WIDTH);
-	      PutDimension (pivFile, pPRule);
+	      if (pPRule->PrType == PtHeight)
+		TtaWriteByte (pivFile, C_PR_HEIGHTPOS);
+	      else
+		TtaWriteByte (pivFile, C_PR_WIDTHPOS);
+	    }
+	  else
+	    {
+	      if (pPRule->PrType == PtHeight)
+		TtaWriteByte (pivFile, C_PR_HEIGHT);
+	      else
+		TtaWriteByte (pivFile, C_PR_WIDTH);
 	    }
 	  break;
 	case PtVertPos:
+	  TtaWriteByte (pivFile, C_PR_VPOS);
+	  break;
 	case PtHorizPos:
-	  if (pPRule->PrType == PtVertPos)
-	    TtaWriteByte (pivFile, C_PR_VPOS);
+	  TtaWriteByte (pivFile, C_PR_HPOS);
+	  break;
+	case PtFont:
+	  TtaWriteByte (pivFile, C_PR_FONT);
+	  break;
+	case PtStyle:
+	  TtaWriteByte (pivFile, C_PR_STYLE);
+	  break;
+	case PtWeight:
+	  TtaWriteByte (pivFile, C_PR_WEIGHT);
+	  break;
+	case PtUnderline:
+	  TtaWriteByte (pivFile, C_PR_UNDERLINE);
+	  break;
+	case PtThickness:
+	  TtaWriteByte (pivFile, C_PR_UNDER_THICK);
+	  break;
+	case PtLineStyle:
+	  TtaWriteByte (pivFile, C_PR_LINESTYLE);
+	  break;
+        case PtBorderTopStyle:
+	  TtaWriteByte (pivFile, C_PR_BORDERTOPSTYLE);
+	  break;
+        case PtBorderRightStyle:
+	  TtaWriteByte (pivFile, C_PR_BORDERRIGHTSTYLE);
+	  break;
+        case PtBorderBottomStyle:
+	  TtaWriteByte (pivFile, C_PR_BORDERBOTTOMSTYLE);
+	  break;
+        case PtBorderLeftStyle:
+	  TtaWriteByte (pivFile, C_PR_BORDERLEFTSTYLE);
+	  break;
+	case PtBreak1:
+	  TtaWriteByte (pivFile, C_PR_BREAK1);
+	  break;
+	case PtBreak2:
+	  TtaWriteByte (pivFile, C_PR_BREAK2);
+	  break;
+	case PtIndent:
+	  TtaWriteByte (pivFile, C_PR_INDENT);
+	  break;
+	case PtSize:
+	  TtaWriteByte (pivFile, C_PR_SIZE);
+	  break;
+	case PtLineSpacing:
+	  TtaWriteByte (pivFile, C_PR_LINESPACING);
+	  break;
+	case PtLineWeight:
+	  TtaWriteByte (pivFile, C_PR_LINEWEIGHT);
+	  break;
+        case PtMarginTop:
+	  TtaWriteByte (pivFile, C_PR_MARGINTOP);
+	  break;
+        case PtMarginRight:
+	  TtaWriteByte (pivFile, C_PR_MARGINRIGHT);
+	  break;
+        case PtMarginBottom:
+	  TtaWriteByte (pivFile, C_PR_MARGINBOTTOM);
+	  break;
+        case PtMarginLeft:
+	  TtaWriteByte (pivFile, C_PR_MARGINLEFT);
+	  break;
+        case PtPaddingTop:
+	  TtaWriteByte (pivFile, C_PR_PADDINGTOP);
+	  break;
+        case PtPaddingRight:
+	  TtaWriteByte (pivFile, C_PR_PADDINGRIGHT);
+	  break;
+        case PtPaddingBottom:
+	  TtaWriteByte (pivFile, C_PR_PADDINGBOTTOM);
+	  break;
+        case PtPaddingLeft:
+	  TtaWriteByte (pivFile, C_PR_PADDINGLEFT);
+	  break;
+        case PtBorderTopWidth:
+	  TtaWriteByte (pivFile, C_PR_BORDERTOPWIDTH);
+	  break;
+        case PtBorderRightWidth:
+	  TtaWriteByte (pivFile, C_PR_BORDERRIGHTWIDTH);
+	  break;
+        case PtBorderBottomWidth:
+	  TtaWriteByte (pivFile, C_PR_BORDERBOTTOMWIDTH);
+	  break;
+        case PtBorderLeftWidth:
+	  TtaWriteByte (pivFile, C_PR_BORDERLEFTWIDTH);
+	  break;
+        case PtXRadius:
+	  TtaWriteByte (pivFile, C_PR_XRADIUS);
+	  break;
+        case PtYRadius:
+	  TtaWriteByte (pivFile, C_PR_YRADIUS);
+	  break;
+	case PtJustify:
+	  TtaWriteByte (pivFile, C_PR_JUSTIFY);
+	  break;
+	case PtHyphenate:
+	  TtaWriteByte (pivFile, C_PR_HYPHENATE);
+	  break;
+	case PtDepth:
+	  TtaWriteByte (pivFile, C_PR_DEPTH);
+	  break;
+	case PtFillPattern:
+	  TtaWriteByte (pivFile, C_PR_FILLPATTERN);
+	  break;
+	case PtBackground:
+	  TtaWriteByte (pivFile, C_PR_BACKGROUND);
+	  break;
+	case PtForeground:
+	  TtaWriteByte (pivFile, C_PR_FOREGROUND);
+	  break;
+        case PtBorderTopColor:
+	  TtaWriteByte (pivFile, C_PR_BORDERTOPCOLOR);
+	  break;
+        case PtBorderRightColor:
+	  TtaWriteByte (pivFile, C_PR_BORDERRIGHTCOLOR);
+	  break;
+        case PtBorderBottomColor:
+	  TtaWriteByte (pivFile, C_PR_BORDERBOTTOMCOLOR);
+	  break;
+        case PtBorderLeftColor:
+	  TtaWriteByte (pivFile, C_PR_BORDERLEFTCOLOR);
+	  break;
+	default:
+	  break;
+	}
+      /* ecrit les parametres de la regle */
+      switch (rType)
+	{
+	case PtAdjust:
+	  /* mode de mise en ligne */
+	  PutAlignment (pivFile, pPRule->PrAdjust);
+	  break;
+	case PtHeight:
+	case PtWidth:
+	  if (pPRule->PrDimRule.DrPosition)
+	    {
+	      PutShort (pivFile, pPRule->PrDimRule.DrPosRule.PoPosDef);
+	      PutShort (pivFile, pPRule->PrDimRule.DrPosRule.PoPosRef);
+	      PutShort (pivFile, pPRule->PrDimRule.DrPosRule.PoRelation);
+	      PutShort (pivFile, abs (pPRule->PrDimRule.DrPosRule.PoDistance));
+	      PutUnit (pivFile, pPRule->PrDimRule.DrPosRule.PoDistUnit);
+	      PutSign (pivFile, (ThotBool)(pPRule->PrDimRule.DrPosRule.PoDistance >= 0));
+	    }
 	  else
-	    TtaWriteByte (pivFile, C_PR_HPOS);
+	    PutDimension (pivFile, pPRule);
+	  break;
+	case PtVertPos:
+	case PtHorizPos:
+	  PutShort (pivFile, pPRule->PrPosRule.PoPosDef);
+	  PutShort (pivFile, pPRule->PrPosRule.PoPosRef);
+	  PutShort (pivFile, pPRule->PrPosRule.PoRelation);
 	  PutShort (pivFile, abs (pPRule->PrPosRule.PoDistance));
 	  PutUnit (pivFile, pPRule->PrPosRule.PoDistUnit);
 	  PutSign (pivFile, (ThotBool)(pPRule->PrPosRule.PoDistance >= 0));
 	  break;
-	case PtFont:
-	  TtaWriteByte (pivFile, C_PR_FONT);
-	  TtaWriteByte (pivFile, pPRule->PrChrValue);
-	  break;
-	case PtStyle:
-	  TtaWriteByte (pivFile, C_PR_STYLE);
-	  TtaWriteByte (pivFile, pPRule->PrChrValue);
-	  break;
-	case PtWeight:
-	  TtaWriteByte (pivFile, C_PR_WEIGHT);
-	  TtaWriteByte (pivFile, pPRule->PrChrValue);
-	  break;
-	case PtUnderline:
-	  TtaWriteByte (pivFile, C_PR_UNDERLINE);
-	  TtaWriteByte (pivFile, pPRule->PrChrValue);
-	  break;
-	case PtThickness:
-	  TtaWriteByte (pivFile, C_PR_UNDER_THICK);
-	  TtaWriteByte (pivFile, pPRule->PrChrValue);
-	  break;
-	case PtLineStyle:
-	  TtaWriteByte (pivFile, C_PR_LINESTYLE);
-	  TtaWriteByte (pivFile, pPRule->PrChrValue);
-	  break;
-        case PtBorderTopStyle:
-	  TtaWriteByte (pivFile, C_PR_BORDERTOPSTYLE);
-	  TtaWriteByte (pivFile, pPRule->PrChrValue);
-	  break;
-        case PtBorderRightStyle:
-	  TtaWriteByte (pivFile, C_PR_BORDERRIGHTSTYLE);
-	  TtaWriteByte (pivFile, pPRule->PrChrValue);
-	  break;
-        case PtBorderBottomStyle:
-	  TtaWriteByte (pivFile, C_PR_BORDERBOTTOMSTYLE);
-	  TtaWriteByte (pivFile, pPRule->PrChrValue);
-	  break;
-        case PtBorderLeftStyle:
-	  TtaWriteByte (pivFile, C_PR_BORDERLEFTSTYLE);
-	  TtaWriteByte (pivFile, pPRule->PrChrValue);
-	  break;
 	case PtBreak1:
-	  TtaWriteByte (pivFile, C_PR_BREAK1);
-	  PutShort (pivFile, pPRule->PrMinValue);
-	  PutUnit (pivFile, pPRule->PrMinUnit);
-	  break;
 	case PtBreak2:
-	  TtaWriteByte (pivFile, C_PR_BREAK2);
-	  PutShort (pivFile, pPRule->PrMinValue);
-	  PutUnit (pivFile, pPRule->PrMinUnit);
+	case PtSize:
+	case PtLineSpacing:
+	case PtLineWeight:
+        case PtPaddingTop:
+        case PtPaddingRight:
+        case PtPaddingBottom:
+        case PtPaddingLeft:
+        case PtBorderTopWidth:
+        case PtBorderRightWidth:
+        case PtBorderBottomWidth:
+        case PtBorderLeftWidth:
+        case PtXRadius:
+        case PtYRadius:
+	  PutBoolean (pivFile, (pPRule->PrPresMode == PresImmediate));
+	  if (pPRule->PrPresMode == PresImmediate)
+	    {
+	      PutShort (pivFile, pPRule->PrMinValue);
+	      PutUnit (pivFile, pPRule->PrMinUnit);
+	    }
+	  else
+	    {
+	      PutShort (pivFile, pPRule->PrInhDelta);
+	      PutUnit (pivFile, pPRule->PrInhUnit);
+	    }
 	  break;
 	case PtIndent:
-	  TtaWriteByte (pivFile, C_PR_INDENT);
-	  PutShort (pivFile, abs (pPRule->PrMinValue));
-	  PutUnit (pivFile, pPRule->PrMinUnit);
-	  PutSign (pivFile, (ThotBool)(pPRule->PrMinValue >= 0));
-	  break;
-	case PtSize:
-	  TtaWriteByte (pivFile, C_PR_SIZE);
-	  PutShort (pivFile, pPRule->PrMinValue);
-	  PutUnit (pivFile, pPRule->PrMinUnit);
-	  break;
-	case PtLineSpacing:
-	  TtaWriteByte (pivFile, C_PR_LINESPACING);
-	  PutShort (pivFile, pPRule->PrMinValue);
-	  PutUnit (pivFile, pPRule->PrMinUnit);
-	  break;
-	case PtLineWeight:
-	  TtaWriteByte (pivFile, C_PR_LINEWEIGHT);
-	  PutShort (pivFile, pPRule->PrMinValue);
-	  PutUnit (pivFile, pPRule->PrMinUnit);
-	  break;
         case PtMarginTop:
-	  TtaWriteByte (pivFile, C_PR_MARGINTOP);
-	  PutShort (pivFile, pPRule->PrMinValue);
-	  PutUnit (pivFile, pPRule->PrMinUnit);
-	  PutSign (pivFile, (ThotBool)(pPRule->PrMinValue >= 0));
-	  break;
         case PtMarginRight:
-	  TtaWriteByte (pivFile, C_PR_MARGINRIGHT);
-	  PutShort (pivFile, pPRule->PrMinValue);
-	  PutUnit (pivFile, pPRule->PrMinUnit);
-	  PutSign (pivFile, (ThotBool)(pPRule->PrMinValue >= 0));
-	  break;
         case PtMarginBottom:
-	  TtaWriteByte (pivFile, C_PR_MARGINBOTTOM);
-	  PutShort (pivFile, pPRule->PrMinValue);
-	  PutUnit (pivFile, pPRule->PrMinUnit);
-	  PutSign (pivFile, (ThotBool)(pPRule->PrMinValue >= 0));
-	  break;
         case PtMarginLeft:
-	  TtaWriteByte (pivFile, C_PR_MARGINLEFT);
-	  PutShort (pivFile, pPRule->PrMinValue);
-	  PutUnit (pivFile, pPRule->PrMinUnit);
-	  PutSign (pivFile, (ThotBool)(pPRule->PrMinValue >= 0));
-	  break;
-        case PtPaddingTop:
-	  TtaWriteByte (pivFile, C_PR_PADDINGTOP);
-	  PutShort (pivFile, pPRule->PrMinValue);
-	  PutUnit (pivFile, pPRule->PrMinUnit);
-	  break;
-        case PtPaddingRight:
-	  TtaWriteByte (pivFile, C_PR_PADDINGRIGHT);
-	  PutShort (pivFile, pPRule->PrMinValue);
-	  PutUnit (pivFile, pPRule->PrMinUnit);
-	  break;
-        case PtPaddingBottom:
-	  TtaWriteByte (pivFile, C_PR_PADDINGBOTTOM);
-	  PutShort (pivFile, pPRule->PrMinValue);
-	  PutUnit (pivFile, pPRule->PrMinUnit);
-	  break;
-        case PtPaddingLeft:
-	  TtaWriteByte (pivFile, C_PR_PADDINGLEFT);
-	  PutShort (pivFile, pPRule->PrMinValue);
-	  PutUnit (pivFile, pPRule->PrMinUnit);
-	  break;
-        case PtBorderTopWidth:
-	  TtaWriteByte (pivFile, C_PR_BORDERTOPWIDTH);
-	  PutShort (pivFile, pPRule->PrMinValue);
-	  PutUnit (pivFile, pPRule->PrMinUnit);
-	  break;
-        case PtBorderRightWidth:
-	  TtaWriteByte (pivFile, C_PR_BORDERRIGHTWIDTH);
-	  PutShort (pivFile, pPRule->PrMinValue);
-	  PutUnit (pivFile, pPRule->PrMinUnit);
-	  break;
-        case PtBorderBottomWidth:
-	  TtaWriteByte (pivFile, C_PR_BORDERBOTTOMWIDTH);
-	  PutShort (pivFile, pPRule->PrMinValue);
-	  PutUnit (pivFile, pPRule->PrMinUnit);
-	  break;
-        case PtBorderLeftWidth:
-	  TtaWriteByte (pivFile, C_PR_BORDERLEFTWIDTH);
-	  PutShort (pivFile, pPRule->PrMinValue);
-	  PutUnit (pivFile, pPRule->PrMinUnit);
-	  break;
-	case PtJustify:
-	  TtaWriteByte (pivFile, C_PR_JUSTIFY);
-	  PutBoolean (pivFile, pPRule->PrJustify);
-	  break;
-	case PtHyphenate:
-	  TtaWriteByte (pivFile, C_PR_HYPHENATE);
-	  PutBoolean (pivFile, pPRule->PrJustify);
+	  PutBoolean (pivFile, (pPRule->PrPresMode == PresImmediate));
+	  if (pPRule->PrPresMode == PresImmediate)
+	    {
+	      PutShort (pivFile, abs (pPRule->PrMinValue));
+	      PutUnit (pivFile, pPRule->PrMinUnit);
+	      PutSign (pivFile, (ThotBool)(pPRule->PrMinValue >= 0));
+	    }
+	  else
+	    {
+	      PutShort (pivFile, pPRule->PrInhDelta);
+	      PutUnit (pivFile, pPRule->PrInhUnit);
+	      PutSign (pivFile, (ThotBool)(pPRule->PrInhDelta >= 0));
+	    }
 	  break;
 	case PtDepth:
-	  TtaWriteByte (pivFile, C_PR_DEPTH);
-	  PutShort (pivFile, pPRule->PrIntValue);
-	  break;
 	case PtFillPattern:
-	  TtaWriteByte (pivFile, C_PR_FILLPATTERN);
 	  PutShort (pivFile, pPRule->PrIntValue);
 	  break;
 	case PtBackground:
-	  TtaWriteByte (pivFile, C_PR_BACKGROUND);
-	  TtaGiveThotRGB (pPRule->PrIntValue, &red, &green, &blue);
-	  PutShort (pivFile, red);
-	  PutShort (pivFile, green);
-	  PutShort (pivFile, blue);
-	  break;
 	case PtForeground:
-	  TtaWriteByte (pivFile, C_PR_FOREGROUND);
-	  TtaGiveThotRGB (pPRule->PrIntValue, &red, &green, &blue);
-	  PutShort (pivFile, red);
-	  PutShort (pivFile, green);
-	  PutShort (pivFile, blue);
-	  break;
         case PtBorderTopColor:
-	  TtaWriteByte (pivFile, C_PR_BORDERTOPCOLOR);
-	  TtaGiveThotRGB (pPRule->PrIntValue, &red, &green, &blue);
-	  PutShort (pivFile, red);
-	  PutShort (pivFile, green);
-	  PutShort (pivFile, blue);
-	  break;
         case PtBorderRightColor:
-	  TtaWriteByte (pivFile, C_PR_BORDERRIGHTCOLOR);
-	  TtaGiveThotRGB (pPRule->PrIntValue, &red, &green, &blue);
-	  PutShort (pivFile, red);
-	  PutShort (pivFile, green);
-	  PutShort (pivFile, blue);
-	  break;
         case PtBorderBottomColor:
-	  TtaWriteByte (pivFile, C_PR_BORDERBOTTOMCOLOR);
+        case PtBorderLeftColor:
 	  TtaGiveThotRGB (pPRule->PrIntValue, &red, &green, &blue);
 	  PutShort (pivFile, red);
 	  PutShort (pivFile, green);
 	  PutShort (pivFile, blue);
 	  break;
-        case PtBorderLeftColor:
-	  TtaWriteByte (pivFile, C_PR_BORDERLEFTCOLOR);
-	  TtaGiveThotRGB (pPRule->PrIntValue, &red, &green, &blue);
-	  PutShort (pivFile, red);
-	  PutShort (pivFile, green);
-	  PutShort (pivFile, blue);
+	case PtFont:
+	case PtStyle:
+	case PtWeight:
+	case PtUnderline:
+	case PtThickness:
+	case PtLineStyle:
+        case PtBorderTopStyle:
+        case PtBorderRightStyle:
+        case PtBorderBottomStyle:
+        case PtBorderLeftStyle:
+	  TtaWriteByte (pivFile, pPRule->PrChrValue);
+	  break;
+	case PtJustify:
+	case PtHyphenate:
+	  PutBoolean (pivFile, pPRule->PrJustify);
 	  break;
 	default:
 	  break;
@@ -969,6 +1020,9 @@ ThotBool            subTree;
 	case YRepeat:
 	  TtaWriteByte (pivFile, C_PIV_YREPEAT);
 	  break;
+	default:
+	  TtaWriteByte (pivFile, ' ');
+	  break;
 	}
       PutShort (pivFile, ((PictInfo *)(pEl1->ElPictInfo))->PicType);
     }
