@@ -35,6 +35,28 @@ typedef int        *SSchema;
 #ifdef __STDC__
 
 
+/*----------------------------------------------------------------------
+   TtaInitDocument
+
+   Creates the internal representation of a new document according to a given
+   structure schema. No file is created immediately, but the backup files
+   (.BAK and .SAV) and the document file (.PIV see TtaNewDocument) will be created
+   in the first directory of the document path (see TtaSetDocumentPath).
+
+   Parameters:
+   structureSchema: name of the structure schema that defines the type of
+   document to be created.
+   documentName: name of the document to be created (maximum length 19
+   characters). The directory name is not part of this parameter
+   (see TtaSetDocumentPath).
+   document: the requested document or 0.
+
+   Return value:
+   the document that has been created or 0 if the document has not
+   been created.
+  ----------------------------------------------------------------------*/
+extern Document     TtaInitDocument (CHAR_T* structureSchema, CHAR_T* documentName, Document document);
+
 /*----------------------------------------------------------------------
    TtaNewDocument
 
@@ -234,6 +256,7 @@ extern void         TtaSetSchemaPath (STRING path);
    the function simply returns the structure schema of that nature.
 
    Parameters:
+   document: the document of interest
    schema: the structure schema to which the nature is added.
    natureName: name of the nature to be added in the structure schema.
    presentationName: name of the presentation schema to be associated with
@@ -246,7 +269,7 @@ extern void         TtaSetSchemaPath (STRING path);
    has not been loaded.
 
   ----------------------------------------------------------------------*/
-extern SSchema      TtaNewNature (SSchema schema, CHAR_T* natureName, CHAR_T* presentationName);
+extern SSchema      TtaNewNature (Document document, SSchema schema, CHAR_T* natureName, CHAR_T* presentationName);
 
 /*----------------------------------------------------------------------
    TtaNewSchemaExtension
@@ -742,8 +765,21 @@ extern void TtaConfigReadConfigFiles ( CHAR_T* aSchemaPath );
   ----------------------------------------------------------------------*/
 extern void TtaConfigSSchemaExternalName (/*OUT*/ CHAR_T* nameUser, CHAR_T* nameSchema, int Typ );
 
+/*----------------------------------------------------------------------
+  TtaGetDocumentCharset gets the document charset
+  Returns UNDEFINED_CHARSET when the document uses the default charset.
+ ----------------------------------------------------------------------*/
+extern CHARSET TtaGetDocumentCharset (Document document);
+
+/*----------------------------------------------------------------------
+  TtaSetDocumentCharset sets the document charset
+  ----------------------------------------------------------------------*/
+extern void  TtaSetDocumentCharset (Document document,
+				    CHARSET charSet);
+
 #else  /* __STDC__ */
 
+extern Document     TtaInitDocument (/*CHAR_T* structureSchema, CHAR_T* documentName, Document document*/);
 extern Document     TtaNewDocument ( /* CHAR_T* structureSchema, CHAR_T* documentName */ );
 extern Document     TtaOpenDocument ( /* char *documentName, int accessMode */ );
 extern void         TtaSaveDocument ( /* Document document, char *documentName */ );
@@ -756,7 +792,7 @@ extern ThotBool     TtaCheckPath (/* char *path */);
 extern ThotBool     TtaIsInDocumentPath ( /*CHAR_T* aDirectory */ );
 extern void         TtaAppendDocumentPath ( /*CHAR_T* aDirectory */ );
 extern void         TtaSetSchemaPath ( /* char *path */ );
-extern SSchema      TtaNewNature ( /* SSchema schema, CHAR_T* natureName, CHAR_T* presentationName */ );
+extern SSchema      TtaNewNature ( /* Document document, SSchema schema, CHAR_T* natureName, CHAR_T* presentationName */ );
 extern SSchema      TtaNewSchemaExtension ( /* Document document, CHAR_T* extensionName, CHAR_T* presentationName */ );
 extern SSchema      TtaGetSchemaExtension ( /*Document document, char *NomExtension */ );
 extern void         TtaRemoveSchemaExtension ( /* Document document, SSchema extension, int *removedElements, int *removedAttributes */ );
@@ -789,11 +825,13 @@ extern void         TtaGetDocumentPath ( /* char *buffer, int bufferLength */ );
 extern void         TtaGetSchemaPath ( /* char *buffer, int bufferLength */ );
 extern Document     TtaGetDocumentOfSavedElements ();
 
-extern void TtaConfigReadConfigFiles (/* CHAR_T* aSchemaPath */);
-extern void TtaConfigSSchemaExternalName (/* CHAR_T *nameUser,
+extern void         TtaConfigReadConfigFiles (/* CHAR_T* aSchemaPath */);
+extern void         TtaConfigSSchemaExternalName (/* CHAR_T *nameUser,
                                              CHAR_T *nameSchema,
                                              int Typ */);
-
+extern CHARSET      TtaGetDocumentCharset (/* Document document */);
+extern void         TtaSetDocumentCharset (/* Document document,
+					      CHARSET charSet */);
 #endif /* __STDC__ */
 #endif /* __CEXTRACT__ */
 
