@@ -83,7 +83,9 @@ typedef int        *PRule;
 #define PRPageBefore 57
 #define PRPageAfter 58
 #define PRPageInside 59
-#define LAST_PRES_RULE_TYPE 59
+#define PRXRadius 60
+#define PRYRadius 61
+#define LAST_PRES_RULE_TYPE 61
 
 /* values for rule PRStyle */
 #define StyleRoman 1
@@ -231,6 +233,19 @@ extern PRule        TtaCopyPRule (PRule pRule);
   ----------------------------------------------------------------------*/
 extern void         TtaAttachPRule (Element element, PRule pRule, Document document);
 
+/*----------------------------------------------------------------------
+   TtaIsCSSPRule
+
+   Check whether a presentation rule is associated with a CSS rule
+
+   Parameters:
+   pRule: the presentation rule to be tested.
+
+   Return value:
+   TRUE if pRule is a CSS rule
+  ----------------------------------------------------------------------*/
+extern ThotBool    TtaIsCSSPRule (PRule pRule);
+
 /*----------------------------------------------------------------------
    TtaRemovePRule
 
@@ -367,6 +382,17 @@ extern void         TtaChangeBoxSize (Element element, Document document, View v
   ----------------------------------------------------------------------*/
 extern void         TtaChangeBoxPosition (Element element, Document document, View view, int X, int Y, TypeUnit unit);
 
+/*----------------------------------------------------------------------
+   TtaGetDepth
+   Returns the depth of the box corresponding to an element in a given view.
+
+   Parameters:
+   element: the element of interest.
+   document: the document of interest.
+   view: the view.
+  ----------------------------------------------------------------------*/
+extern int          TtaGetDepth (Element element, Document document, View view);
+
 /*----------------------------------------------------------------------
    TtaGiveBoxSize
 
@@ -612,12 +638,14 @@ extern PRule        TtaNewPRuleForView (/* int presentationType, int view, Docum
 extern PRule        TtaNewPRuleForNamedView ( /* int presentationType, STRING viewName, Document document */ );
 extern PRule        TtaCopyPRule ( /* PRule pRule */ );
 extern void         TtaAttachPRule ( /* Element element, PRule pRule, Document document */ );
+extern ThotBool     TtaIsCSSPRule ( /* PRule pRule */ );
 extern void         TtaRemovePRule ( /* Element element, PRule pRule, Document document */ );
 extern void         TtaSetPRuleValue ( /* Element element, PRule pRule, int value, Document document */ );
 extern void         TtaSetPRuleValueWithUnit ( /* Element element, PRule pRule, int value, TypeUnit unit, Document document */ );
 extern void         TtaSetPRuleView ( /* PRule pRule, int view */ );
 extern void         TtaChangeBoxSize ( /* Element element, View view, Document document, int deltaX, int deltaY, TypeUnit unit */ );
 extern void         TtaChangeBoxPosition ( /* Element element, Document document, View view, int X, int Y, TypeUnit unit */ );
+extern int          TtaGetDepth (/*Element element, Document document, View view*/);
 extern void         TtaGiveBoxSize ( /* Element element, Document document, View view, TypeUnit unit, int *width, int *height */ );
 extern void         TtaGiveBoxPosition ( /* Element element, Document document, View view, TypeUnit unit, int *xCoord, int *yCoord */ );
 extern void         TtaGiveBoxAbsPosition ( /* Element element, Document document, View view, TypeUnit unit, int *xCoord, int *yCoord */ );
