@@ -373,12 +373,6 @@ extern void         TtaRaiseView (Document document, View view);
   ----------------------------------------------------------------------*/
 extern void         TtaGiveActiveView (Document * document, /*OUT*/ View * view);
 
-/*----------------------------------------------------------------------
-  TtaClipPolyline update the Polyline box to fit the polyline bounding
-  box. Need to be within a draw
-  ----------------------------------------------------------------------*/
-extern void         TtaClipPolyline (Element element, Document doc, View view);
-
 /*----------------------------------------------------------------------
    TtaSetDisplayMode
 
@@ -505,7 +499,6 @@ extern ThotBool     TtaIsViewOpened ( /*Document document, View view */ );
 extern View         TtaGetViewFromName ( /* Document document, char* viewName */ );
 extern void         TtaRaiseView ( /*Document document, View view */ );
 extern void         TtaGiveActiveView ( /* Document *document, View *view */ );
-extern void         TtaClipPolyline ( /* Element element, Document doc, View view */ );
 extern void         TtaSetDisplayMode ( /* Document document, DisplayMode newDisplayMode */ );
 extern DisplayMode  TtaGetDisplayMode ( /* Document document */ );
 extern void         TtaLockTableFormatting ();
