@@ -1,6 +1,6 @@
 /*
  *
- *  (c) COPYRIGHT INRIA 1996.
+ *  (c) COPYRIGHT INRIA 1996-2000
  *  Please first read the full copyright statement in file COPYRIGHT.
  *
  */
@@ -254,10 +254,7 @@ typedef struct _PasteElemDescr
 	PtrElement	PeAscend[MAX_PASTE_LEVEL]; /* former ancestor element*/
 } PasteElemDescr;
 
-/* Document view number */
-typedef int		DocViewNumber;
-
-/* An element of an abstract tree */
+/* an element of an abstract tree */
 typedef struct _ElementDescr
 {
     PtrElement		ElParent;	/* Parent in the abstract tree */
@@ -470,16 +467,35 @@ typedef struct _ReferenceChange
 	PathBuffer		RcFileName;	/* complete name of the .REF
 						   file */
 } ReferenceChange;
-	
+
+/* identifier of a document view */
+typedef int		DocViewNumber;
+
+typedef struct _GuestViewDescr *PtrGuestViewDescr;
+
+/* descriptor of a guest view, i.e. a view (defined in a presentation schema)
+   that can be hosted by a document view (see clause MERGE in language P) */
+typedef struct _GuestViewDescr
+{
+        PtrSSchema        GvSSchema;       /* structure schema of the present.
+					      schema that defines the view */
+        int               GvPSchemaView;   /* number of the view in the present
+					      schema that defines it */
+        PtrGuestViewDescr GvNextGuestView; /* next guest view for the same
+					      document view */
+} GuestViewDescr;
+
 /* descriptor of a document view */
 typedef struct _DocViewDescr
 {
-	PtrSSchema	DvSSchema;	/* structure schema of the presentation
+	PtrSSchema	  DvSSchema;    /* structure schema of the presentation
 					   schema that defines the view */
-	int		DvPSchemaView;	/* Number of the view in the
+	int		  DvPSchemaView;/* number of the view in the
 					   presentation schema */
-	ThotBool        DvSync;		/* this view must be synchronized with
+	ThotBool          DvSync;	/* this view must be synchronized with
 					   the active view */
+        PtrGuestViewDescr DvFirstGuestView; /* list of guest views for that
+					   document view */
 } DocViewDescr;
 
 /* type of an editing operation recorded in the history */
@@ -551,119 +567,118 @@ union
 typedef struct _DocumentDescr *PtrDocument;
 
 /* a document under is internal representation */
+#define MAX_LANGUAGES_DOC 10
+#define MAX_NATURES_DOC 20	/* maximum length of the nature table */
 typedef struct _DocumentDescr
 {
-	PtrDocument     DocNext;	/* to link the free blocks */
-	PtrTextBuffer	DocComment;	/* comment attached to the document */
-	PtrSSchema	DocSSchema;	/* structure schema of the document */
-	PtrElement      DocRootElement; /* pointer on the root element of the
-					   main abstract tree */
-	PtrElement      DocAssocRoot[MAX_ASSOC_DOC];	/* pointers on the
-					   roots of each associated tree */
-	PtrElement	DocAssocSubTree[MAX_ASSOC_DOC]; /* root of the subtree
-					   to be displayed in the associated
-					   elements view, NULL if the whole
-					   tree is displayed */
-	int             DocAssocFrame[MAX_ASSOC_DOC];  /* number of the windows
-					   of the associated trees */
-	int             DocAssocVolume[MAX_ASSOC_DOC]; /* total volume of the
-					   windows of the associated trees */
-	int             DocAssocFreeVolume[MAX_ASSOC_DOC]; /* free space
-					   remaining in the windows of the
-					   the associated trees */
-	int             DocAssocNPages[MAX_ASSOC_DOC];   /* number of pages in
-					   each associated tree */
-	PtrAbstractBox  DocAssocModifiedAb[MAX_ASSOC_DOC]; /* pointers on the
-					   abstract box to be redisplayed for
-					   the associated trees */
-	PtrElement      DocParameters[MAX_PARAM_DOC]; /* pointers on the
-					   parameters of each type */
-	/* In the field DocParameters, the pointers are in the same order as
-	   the rules that define the structure of the pointed elements in the
-	   array StructSchema.SsRule. */
-	PtrReferredDescr DocReferredEl;	/* pointer on the fake descriptor,
+  PtrDocument     DocNext;	/* to link the free blocks */
+  PtrTextBuffer	  DocComment;	/* comment attached to the document */
+  PtrSSchema	  DocSSchema;	/* structure schema of the document */
+  PtrElement      DocRootElement; /* pointer on the root element of the
+				     main abstract tree */
+  PtrElement      DocAssocRoot[MAX_ASSOC_DOC];	/* pointers on the
+						   roots of each associated tree */
+  PtrElement	  DocAssocSubTree[MAX_ASSOC_DOC]; /* root of the subtree
+						     to be displayed in the associated
+						     elements view, NULL if the whole
+						     tree is displayed */
+  int             DocAssocFrame[MAX_ASSOC_DOC];  /* number of the windows
+						    of the associated trees */
+  int             DocAssocVolume[MAX_ASSOC_DOC]; /* total volume of the
+						    windows of the associated trees */
+  int             DocAssocFreeVolume[MAX_ASSOC_DOC]; /* free space
+							remaining in the windows of the
+							the associated trees */
+  int             DocAssocNPages[MAX_ASSOC_DOC];   /* number of pages in
+						      each associated tree */
+  PtrAbstractBox  DocAssocModifiedAb[MAX_ASSOC_DOC]; /* pointers on the
+							abstract box to be redisplayed for
+							the associated trees */
+  PtrElement      DocParameters[MAX_PARAM_DOC]; /* pointers on the
+						   parameters of each type */
+  /* In the field DocParameters, the pointers are in the same order as
+     the rules that define the structure of the pointed elements in the
+     array StructSchema.SsRule. */
+  PtrReferredDescr DocReferredEl;	/* pointer on the fake descriptor,
 					   beginning of the string of
 					   descriptors of elements referenced
 					   in the document */
-	DocViewDescr    DocView[MAX_VIEW_DOC]; /* correspondence between the
-					   document views and those defined in
-					   the presentation schemas used in
-					   the document */
-	PtrAbstractBox  DocViewRootAb[MAX_VIEW_DOC]; /* pointer on the root
-					   abstract box of each view of the
-					   main tree, in the same order as
-					   in DocView */
-	PtrElement	DocViewSubTree[MAX_VIEW_DOC];	/* root of the subtree
-					   of the main tree to display in the
-					   view, null by default */
-	int		DocViewFrame[MAX_VIEW_DOC];	/* number of the window
-					   corresponding to the view */
-	int		DocViewVolume[MAX_VIEW_DOC];   /* volume of the view */
-	int             DocViewFreeVolume[MAX_VIEW_DOC]; /* free volume of the
-							    view */
-	int             DocViewNPages[MAX_VIEW_DOC];  /* number of pages */
-	PtrAbstractBox  DocViewModifiedAb[MAX_VIEW_DOC]; /* pointer on the
-					   abstract box to redisplay for the
-					   view */
-	ADocumentName	DocDName;	/* document name for the user */
-	DocumentIdentifier DocIdent;	/* unique document id for the storage
-					   system */
-	PathBuffer      DocDirectory;	/* directory of the document */
-	PathBuffer	DocSchemasPath;	/* path of the document schemas */
-	ThotBool		DocNotifyAll;	/* Thot must indicate to the application the creation
-					   of ALL the subtree elements that were created or
-					   pasted */
-	ThotBool        DocReadOnly;	/* document is read only */
-	ThotBool        DocExportStructure; /* the logical structure contains
-					   only the skeleton elements */
-	int             DocLabelExpMax;	/* maximum value of element labels */
-	int		DocMaxPairIdent;/* higher document pair id. value */
-	ThotBool        DocModified;	/* the document was modified */
-	ThotBool        DocUpdated;	/* any change occurs since the last reset */
-	int		DocBackUpInterval; /* interval between automatic backup */
-	int             DocNTypedChars;	/* number of characters typed since the
+  DocViewDescr    DocView[MAX_VIEW_DOC]; /* correspondence between the
+					    document views and those defined in
+					    the presentation schemas used in
+					    the document */
+  PtrAbstractBox  DocViewRootAb[MAX_VIEW_DOC]; /* pointer on the root
+						  abstract box of each view of the
+						  main tree, in the same order as
+						  in DocView */
+  PtrElement	  DocViewSubTree[MAX_VIEW_DOC];	/* root of the subtree
+						   of the main tree to display in the
+						   view, null by default */
+  int		  DocViewFrame[MAX_VIEW_DOC];	/* ident. of the window
+						   corresponding to the view */
+  int		  DocViewVolume[MAX_VIEW_DOC];   /* volume of the view */
+  int             DocViewFreeVolume[MAX_VIEW_DOC]; /* free volume of the view */
+  int             DocViewNPages[MAX_VIEW_DOC];  /* number of pages */
+  PtrAbstractBox  DocViewModifiedAb[MAX_VIEW_DOC]; /* pointer on the
+						      abstract box to redisplay for the
+						      view */
+  ADocumentName	  DocDName;	/* document name for the user */
+  DocumentIdentifier DocIdent;	/* unique document id for the storage
+				   system */
+  PathBuffer      DocDirectory;	/* directory of the document */
+  PathBuffer	  DocSchemasPath;	/* path of the document schemas */
+  ThotBool	  DocNotifyAll;	/* Thot must indicate to the application the creation
+				   of ALL the subtree elements that were created or
+				   pasted */
+  ThotBool        DocReadOnly;	/* document is read only */
+  ThotBool        DocExportStructure; /* the logical structure contains
+					 only the skeleton elements */
+  int             DocLabelExpMax;	/* maximum value of element labels */
+  int		  DocMaxPairIdent;/* higher document pair id. value */
+  ThotBool        DocModified;	/* the document was modified */
+  ThotBool        DocUpdated;	/* any change occurs since the last reset */
+  int		  DocBackUpInterval; /* interval between automatic backup */
+  int             DocNTypedChars;	/* number of characters typed since the
 					   last time the document was saved */
-	PtrOutReference	DocNewOutRef;	/* beginning of the list of outgoing
+  PtrOutReference DocNewOutRef;	/* beginning of the list of outgoing
 					   references created since the last
 					   time the document was saved */
-	PtrOutReference	DocDeadOutRef;	/* beginning of the list of outgoing
+  PtrOutReference DocDeadOutRef;	/* beginning of the list of outgoing
 					   references deleteted since the last
 					   time the document was saved */
-	PtrChangedReferredEl	DocChangedReferredEl;	/* beginning of the
-					   list of elements referenced from
-					   outside that were deleted or moved
-					   since the last time the document
-					   was saved */
-#define MAX_LANGUAGES_DOC 10
-	int		DocNLanguages;	/* number of languages actually used */
-	Language	DocLanguages[MAX_LANGUAGES_DOC]; /* the languages used
-					   in the document */
-	/* table of the structure schemas (natures) used in the document */
-#define MAX_NATURES_DOC 20	/* maximum length of the nature table */
-	int		DocNNatures;	/* number of natures */
-	Name		DocNatureName[MAX_NATURES_DOC]; /* names of natures */
-	Name		DocNaturePresName[MAX_NATURES_DOC]; /* names of the
-				presentation schemas of these natures */
-	PtrSSchema	DocNatureSSchema[MAX_NATURES_DOC];/* structure schemas
-					   of these natures */
-	/* information used while reading the pivot file */
-	int		DocPivotVersion;/* pivot version number */
-        PtrReferredDescr DocLabels;	/* external labels */
-	UCHAR_T	DocCheckingMode; /* check document structure against
-					    the structure schemas */
-	ThotBool		DocPivotError;  /* an format error has been detected */
-
-	/* history of last changes made in the document */
-	PtrEditOperation DocLastEdit;	/* latest editing operation */
-	int		DocNbEditsInHistory; /* number of editing commands
-					     recorded in the history */
-	ThotBool		DocEditSequence;/* indicate whether a sequence of
-					   editing operations is open */
-	/* queue of latest undone commands */
-	PtrEditOperation DocLastUndone;	/* latest editing operation undone */
-	int		DocNbUndone;	/* number of undone editing commands */
-    CHARSET  Charset;
-
+  PtrChangedReferredEl DocChangedReferredEl;	/* beginning of the
+						   list of elements referenced from
+						   outside that were deleted or moved
+						   since the last time the document
+						   was saved */
+  int		  DocNLanguages;	/* number of languages actually used */
+  Language	  DocLanguages[MAX_LANGUAGES_DOC]; /* the languages used
+						    in the document */
+  /* table of the structure schemas (natures) used in the document */
+  int		  DocNNatures;	/* number of natures */
+  Name		  DocNatureName[MAX_NATURES_DOC]; /* names of natures */
+  Name		  DocNaturePresName[MAX_NATURES_DOC]; /* names of the presentation
+						       schemas of these natures */
+  PtrSSchema	  DocNatureSSchema[MAX_NATURES_DOC];/* structure schemas
+						       of these natures */
+  /* information used while reading the pivot file */
+  int		  DocPivotVersion;/* pivot version number */
+  PtrReferredDescr DocLabels;	/* external labels */
+  UCHAR_T	  DocCheckingMode; /* check document structure against
+				      the structure schemas */
+  ThotBool	  DocPivotError;  /* an format error has been detected */
+  
+  /* history of last changes made in the document */
+  PtrEditOperation DocLastEdit;	/* latest editing operation */
+  int		  DocNbEditsInHistory; /* number of editing commands
+					recorded in the history */
+  ThotBool	  DocEditSequence;/* indicate whether a sequence of
+				     editing operations is open */
+  /* queue of latest undone commands */
+  PtrEditOperation DocLastUndone;	/* latest editing operation undone */
+  int		  DocNbUndone;	        /* number of undone editing commands */
+  CHARSET         DocCharset;           /* charset of the document */
+  ThotBool	  DocDefaultCharset;    /* TRUE if the charset is set by default */
 } DocumentDescr;
 
 #endif /* __TYPE_INT_H__ */
