@@ -251,6 +251,11 @@ typedef struct _Box
       short             _BxPacking_;     /* Packing */
       PtrTabSpan        _BxSpans_;	/* list of spanned cells */
     } s3;
+    struct /* BoComplete for Graphics = 'C' */
+    {
+      int        	_BxRx_;         /* Horizontal radius */
+      int        	_BxRy_;         /* Vertical radius */
+    } s4;
   } u;
 } Box;
 
@@ -272,6 +277,8 @@ typedef struct _Box
 #define BxTable u.s3._BxColumns_
 #define BxRows u.s3._BxRows_
 #define BxSpans u.s3._BxSpans_
+#define BxRx u.s4._BxRx_
+#define BxRy u.s4._BxRy_
 
 typedef struct C_points_
 {
@@ -475,6 +482,10 @@ typedef struct _AbstractBox
       char     _AbShape_;         /* Drawing code */
       char     _AbGraphAlphabet_; /* Alphabet used*/
       char     _AbRealShape_;     /* Effective drawing code */
+      int      _AbRx_;            /* Horizontal radius */
+      int      _AbRy_;            /* Vertical radius */     
+      TypeUnit _AbRxUnit_;        /* Unit for the horizontal radius */
+      TypeUnit _AbRyUnit_;        /* Unit for the vertical radius */
     } s2;
     struct /* AbLeafType = LtPicture */
     {
@@ -498,6 +509,10 @@ typedef struct _AbstractBox
 #define AbShape u.s2._AbShape_
 #define AbGraphAlphabet u.s2._AbGraphAlphabet_
 #define AbRealShape u.s2._AbRealShape_
+#define AbRx u.s2._AbRx_
+#define AbRy u.s2._AbRy_
+#define AbRxUnit u.s2._AbRxUnit_
+#define AbRyUnit u.s2._AbRyUnit_
 #define AbPictInfo u.s3._AbPictInfo_
 #define AbPolyLineBuffer u.s4._AbPolyLineBuffer_
 #define AbPolyLineShape u.s4._AbPolyLineShape_
