@@ -425,6 +425,12 @@ PRuleType    pRule;
             case PtBorderLeftWidth:
                return PRBorderLeftWidth;
                break;
+            case PtXRadius:
+               return PRXRadius;
+               break;
+            case PtYRadius:
+               return PRYRadius;
+               break;
             case PtBorderTopColor:
                return PRBorderTopColor;
                break;
@@ -1132,6 +1138,7 @@ ThotBool           remove;
       viewSch = pDoc->DocView[view].DvPSchemaView;
       while (pAb != NULL)
 	{
+	  /* there is probably a problem with visibility rules: pAb doesn't exists */
 	  found = FALSE;
 	  if (elType > 0)
 	    /* presentation rules are associated with an element type */
@@ -1242,6 +1249,36 @@ int                 viewToApply;
    RemoveSpecifPres (pElRoot, pDoc, RulesS, viewToApply);
 }
 
+/*----------------------------------------------------------------------
+   TtaIsCSSPRule
+
+   Check whether a presentation rule is associated with a CSS rule
+
+   Parameters:
+   pRule: the presentation rule to be tested.
+
+   Return value:
+   TRUE if pRule is a CSS rule
+  ----------------------------------------------------------------------*/
+#ifdef __STDC__
+ThotBool            TtaIsCSSPRule (PRule pRule)
+#else  /* __STDC__ */
+ThotBool            TtaIsCSSPRule (pRule)
+PRule               pRule;
+#endif /* __STDC__ */
+{
+   ThotBool         ret;
+
+   ret = FALSE;
+   UserErrorCode = 0;
+   if (pRule == NULL)
+     TtaError (ERR_invalid_parameter);
+   else
+     /* parameter pRule is correct */
+     ret = (((PtrPRule) pRule)->PrLevel != 0);
+   return ret;
+}
+
 /*----------------------------------------------------------------------
    TtaRemovePRule
 
