@@ -1,6 +1,6 @@
 /*
  *
- *  (c) COPYRIGHT INRIA, 1996.
+ *  (c) COPYRIGHT INRIA, 1996-2000
  *  Please first read the full copyright statement in file COPYRIGHT.
  *
  */
@@ -106,7 +106,8 @@ Thotbool            selected;
 
       /* show the selection on the whole image */
       if (selected)
-	if (pFrame->FrSelectOnePosition)
+	if (pFrame->FrSelectOnePosition &&
+	    pBox == pFrame->FrSelectionBegin.VsBox)
 	  /* show the selection on the beginning or the end of the image */
 	  DisplayStringSelection (frame,
 				  pFrame->FrSelectionBegin.VsXPos,
@@ -135,8 +136,7 @@ ThotBool            selected;
   ViewFrame          *pFrame;
   ThotBool            withbackground;
   int                 xd, yd, i;
-  int                 fg;
-  int                 bg;
+  int                 fg, bg;
   int                 op, RO;
   int                 width, height;
 
@@ -172,6 +172,11 @@ ThotBool            selected;
 	  
 	  if (withbackground)
 	    DrawRectangle (frame, 0, 0, xd, yd, width, height, 0, 0, 0, bg, 2);
+	  /* display the background selection */
+	  if (selected &&
+	      !pFrame->FrSelectOnePosition &&
+	      pFrame->FrSelectionBegin.VsXPos != pBox->BxW)
+	    DisplayStringSelection (frame, 0, pBox->BxW, pBox);
 	  
 	  /* Line thickness */
 	  i = GetLineWeight (pBox->BxAbstractBox);
@@ -202,19 +207,25 @@ ThotBool            selected;
 	    case 'U':
 	      DrawUnion (frame, xd, yd, width, height, font, RO, op, fg);
 	      break;
+            case 'o':
+	      DrawHorizontalBrace (frame, i, 5, xd, yd, width, height, 0, RO, op, fg);
+	      break;
+            case 'u':
+	      DrawHorizontalBrace (frame, i, 5, xd, yd, width, height, 1, RO, op, fg);
+	      break;
 	    case 'h':
 	      DrawHorizontalLine (frame, i, 5, xd, yd, width, height, 1, RO, op, fg);
 	      break;
 	    case 'v':
 	      DrawVerticalLine (frame, i, 5, xd, yd, width, height, 1, RO, op, fg);
 	      break;
-	    case '>':
+	    case 'R':
 	      DrawArrow (frame, i, 5, xd, yd, width, height, 0, RO, op, fg);
 	      break;
 	    case '^':
 	      DrawArrow (frame, i, 5, xd, yd, width, height, 90, RO, op, fg);
 	      break;
-	    case '<':
+	    case 'L':
 	      DrawArrow (frame, i, 5, xd, yd, width, height, 180, RO, op, fg);
 	      break;
 	    case 'V':
@@ -238,6 +249,18 @@ ThotBool            selected;
 	    case ']':
 	      DrawBracket (frame, i, xd, yd, width, height, 1, font, RO, op, fg);
 	      break;
+	    case '<':
+	      DrawPointyBracket (frame, i, xd, yd, width, height, 0, font, RO, op, fg);
+	      break;
+	    case '>':
+	      DrawPointyBracket (frame, i, xd, yd, width, height, 1, font, RO, op, fg);
+	      break;
+	    case '|':
+	      DrawVerticalLine (frame, i, 5, xd, yd, width, height, 1, RO, op, fg);
+	      break;
+	    case 'D':
+	      DrawDoubleVerticalLine (frame, i, 5, xd, yd, width, height, 1, RO, op, fg);
+	      break;
 	    default:
 	      break;
 	    }
@@ -250,22 +273,15 @@ ThotBool            selected;
 	    }
 
 	  /* show the selection on the beginning or the end of the image */
-	  if (selected)
-	    {
-	      if (pFrame->FrSelectOnePosition ||
-		  pFrame->FrSelectionBegin.VsXPos == pBox->BxW)
-		/* display a carret 
-		   or the selection starts at the end of the box */
-		DisplayStringSelection (frame,
-					pFrame->FrSelectionBegin.VsXPos,
-					pFrame->FrSelectionBegin.VsXPos + 2,
-					pBox);
-	      else
-		DisplayStringSelection (frame,
-					pFrame->FrSelectionBegin.VsXPos,
-					pFrame->FrSelectionEnd.VsXPos,
-					pBox);
-	    }
+	  if (selected &&
+	      (pFrame->FrSelectOnePosition ||
+	       pFrame->FrSelectionBegin.VsXPos == pBox->BxW))
+	    /* display a carret 
+	       or the selection starts at the end of the box */
+	    DisplayStringSelection (frame,
+				    pFrame->FrSelectionBegin.VsXPos,
+				    pFrame->FrSelectionBegin.VsXPos + 2,
+				    pBox);
 	}
     }
 }
@@ -362,14 +378,15 @@ ThotBool            selected;
   PtrAbstractBox      pAb;
   int                 i, xd, yd;
   int                 op, RO;
-  int                 fg;
-  int                 bg;
+  int                 fg, bg;
+  int                 pat;
   int                 style;
   int                 width, height;
 
   pAb = pBox->BxAbstractBox;
   fg = pAb->AbForeground;
   bg = pAb->AbBackground;
+  pat = pAb->AbFillPattern;
   pFrame = &ViewFrameTable[frame - 1];
   if (pAb->AbVisibility >= pFrame->FrVisibility)
     {
@@ -459,7 +476,7 @@ ThotBool            selected;
 	    DrawRectangle (frame, 0, 0, xd, yd, width, height, RO, op, fg, bg, 4);
 	    break;
 	  case '0':
-	    DrawRectangle (frame, 0, 0, xd, yd, width, height, RO, op, fg, bg, pAb->AbFillPattern);
+	    DrawRectangle (frame, 0, 0, xd, yd, width, height, RO, op, fg, bg, pat);
 	    break;
 	  case '1':
 	  case '2':
@@ -470,17 +487,31 @@ ThotBool            selected;
 	  case '7':
 	  case '8':
 	  case 'R':
-	    DrawRectangle (frame, i, style, xd, yd, width, height, RO, op, fg, bg, pAb->AbFillPattern);
+	    DrawRectangle (frame, i, style, xd, yd, width, height, RO, op, fg, bg, pat);
+	    break;
+	  case 'g':
+	    /* Coords of the line are given by the enclosing box */
+	    pAb = pAb->AbEnclosing;
+	    if ((pAb->AbHorizPos.PosEdge == Left && pAb->AbVertPos.PosEdge == Top) ||
+		(pAb->AbHorizPos.PosEdge == Right && pAb->AbVertPos.PosEdge == Bottom))
+	      /* draw a \ */
+	      DrawSlash (frame, i, style, xd, yd, width, height, 1, RO, op, fg);
+	    else
+	      /* draw a / */
+	      DrawSlash (frame, i, style, xd, yd, width, height, 0, RO, op, fg);
 	    break;
 	  case 'C':
-	    DrawOval (frame, i, style, xd, yd, width, height, RO, op, fg, bg, pAb->AbFillPattern);
+	    if (pBox->BxRx == 0 && pBox->BxRy == 0)
+	      DrawRectangle (frame, i, style, xd, yd, width, height, RO, op, fg, bg, pat);
+	    else
+	      DrawOval (frame, i, style, xd, yd, width, height, pBox->BxRx, pBox->BxRy, RO, op, fg, bg, pat);
 	    break;
 	  case 'L':
-	    DrawDiamond (frame, i, style, xd, yd, width, height, RO, op, fg, bg, pAb->AbFillPattern);
+	    DrawDiamond (frame, i, style, xd, yd, width, height, RO, op, fg, bg, pat);
 	    break;
 	  case 'a':
 	  case 'c':
-	    DrawEllips (frame, i, style, xd, yd, width, height, RO, op, fg, bg, pAb->AbFillPattern);
+	    DrawEllips (frame, i, style, xd, yd, width, height, RO, op, fg, bg, pat);
 	    break;
 	  case 'h':
 	    DrawHorizontalLine (frame, i, style, xd, yd, width, height, 1, RO, op, fg);
@@ -501,12 +532,10 @@ ThotBool            selected;
 	    DrawVerticalLine (frame, i, style, xd, yd, width, height, 2, RO, op, fg);
 	    break;
 	  case '/':
-	    DrawSlash (frame, i, style, xd, yd, width,
-		       height, 0, RO, op, fg);
+	    DrawSlash (frame, i, style, xd, yd, width, height, 0, RO, op, fg);
 	    break;
 	  case '\\':
-	    DrawSlash (frame, i, style, xd, yd, width,
-		       height, 1, RO, op, fg);
+	    DrawSlash (frame, i, style, xd, yd, width, height, 1, RO, op, fg);
 	    break;
 	  case '>':
 	     DrawArrow (frame, i, style, xd, yd, width, height, 0, RO, op, fg);
@@ -536,11 +565,11 @@ ThotBool            selected;
 	    
 	  case 'P':
 	    DrawRectangleFrame (frame, i, style, xd, yd, width,
-				height, RO, op, fg, bg, pAb->AbFillPattern);
+				height, RO, op, fg, bg, pat);
 	    break;
 	  case 'Q':
 	    DrawEllipsFrame (frame, i, style, xd, yd, width,
-			     height, RO, op, fg, bg, pAb->AbFillPattern);
+			     height, RO, op, fg, bg, pat);
 	    break;
 	  case 'W':
 	    DrawCorner (frame, i, style, xd, yd, width, height, 0, RO, op, fg);
@@ -568,7 +597,10 @@ ThotBool            selected;
 
 	/* show the selection on the whole image */
 	if (selected)
-	  DisplayPointSelection (frame, pBox, 0);
+	  if (pFrame->FrSelectOnePosition)
+	    DisplayPointSelection (frame, pBox, pFrame->FrSelectionBegin.VsIndBox);
+	  else
+	    DisplayPointSelection (frame, pBox, 0);
      }
 }
 
@@ -581,77 +613,79 @@ ThotBool            selected;
   Box-Width/Lim-X ratio horizontally and Box-Height/Lim-Y ratio vertically.
   ----------------------------------------------------------------------*/
 #ifdef __STDC__
-void                PolyTransform (PtrBox pBox)
+static void         PolyTransform (PtrBox pBox, int frame)
 #else  /* __STDC__ */
-void                PolyTransform (pBox)
+static void         PolyTransform (pBox, frame)
 PtrBox              pBox;
+int                 frame;
 #endif /* __STDC__ */
 {
-   float               xRatio, yRatio, pointIndex;
-   PtrTextBuffer       adbuff;
-   int                 i;
-   int                 j, val;
-   int                 width, height;
+  float               xRatio, yRatio, pointIndex;
+  PtrTextBuffer       adbuff;
+  int                 i;
+  int                 j, val;
+  int                 width, height;
+  int                 zoom;
 
-   /* box sizes have to be positive */
-   width = pBox->BxW;
-   if (width < 0)
-     width = 0;
-   height = pBox->BxH;
-   if (height < 0)
-     height = 0;
+  /* box sizes have to be positive */
+  width = pBox->BxW;
+  if (width < 0)
+    width = 0;
+  height = pBox->BxH;
+  if (height < 0)
+    height = 0;
+  zoom = ViewFrameTable[frame - 1].FrMagnification;
+  val = PixelValue (pBox->BxBuffer->BuPoints[0].XCoord, UnPixel, NULL, zoom);
+  /* Compute ratio for axis X */
+  if (val != width && pBox->BxBuffer->BuPoints[0].XCoord > 0)
+    {
+      val = LogicalValue (width, UnPixel, NULL, zoom);
+      pointIndex = (float) pBox->BxBuffer->BuPoints[0].XCoord / pBox->BxXRatio;
+      /* save the new distortion ratio between box and abstract box */
+      pBox->BxXRatio = (float) val / pointIndex;
+      /* ratio applied to the box */
+      xRatio = (float) val / (float) pBox->BxBuffer->BuPoints[0].XCoord;
+      pBox->BxBuffer->BuPoints[0].XCoord = val;
+    }
+  else
+    xRatio = 1.0;
 
-   /* Compute ratio for axis X */
-   val = PixelToPoint (width) * 1000;
-   if (val != pBox->BxBuffer->BuPoints[0].XCoord
-       && pBox->BxBuffer->BuPoints[0].XCoord > 0)
-     {
-	pointIndex = (float) pBox->BxBuffer->BuPoints[0].XCoord / pBox->BxXRatio;
-	/* save the new distortion ratio between box and abstract box */
-	pBox->BxXRatio = (float) val / pointIndex;
-	/* ratio applied to the box */
-	xRatio = (float) val / (float) pBox->BxBuffer->BuPoints[0].XCoord;
-	pBox->BxBuffer->BuPoints[0].XCoord = val;
-     }
-   else
-      xRatio = 1.0;
+  /* Compute ratio for axis Y */
+  val = PixelValue (pBox->BxBuffer->BuPoints[0].YCoord, UnPixel, NULL, zoom);
+  if (val != height && pBox->BxBuffer->BuPoints[0].YCoord > 0)
+    {
+      val = LogicalValue (height, UnPixel, NULL, zoom);
+      pointIndex = (float) pBox->BxBuffer->BuPoints[0].YCoord / pBox->BxYRatio;
+      /* save the new distortion ratio between box and abstract box */
+      pBox->BxYRatio = (float) val / pointIndex;
+      /* ratio applied to the box */
+      yRatio = (float) val / (float) pBox->BxBuffer->BuPoints[0].YCoord;
+      pBox->BxBuffer->BuPoints[0].YCoord = val;
+    }
+  else
+    yRatio = 1.0;
 
-   /* Compute ratio for axis Y */
-   val = PixelToPoint (height) * 1000;
-   if (val != pBox->BxBuffer->BuPoints[0].YCoord
-       && pBox->BxBuffer->BuPoints[0].YCoord > 0)
-     {
-	pointIndex = (float) pBox->BxBuffer->BuPoints[0].YCoord / pBox->BxYRatio;
-	/* save the new distortion ratio between box and abstract box */
-	pBox->BxYRatio = (float) val / pointIndex;
-	/* ratio applied to the box */
-	yRatio = (float) val / (float) pBox->BxBuffer->BuPoints[0].YCoord;
-	pBox->BxBuffer->BuPoints[0].YCoord = val;
-     }
-   else
-      yRatio = 1.0;
-
-   if (xRatio != 1 || yRatio != 1)
-     {
-	j = 1;
-	adbuff = pBox->BxBuffer;
-	val = pBox->BxNChars;
-	for (i = 1; i < val; i++)
-	  {
-	     if (j >= adbuff->BuLength)
-	       {
-		  if (adbuff->BuNext != NULL)
-		    {
-		       /* Next buffer */
-		       adbuff = adbuff->BuNext;
-		       j = 0;
-		    }
-	       }
-	     adbuff->BuPoints[j].XCoord = (int) ((float) adbuff->BuPoints[j].XCoord * xRatio);
-	     adbuff->BuPoints[j].YCoord = (int) ((float) adbuff->BuPoints[j].YCoord * yRatio);
-	     j++;
-	  }
-     }
+  if (xRatio != 1 || yRatio != 1)
+    {
+      j = 1;
+      adbuff = pBox->BxBuffer;
+      val = pBox->BxNChars;
+      for (i = 1; i < val; i++)
+	{
+	  if (j >= adbuff->BuLength)
+	    {
+	      if (adbuff->BuNext != NULL)
+		{
+		  /* Next buffer */
+		  adbuff = adbuff->BuNext;
+		  j = 0;
+		}
+	    }
+	  adbuff->BuPoints[j].XCoord = (int) ((float) adbuff->BuPoints[j].XCoord * xRatio);
+	  adbuff->BuPoints[j].YCoord = (int) ((float) adbuff->BuPoints[j].YCoord * yRatio);
+	  j++;
+	}
+    }
 }
 
 
@@ -671,8 +705,8 @@ ThotBool            selected;
   ViewFrame          *pFrame;
   int                 i, xd, yd;
   int                 op, RO;
-  int                 fg;
-  int                 bg;
+  int                 fg, bg;
+  int                 pat;
   int                 style, arrow;
   int                 width;
 
@@ -681,10 +715,11 @@ ThotBool            selected;
     return;
 
   /* Transform the polyline if the box size has changed */
-  PolyTransform (pBox);
+  PolyTransform (pBox, frame);
   pAb = pBox->BxAbstractBox;
   fg = pAb->AbForeground;
   bg = pAb->AbBackground;
+  pat = pAb->AbFillPattern;
   pFrame = &ViewFrameTable[frame - 1];
   if (pAb->AbVisibility >= pFrame->FrVisibility)
     {
@@ -756,20 +791,20 @@ ThotBool            selected;
 	    arrow = 3;
 	  /* compute control points */
 	  if (pBox->BxPictInfo == NULL)
-	    pBox->BxPictInfo = (int *) ComputeControlPoints (pBox->BxBuffer, pBox->BxNChars);
+	    pBox->BxPictInfo = (int *) ComputeControlPoints (pBox->BxBuffer, pBox->BxNChars, ViewFrameTable[frame - 1].FrMagnification);
 	  DrawCurb (frame, i, style, xd, yd, pBox->BxBuffer,
 		    pBox->BxNChars, RO, op, fg, arrow, (C_points *) pBox->BxPictInfo);
 	  break;
 	case 'p':	/* polygon */
 	  DrawPolygon (frame, i, style, xd, yd, pBox->BxBuffer,
-		       pBox->BxNChars, RO, op, fg, bg, pAb->AbFillPattern);
+		       pBox->BxNChars, RO, op, fg, bg, pat);
 	  break;
 	case 's':	/* closed spline */
 	  /* compute control points */
 	  if (pBox->BxPictInfo == NULL)
-	    pBox->BxPictInfo = (int *) ComputeControlPoints (pBox->BxBuffer, pBox->BxNChars);
+	    pBox->BxPictInfo = (int *) ComputeControlPoints (pBox->BxBuffer, pBox->BxNChars, ViewFrameTable[frame - 1].FrMagnification);
 	  DrawSpline (frame, i, style, xd, yd, pBox->BxBuffer,
-		      pBox->BxNChars, RO, op, fg, bg, pAb->AbFillPattern, (C_points *) pBox->BxPictInfo);
+		      pBox->BxNChars, RO, op, fg, bg, pat, (C_points *) pBox->BxPictInfo);
 	  break;
 	default:
 	  break;
@@ -786,7 +821,7 @@ ThotBool            selected;
       if (selected)
 	if (pFrame->FrSelectOnePosition)
 	  DisplayPointSelection (frame, pBox, pFrame->FrSelectionBegin.VsIndBox);
-	else
+	else if (pBox->BxNChars > 1)
 	  DisplayPointSelection (frame, pBox, 0);
     }
 }
@@ -837,6 +872,7 @@ ThotBool            selected;
   adbuff = NULL;
   indbuff = 0;
   restbl = 0;
+  nbcar = 0;
   pAb = pBox->BxAbstractBox;
   
   /* do we have to display stars instead of characters? */
@@ -1415,14 +1451,16 @@ int                 ymax;
       (pAb->AbLeafType == LtPolyLine && box->BxNChars == 1))
     {
       /* Empty */
+      selected = (box == pFrame->FrSelectionBegin.VsBox &&
+		  box == pFrame->FrSelectionEnd.VsBox);
       if (pAb->AbLeafType == LtSymbol)
 	DisplayEmptyBox (box, frame, '2', selected);
-      else
+      else if (pAb->AbLeafType != LtPolyLine && pAb->AbLeafType != LtGraphics)
 	{
-	   if (selected)
-	     DisplayStringSelection (frame, 0, box->BxWidth, box);
-	   else if (ThotLocalActions[T_emptybox] != NULL)
-	     (*ThotLocalActions[T_emptybox]) (box, frame, '2', selected);
+	  if (selected)
+	    DisplayStringSelection (frame, 0, box->BxWidth, box);
+	  else if (ThotLocalActions[T_emptybox] != NULL)
+	    (*ThotLocalActions[T_emptybox]) (box, frame, '2', selected);
 	}
     }
   else if (pAb->AbLeafType == LtText)
