@@ -123,10 +123,14 @@ int                 pointselect;
 		}
 	      if (pointselect == 0 || pointselect == i)
 		{
-		  x = leftX + PointToPixel (pBuffer->BuPoints[j].XCoord / 1000);
+		  x = leftX + PixelValue (pBuffer->BuPoints[j].XCoord,
+					  UnPixel, NULL,
+					  ViewFrameTable[frame - 1].FrMagnification);
 		  if (x > rightX)
 		    x =  rightX;
-		  y = topY + PointToPixel (pBuffer->BuPoints[j].YCoord / 1000);
+		  y = topY + PixelValue (pBuffer->BuPoints[j].YCoord,
+					 UnPixel, NULL,
+					 ViewFrameTable[frame - 1].FrMagnification);
 		  if (y > bottomY)
 		    y =  bottomY;
 		  DrawRectangle (frame, 0, 0,
@@ -138,6 +142,50 @@ int                 pointselect;
 	      j++;
 	    }
 	}
+      else if (pointselect != 0)
+	/* Keep in mind the selected caracteristic point       */
+	/*            1-------------2-------------3            */
+	/*            |                           |            */
+	/*            |                           |            */
+	/*            8                           4            */
+	/*            |                           |            */
+	/*            |                           |            */
+	/*            7-------------6-------------5            */
+	switch (pointselect)
+	  {
+	  case 1:
+	    DrawRectangle (frame, 0, 0, leftX, topY, thick, thick,
+			   0, 0, 0, InsertColor, 2);
+	    break;
+	  case 2:
+	    DrawRectangle (frame, 0, 0, minX, topY, thick, thick,
+			   0, 0, 0, InsertColor, 2);
+	    break;
+	  case 3:
+	    DrawRectangle (frame, 0, 0, rightX, topY, thick, thick,
+			   0, 0, 0, InsertColor, 2);
+	    break;
+	  case 4:
+	    DrawRectangle (frame, 0, 0, rightX, middleY, thick, thick,
+			   0, 0, 0, InsertColor, 2);
+	    break;
+	  case 5:
+	    DrawRectangle (frame, 0, 0, rightX, bottomY, thick, thick,
+			   0, 0, 0, InsertColor, 2);
+	    break;
+	  case 6:
+	    DrawRectangle (frame, 0, 0, minX, bottomY, thick, thick,
+			   0, 0, 0, InsertColor, 2);
+	    break;
+	  case 7:
+	    DrawRectangle (frame, 0, 0, leftX, bottomY, thick, thick,
+			   0, 0, 0, InsertColor, 2);
+	    break;
+	  case 8:
+	    DrawRectangle (frame, 0, 0, leftX, middleY, thick, thick,
+			   0, 0, 0, InsertColor, 2);
+	    break;
+	  }
       else if (pAb->AbLeafType == LtGraphics && pAb->AbVolume != 0)
 	/* C'est une boite graphique */
 	/* On marque en noir les points caracteristiques de la boite */
@@ -297,6 +345,29 @@ int                 pointselect;
 	    DrawRectangle (frame, 0, 0, leftX, bottomY, thick, thick,
 			   0, 0, 0, InsertColor, 2);
 	    break;
+	  case 'g':
+	    /* Coords of the line are given by the enclosing box */
+	    pAb = pAb->AbEnclosing;
+	    if ((pAb->AbHorizPos.PosEdge == Left && pAb->AbVertPos.PosEdge == Top) ||
+		(pAb->AbHorizPos.PosEdge == Right && pAb->AbVertPos.PosEdge == Bottom))
+	      {
+		/* draw a \ */
+		/* 2 characteristic points */
+		DrawRectangle (frame, 0, 0, leftX, topY, thick, thick,
+			       0, 0, 0, InsertColor, 2);
+		DrawRectangle (frame, 0, 0, rightX, bottomY, thick, thick,
+			       0, 0, 0, InsertColor, 2);
+	      }
+	    else
+	      {
+		/* draw a / */
+		/* 2 characteristic points */
+		DrawRectangle (frame, 0, 0, rightX, topY, thick, thick,
+			       0, 0, 0, InsertColor, 2);
+		DrawRectangle (frame, 0, 0, leftX, bottomY, thick, thick,
+			       0, 0, 0, InsertColor, 2);
+	      }
+	    break;
 	  default:
 	    break;
 	  }
@@ -326,9 +397,9 @@ PtrBox       pBox;
       pFrame = &ViewFrameTable[frame - 1];
       pAb = pBox->BxAbstractBox;
       
-      if (pBox->BxType == BoGhost
-	  || (pAb != NULL
-	      && TypeHasException (ExcHighlightChildren, pAb->AbElement->ElTypeNumber, pAb->AbElement->ElStructSchema)))
+      if (pBox->BxType == BoGhost ||
+	  (pAb != NULL &&
+	   TypeHasException (ExcHighlightChildren, pAb->AbElement->ElTypeNumber, pAb->AbElement->ElStructSchema)))
 	{
 	  /* the box is not displayed, select its children */
 	  if (pAb->AbFirstEnclosed != NULL)
@@ -398,8 +469,11 @@ PtrAbstractBox      pAb;
 		 pAb->AbLeafType == LtCompound ||
 		 (pAb != pFrame->FrSelectionBegin.VsBox->BxAbstractBox &&
 		  pAb != pFrame->FrSelectionEnd.VsBox->BxAbstractBox))
+	       {
 		 /* it's not a terminal extremity of the selection */
-	       DisplayBgBoxSelection (frame, pAb->AbBox);
+		 if (pAb->AbVolume > 0)
+		   DisplayBgBoxSelection (frame, pAb->AbBox);
+	       }
 	  }
 	else if (pAb->AbLeafType == LtCompound)
 	   /* Sinon on parcours le sous-arbre */
@@ -494,7 +568,8 @@ ThotBool            status;
 	  {
 	     pFrame = &ViewFrameTable[frame - 1];
 	     pAb->AbSelected = status;
-	     if ( pFrame->FrClipXBegin == 0 && pFrame->FrClipXEnd == 0)
+	     if ( pFrame->FrClipXBegin == 0 && pFrame->FrClipXEnd == 0 &&
+		  pAb->AbVolume != 0)
 	       /* ready to un/display the current selection */
 	       /* doesn't display selection limits */
 	       if (pFrame->FrSelectionBegin.VsBox == NULL ||
