@@ -1,6 +1,6 @@
 /*
  *
- *  (c) COPYRIGHT INRIA, 1996.
+ *  (c) COPYRIGHT INRIA, 1996-2000
  *  Please first read the full copyright statement in file COPYRIGHT.
  *
  */
@@ -506,6 +506,8 @@ DocViewNumber       view;
 
 #endif /* __STDC__ */
 {
+   PtrGuestViewDescr  guestView, next;
+
    view--;
    if (pDoc->DocViewRootAb[view] != NULL)
       FreeAbView (pDoc->DocViewRootAb[view], pDoc->DocViewFrame[view]);
@@ -513,6 +515,17 @@ DocViewNumber       view;
    pDoc->DocView[view].DvSSchema = NULL;
    pDoc->DocView[view].DvPSchemaView = 0;
    pDoc->DocView[view].DvSync = FALSE;
+   if (pDoc->DocView[view].DvFirstGuestView)
+      {
+      guestView = pDoc->DocView[view].DvFirstGuestView;
+      pDoc->DocView[view].DvFirstGuestView = NULL;
+      while (guestView)
+	 {
+	 next = guestView->GvNextGuestView;
+	 TtaFreeMemory (guestView);
+	 guestView = next;
+	 }
+      }
    pDoc->DocViewFrame[view] = 0;
    pDoc->DocViewVolume[view] = 0;
    pDoc->DocViewFreeVolume[view] = 0;
@@ -1088,8 +1101,8 @@ PtrAttribute        pAttr;
 Document            document;
 #endif /* __STDC__ */
 {
-   ThotBool            inheritance, comparaison;
    PtrElement          pElChild;
+   ThotBool            inheritance, comparaison, reDisp;
 
    if (LoadedDocument[document - 1] == NULL || DocumentOfElement (pEl) == NULL)
      return;
@@ -1102,6 +1115,7 @@ Document            document;
    /* doit-on se preoccuper des heritages et comparaisons d'attributs? */
    inheritance = (pAttr->AeAttrSSchema->SsPSchema->PsNHeirElems[pAttr->AeAttrNum - 1] > 0);
    comparaison = (pAttr->AeAttrSSchema->SsPSchema->PsNComparAttrs[pAttr->AeAttrNum - 1] > 0);
+   reDisp = (documentDisplayMode[document - 1] == DisplayImmediately);
    /* d'abord on applique les regles de presentation liees */
    /* a l'attribut sur l'element lui-meme */
    ApplyAttrPRulesToElem (pEl, LoadedDocument[document - 1], pAttr, pEl, FALSE);
@@ -1126,7 +1140,7 @@ Document            document;
    RedisplayCommand (document);
    /* le nouvel attribut doit etre pris en compte dans */
    /* les copies-inclusions de l'element */
-   RedisplayCopies (pEl, LoadedDocument[document - 1], (ThotBool)(documentDisplayMode[document - 1] == DisplayImmediately));
+   RedisplayCopies (pEl, LoadedDocument[document - 1], reDisp);
 }
 
 /*----------------------------------------------------------------------
