@@ -468,7 +468,7 @@ int                 fg;
   int                 l_end;	/* end of the line       */
   FILE               *fout;
 
-  if (y < 0)
+  if (y < 0 || thick <= 0 || fg < 0)
     return;
   y += FrameTable[frame].FrTopMargin;
   
@@ -543,7 +543,7 @@ int                 fg;
    if (y < 0)
       return;
    y += FrameTable[frame].FrTopMargin;
-   if (thick < 0)
+   if (thick < 0 || fg < 0)
       return;
 
    fout = (FILE *) FrRef[frame]; 
@@ -834,7 +834,7 @@ int                 fg;
     return;
    y += FrameTable[frame].FrTopMargin;
 
-   if (thick <= 0)
+   if (thick <= 0 || fg < 0)
       return;
 
    /* Do we need to change the current color ? */
@@ -959,6 +959,76 @@ int                 fg;
 }
 
 
+/*----------------------------------------------------------------------
+   DrawPointyBracket draw an opening or closing pointy bracket (depending
+   on direction)
+  ----------------------------------------------------------------------*/
+#ifdef __STDC__
+void                DrawPointyBracket (int frame, int thick, int x, int y, int l, int h, int direction, ptrfont font, int RO, int func, int fg)
+#else  /* __STDC__ */
+void                DrawPointyBracket (frame, thick, x, y, l, h, direction, font, RO, func, fg)
+int                 frame;
+int                 thick;
+int                 x;
+int                 y;
+int                 l;
+int                 h;
+int                 direction;
+ptrfont             font;
+int                 RO;
+int                 func;
+int                 fg;
+#endif /* __STDC__ */
+{
+   int                 ey, yf;
+   FILE               *fout;
+
+   fout = (FILE *) FrRef[frame];
+  if (y < 0)
+    return;
+
+   y += FrameTable[frame].FrTopMargin;
+   /* Do we need to change the current color ? */
+   CurrentColor (fout, fg);
+
+   /* Do we need to change the current font ? */
+   CurrentFont (fout, font);
+
+   l--;
+   h--;
+   ey = FontHeight (font);
+   h -= ey;
+   y += FontBase (font);
+
+   if (h < ey / 4)
+     {
+       x = PixelToPoint (x + (l / 2));
+       yf = PixelToPoint (y + h);
+       y = PixelToPoint (y) + 1;
+	/* Made of only one glyph */
+	if (direction == 0)
+	   fprintf (fout, "-%d %d (\341) c\n", yf, x);
+	else
+	   fprintf (fout, "-%d %d (\361)c\n", yf, x);
+     }
+   else
+     {
+	/* Drawn with more than one glyph */
+	if (direction == 0)
+	  /* Trace un crochet ouvrant */
+	  fprintf (fout, "%d -%d %d -%d %d -%d %d %d %d Seg\n",
+		   PixelToPoint (x + l), PixelToPoint (y),
+		   PixelToPoint (x), PixelToPoint (y + (h / 2)),
+		   PixelToPoint (x + l), PixelToPoint (y + h), 5, 1, 3);
+	else
+	  fprintf (fout, "%d -%d %d -%d %d -%d %d %d %d Seg\n",
+		   PixelToPoint (x), PixelToPoint (y),
+		   PixelToPoint (x + l), PixelToPoint (y + (h / 2)),
+		   PixelToPoint (x), PixelToPoint (y + h), 5, 1, 3);
+     }
+}
+
+
 /*----------------------------------------------------------------------
    DrawParenthesis draw a closing or opening parenthesis (direction).
   ----------------------------------------------------------------------*/
@@ -986,7 +1056,7 @@ int                 fg;
       return;
 
    y += FrameTable[frame].FrTopMargin;
-   if (thick < 0)
+   if (thick < 0 || fg < 0)
       return;
 
    fout = (FILE *) FrRef[frame];
@@ -1121,15 +1191,17 @@ int                 pattern;
    y += FrameTable[frame].FrTopMargin;
    fout = (FILE *) FrRef[frame];
 
-   /* Do we need to change the current color ? */
-   if (thick > 0)
-      CurrentColor (fout, fg);
-
    xf = PixelToPoint (x + width);
    yf = PixelToPoint (y + height);
    x = PixelToPoint (x);
    y = PixelToPoint (y);
 
+   /* Do we need to change the current color ? */
+   if (fg < 0)
+     thick = 0;
+   if (thick > 0)
+      CurrentColor (fout, fg);
+
    FillWithPattern (fout, fg, bg, pattern);
    fprintf (fout, "%d -%d %d -%d %d -%d  %d -%d %d %d %d Poly\n", x, y, x, yf, xf, yf, xf, y, style, thick, 4);
 }
@@ -1180,24 +1252,22 @@ int                 arrow;
    y += FrameTable[frame].FrTopMargin;
    xp = yp = 0;
    prevx = prevy = 0;
-   if (thick == 0)
+   if (thick == 0 || fg < 0)
       return;
 
    lg = HL + thick;
 
    /* Do we need to change the current color ? */
    CurrentColor (fout, fg);
-   x = PixelToPoint (x);
-   y = PixelToPoint (y);
    adbuff = buffer;
 
    /* backward arrow  */
    if (arrow == 2 || arrow == 3)
       fprintf (fout, "%d %d -%d %d -%d %d %d %d arr\n", style,
-	       FloatToInt ((float) buffer->BuPoints[2].XCoord / 1000 + x),
-	       FloatToInt ((float) buffer->BuPoints[2].YCoord / 1000 + y),
-	       FloatToInt ((float) buffer->BuPoints[1].XCoord / 1000 + x),
-	       FloatToInt ((float) buffer->BuPoints[1].YCoord / 1000 + y), thick, lg, lg);
+	       PixelToPoint (buffer->BuPoints[2].XCoord + x),
+	       PixelToPoint (buffer->BuPoints[2].YCoord + y),
+	       PixelToPoint (buffer->BuPoints[1].XCoord + x),
+	       PixelToPoint (buffer->BuPoints[1].YCoord + y), thick, lg, lg);
 
    j = 1;
    for (i = 1; i < nb; i++)
@@ -1218,8 +1288,8 @@ int                 arrow;
 	     prevy = FloatToInt (yp);
 	  }
 	/* Coordinate for next point */
-	xp = (float) adbuff->BuPoints[j].XCoord / 1000 + x;
-	yp = (float) adbuff->BuPoints[j].YCoord / 1000 + y;
+	xp = (float) PixelToPoint (adbuff->BuPoints[j].XCoord + x);
+	yp = (float) PixelToPoint (adbuff->BuPoints[j].YCoord + y);
 	fprintf (fout, "%f -%f\n", xp, yp);
 	j++;
      }
@@ -1275,8 +1345,6 @@ int                 pattern;
    /* Do we need to change the current color ? */
    CurrentColor (fout, fg);
    FillWithPattern (fout, fg, bg, pattern);
-   x = PixelToPoint (x);
-   y = PixelToPoint (y);
    adbuff = buffer;
    j = 1;
 
@@ -1292,8 +1360,8 @@ int                 pattern;
 	       }
 	  }
 	/* Coordinate for next point */
-	xp = (float) adbuff->BuPoints[j].XCoord / 1000. + x;
-	yp = (float) adbuff->BuPoints[j].YCoord / 1000. + y;
+	xp = (float) PixelToPoint (adbuff->BuPoints[j].XCoord + x);
+	yp = (float) PixelToPoint (adbuff->BuPoints[j].YCoord + y);
 	fprintf (fout, "%f -%f\n", xp, yp);
 	j++;
      }
@@ -1344,15 +1412,13 @@ C_points           *controls;
   if (y < 0)
     return;
    y += FrameTable[frame].FrTopMargin;
-   if (thick == 0)
+   if (thick == 0 || fg < 0)
       return;
 
    /* Do we need to change the current color ? */
    CurrentColor (fout, fg);
 
    lg = HL + thick;
-   x = PixelToPoint (x);
-   y = PixelToPoint (y);
    j = 1;
    i = 2;
    adbuff = buffer;
@@ -1362,12 +1428,12 @@ C_points           *controls;
    newx = adbuff->BuPoints[j].XCoord;
    newy = adbuff->BuPoints[j].YCoord;
    /* control points for first arc */
-   x1 = (float) lastx / 1000 + x;
-   y1 = (float) lasty / 1000 + y;
-   x2 = (float) (PixelToPoint ((int) (controls[i].lx * 3000)) + lastx) / 4000 + x;
-   y2 = (float) (PixelToPoint ((int) (controls[i].ly * 3000)) + lasty) / 4000 + y;
-   x3 = (float) (PixelToPoint ((int) (controls[i].lx * 3000)) + newx) / 4000 + x;
-   y3 = (float) (PixelToPoint ((int) (controls[i].ly * 3000)) + newy) / 4000 + y;
+   x1 = (float) (lastx + x);
+   y1 = (float) (lasty + y);
+   x2 = (float) (PixelToPoint ((int) controls[i].lx * 3 + lastx / 4 + x));
+   y2 = (float) (PixelToPoint ((int) controls[i].ly * 3 + lasty / 4 + y));
+   x3 = (float) (PixelToPoint ((int) controls[i].lx * 3 + newx / 4 + x));
+   y3 = (float) (PixelToPoint ((int) controls[i].ly * 3 + newy / 4 + y));
 
    /* backward arrow  */
    if (arrow == 2 || arrow == 3)
@@ -1392,21 +1458,21 @@ C_points           *controls;
 	lasty = newy;
 	newx = adbuff->BuPoints[j].XCoord;
 	newy = adbuff->BuPoints[j].YCoord;
-	x1 = (float) lastx / 1000 + x;
-	y1 = (float) lasty / 1000 + y;
+	x1 = (float) lastx + x;
+	y1 = (float) lasty + y;
 	if (i < nb - 2)
 	  {
-	     x2 = (float) PixelToPoint ((int) (controls[i].rx * 1000)) / 1000 + x;
-	     y2 = (float) PixelToPoint ((int) (controls[i].ry * 1000)) / 1000 + y;
-	     x3 = (float) PixelToPoint ((int) (controls[i + 1].lx * 1000)) / 1000 + x;
-	     y3 = (float) PixelToPoint ((int) (controls[i + 1].ly * 1000)) / 1000 + y;
+	     x2 = (float) PixelToPoint ((int) controls[i].rx + x);
+	     y2 = (float) PixelToPoint ((int) controls[i].ry + y);
+	     x3 = (float) PixelToPoint ((int) controls[i + 1].lx + x);
+	     y3 = (float) PixelToPoint ((int) controls[i + 1].ly + y);
 	  }
 	else if (i == nb - 2)
 	  {
-	     x2 = (float) (PixelToPoint ((int) (controls[i].rx * 3000)) + lastx) / 4000 + x;
-	     y2 = (float) (PixelToPoint ((int) (controls[i].ry * 3000)) + lasty) / 4000 + y;
-	     x3 = (float) (PixelToPoint ((int) (controls[i].rx * 3000)) + newx) / 4000 + x;
-	     y3 = (float) (PixelToPoint ((int) (controls[i].ry * 3000)) + newy) / 4000 + y;
+	     x2 = (float) PixelToPoint (((int) controls[i].rx * 3 + lastx) / 4 + x);
+	     y2 = (float) PixelToPoint (((int) controls[i].ry * 3 + lasty) / 4 + y);
+	     x3 = (float) PixelToPoint (((int) controls[i].rx * 3 + newx) / 4 + x);
+	     y3 = (float) PixelToPoint (((int) controls[i].ry * 3 + newy) / 4 + y);
 	  }
      }
    fprintf (fout, "%f -%f %d %d %d Curv\n", x1, y1, style, thick, nb - 1);
@@ -1460,20 +1526,18 @@ C_points           *controls;
    /* Do we need to change the current color ? */
    CurrentColor (fout, fg);
    FillWithPattern (fout, fg, bg, pattern);
-   x = PixelToPoint (x);
-   y = PixelToPoint (y);
    j = 1;
    i = 1;
    adbuff = buffer;
-   x0 = x1 = (float) adbuff->BuPoints[j].XCoord / 1000 + x;
-   y0 = y1 = (float) adbuff->BuPoints[j].YCoord / 1000 + y;
-   x2 = (float) PixelToPoint ((int) ((controls[i].rx * 1000))) / 1000 + x;
-   y2 = (float) PixelToPoint ((int) ((controls[i].ry * 1000))) / 1000 + y;
+   x0 = x1 = (float) adbuff->BuPoints[j].XCoord + x;
+   y0 = y1 = (float) adbuff->BuPoints[j].YCoord + y;
+   x2 = (float) PixelToPoint ((int) controls[i].rx + x);
+   y2 = (float) PixelToPoint ((int) controls[i].ry + y);
 
    for (i = 2; i < nb; i++)
      {
-	x3 = (float) PixelToPoint ((int) ((controls[i].lx * 1000))) / 1000 + x;
-	y3 = (float) PixelToPoint ((int) ((controls[i].ly * 1000))) / 1000 + y;
+	x3 = (float) PixelToPoint ((int) controls[i].lx + x);
+	y3 = (float) PixelToPoint ((int) controls[i].ly + y);
 	fprintf (fout, "%f -%f %f -%f %f -%f\n", x3, y3, x2, y2, x1, y1);
 	j++;
 	if (j >= adbuff->BuLength)
@@ -1485,15 +1549,15 @@ C_points           *controls;
 		  j = 0;
 	       }
 	  }
-	x1 = (float) adbuff->BuPoints[j].XCoord / 1000 + x;
-	y1 = (float) adbuff->BuPoints[j].YCoord / 1000 + y;
-	x2 = (float) PixelToPoint ((int) ((controls[i].rx * 1000))) / 1000 + x;
-	y2 = (float) PixelToPoint ((int) ((controls[i].ry * 1000))) / 1000 + y;
+	x1 = (float) adbuff->BuPoints[j].XCoord + x;
+	y1 = (float) adbuff->BuPoints[j].YCoord + y;
+	x2 = (float) PixelToPoint ((int) controls[i].rx + x);
+	y2 = (float) PixelToPoint ((int) controls[i].ry + y);
      }
 
    /* Close the stroke */
-   x3 = (float) PixelToPoint ((int) ((controls[1].lx * 1000))) / 1000 + x;
-   y3 = (float) PixelToPoint ((int) ((controls[1].ly * 1000))) / 1000 + y;
+   x3 = (float) PixelToPoint ((int) (controls[1].lx)) + x;
+   y3 = (float) PixelToPoint ((int) (controls[1].ly)) + y;
    fprintf (fout, "%f -%f %f -%f %f -%f\n", x3, y3, x2, y2, x1, y1);
    fprintf (fout, "%f -%f %d %d %d Splin\n", x0, y0, style, thick, nb);
 }
@@ -1550,9 +1614,9 @@ int                 pattern;
    color, background color and fill pattern.
   ----------------------------------------------------------------------*/
 #ifdef __STDC__
-void                DrawOval (int frame, int thick, int style, int x, int y, int width, int height, int RO, int active, int fg, int bg, int pattern)
+void                DrawOval (int frame, int thick, int style, int x, int y, int width, int height, int rx, int ry, int RO, int active, int fg, int bg, int pattern)
 #else  /* __STDC__ */
-void                DrawOval (frame, thick, style, x, y, width, height, RO, active, fg, bg, pattern)
+void                DrawOval (frame, thick, style, x, y, width, height, rx, ry, RO, active, fg, bg, pattern)
 int                 frame;
 int                 thick;
 int                 style;
@@ -1560,6 +1624,8 @@ int                 x;
 int                 y;
 int                 width;
 int                 height;
+int                 rx;
+int                 ry;
 int                 RO;
 int                 active;
 int                 fg;
@@ -1567,29 +1633,39 @@ int                 bg;
 int                 pattern;
 #endif /* __STDC__ */
 {
-   int                 arc, xf, yf;
-   FILE               *fout;
+  int                 arc, xf, yf;
+  FILE               *fout;
 
-   fout = (FILE *) FrRef[frame];
+  fout = (FILE *) FrRef[frame];
   if (y < 0)
     return;
+
    y += FrameTable[frame].FrTopMargin;
-
    /* Do we need to change the current color ? */
+   if (fg < 0)
+     thick = 0;
    if (thick > 0)
-      CurrentColor (fout, fg);
+     CurrentColor (fout, fg);
 
-   arc = 3 * 72 / 25.4;
+   /* radius of arcs */
+   if (rx == 0 && ry != 0)
+     rx = ry;
+   else if (ry == 0 && rx != 0)
+     ry = rx;
+   arc = width / 2;
+   if (rx > arc)
+     rx = arc;
+   arc = height / 2;
+   if (ry > arc)
+     ry = arc;
+ 
    xf = PixelToPoint (x + width - 1);
    yf = PixelToPoint (y + height - 1);
    x = PixelToPoint (x);
    y = PixelToPoint (y);
-
    FillWithPattern (fout, fg, bg, pattern);
-   fprintf (fout, "%d %d %d -%d %d -%d %d -%d %d -%d %d -%d %d -%d %d ov\n",
-	    style, thick,
-	    /*5 */ x, y, /*4 */ x, yf, /*3 */ xf, yf, /*2 */ xf, y, /*1 */ x, y, /*o */ x, yf - arc,
-	    arc);
+   fprintf (fout, "%d %d %d -%d %d -%d %d %d oval\n", style, thick,
+	    x, y, xf, yf, PixelToPoint (rx), PixelToPoint (ry));
 }
 
 
@@ -1626,6 +1702,8 @@ int                 pattern;
    if (y < 0)
       return;
    /* Do we need to change the current color ? */
+   if (fg < 0)
+     thick = 0;
    if (thick > 0)
       CurrentColor (fout, fg);
    width = width / 2;
@@ -1675,7 +1753,7 @@ int                 fg;
    if (y < 0)
       return;
    y += FrameTable[frame].FrTopMargin;
-   if (thick <= 0)
+   if (thick <= 0 || fg < 0)
       return;
    fout = (FILE *) FrRef[frame];
 
@@ -1736,6 +1814,8 @@ int                 pattern;
     return;
    y += FrameTable[frame].FrTopMargin;
    /* Do we need to change the current color ? */
+   if (fg < 0)
+     thick = 0;
    if (thick > 0)
       CurrentColor (fout, fg);
 
@@ -1787,6 +1867,8 @@ int                 pattern;
     return;
    y += FrameTable[frame].FrTopMargin;
    /* Do we need to change the current color ? */
+   if (fg < 0)
+     thick = 0;
    if (thick > 0)
       CurrentColor (fout, fg);
 
@@ -1806,7 +1888,7 @@ int                 pattern;
    else
      {
 	/* draw an ellipse */
-	fprintf (fout, "%d %d %d %d %d %d ellipse\n", style, thick, xm, -ym, width, height);
+	fprintf (fout, "%d %d %d -%d %d %d ellipse\n", style, thick, xm, ym, width, height);
      }
    px7mm = 7 * 72 / 25.4 + 0.5;
    if (height > px7mm)
@@ -1821,7 +1903,7 @@ int                 pattern;
 
 
 /*----------------------------------------------------------------------
-   DrawHorizontalLine draw a vertical line aligned top center or bottom
+   DrawHorizontalLine draw a horizontal line aligned top center or bottom
    depending on align value.
   ----------------------------------------------------------------------*/
 #ifdef __STDC__
@@ -1847,7 +1929,7 @@ int                 fg;
    if (y < 0)
      return;
    y += FrameTable[frame].FrTopMargin;
-   if (thick <= 0)
+   if (thick <= 0 || fg < 0)
      return;
 
    fout = (FILE *) FrRef[frame];
@@ -1868,6 +1950,70 @@ int                 fg;
 }
 
 
+/*----------------------------------------------------------------------
+   DrawHorizontalBrace draw a horizontal brace aligned top or bottom
+   depending on align value.
+  ----------------------------------------------------------------------*/
+#ifdef __STDC__
+void                DrawHorizontalBrace (int frame, int thick, int style, int x, int y, int l, int h, int align, int RO, int func, int fg)
+#else  /* __STDC__ */
+void                DrawHorizontalBrace (frame, thick, style, x, y, l, h, align, RO, func, fg)
+int                 frame;
+int                 thick;
+int                 style;
+int                 x;
+int                 y;
+int                 l;
+int                 h;
+int                 align;
+int                 RO;
+int                 func;
+int                 fg;
+#endif /* __STDC__ */
+{
+  int                 xf, xm, Y;
+  FILE               *fout;
+
+  if (y < 0)
+    return;
+  y += FrameTable[frame].FrTopMargin;
+  if (thick <= 0 || fg < 0)
+    return;
+
+  fout = (FILE *) FrRef[frame];
+  /* Do we need to change the current color ? */
+  CurrentColor (fout, fg);
+  Y = PixelToPoint (y + (h - thick) / 2);
+  xf = PixelToPoint (x + l);
+  xm = PixelToPoint (x + (l / 2));
+  x = PixelToPoint (x);
+  if (align == 0)
+    /* Over brace */
+    {
+      fprintf (fout, "%d -%d %d -%d %d -%d %d -%d %d %d %d Seg\n",
+	       x, PixelToPoint (y + h),
+	       x, Y,
+	       xf, Y,
+	       xf, PixelToPoint (y + h), style, thick, 4);
+      fprintf (fout, "%d -%d %d -%d %d %d %d Seg\n",
+	       xm, PixelToPoint (y),
+	       xm, Y, style, thick, 2);
+    }
+  else
+    /* Underbrace */
+    {
+      fprintf (fout, "%d -%d %d -%d %d -%d %d -%d %d %d %d Seg\n",
+	       x, PixelToPoint (y),
+	       x, Y,
+	       xf, Y,
+	       xf, PixelToPoint (y), style, thick, 4);
+      fprintf (fout, "%d -%d %d -%d %d %d %d Seg\n",
+		xm, PixelToPoint (y + h),
+	       xm, Y, style, thick, 2);
+    }
+}
+
+
 /*----------------------------------------------------------------------
    DrawVerticalLine draw a vertical line aligned left center or right
    depending on align value.
@@ -1896,7 +2042,55 @@ int                 fg;
       return;
 
    y += FrameTable[frame].FrTopMargin;
-   if (thick <= 0)
+   if (thick <= 0 || fg < 0)
+      return;
+
+   fout = (FILE *) FrRef[frame];
+   /* Do we need to change the current color ? */
+   CurrentColor (fout, fg);
+   if (align == 1)
+      X = x + (l - thick) / 2;
+   else if (align == 2)
+      X = x + l - thick / 2;
+   else
+      X = x + thick / 2;
+   yf = PixelToPoint (y + h);
+   y = PixelToPoint (y);
+   X = PixelToPoint (X);
+   thick = PixelToPoint (thick);
+      fprintf (fout, "%d -%d %d -%d %d %d %d Seg\n", X, yf, X, y, style, thick, 2);
+}
+
+
+/*----------------------------------------------------------------------
+   DrawDoubleVerticalLine draw a doubled vertical line aligned left center
+   or right depending on align value.
+  ----------------------------------------------------------------------*/
+#ifdef __STDC__
+void                DrawDoubleVerticalLine (int frame, int thick, int style, int x, int y, int l, int h, int align, int RO, int func, int fg)
+#else  /* __STDC__ */
+void                DrawDoubleVerticalLine (frame, thick, style, x, y, l, h, align, RO, func, fg)
+int                 frame;
+int                 thick;
+int                 style;
+int                 x;
+int                 y;
+int                 l;
+int                 h;
+int                 align;
+int                 RO;
+int                 func;
+int                 fg;
+#endif /* __STDC__ */
+{
+   int                 X, yf;
+   FILE               *fout;
+
+   if (y < 0)
+      return;
+
+   y += FrameTable[frame].FrTopMargin;
+   if (thick <= 0 || fg < 0)
       return;
 
    fout = (FILE *) FrRef[frame];
@@ -1978,6 +2172,8 @@ int                 fg;
   if (y < 0)
     return;
    y += FrameTable[frame].FrTopMargin;
+   if (fg < 0)
+     thick = 0;
    if (thick <= 0)
       return;
 
